@@ -18,15 +18,13 @@ const TEST_URI = `
   </div>
 `;
 
-add_task(async function() {
+add_task(async function () {
   await addTab("data:text/html;charset=utf-8," + encodeURIComponent(TEST_URI));
   const { inspector, view } = await openRuleView();
   const highlighters = view.highlighters;
   const HIGHLIGHTER_TYPE = inspector.highlighters.TYPES.GRID;
-  const {
-    waitForHighlighterTypeShown,
-    waitForHighlighterTypeHidden,
-  } = getHighlighterTestHelpers(inspector);
+  const { waitForHighlighterTypeShown, waitForHighlighterTypeHidden } =
+    getHighlighterTestHelpers(inspector);
 
   await selectNode("#grid", inspector);
   const container = getRuleViewProperty(view, "#grid", "display").valueSpan;
@@ -37,8 +35,9 @@ add_task(async function() {
     !gridToggle.hasAttribute("disabled"),
     "Grid highlighter toggle is not disabled."
   );
-  ok(
-    !gridToggle.classList.contains("active"),
+  is(
+    gridToggle.getAttribute("aria-pressed"),
+    "false",
     "Grid highlighter toggle button is not active."
   );
   ok(!highlighters.gridHighlighters.size, "No CSS grid highlighter is shown.");
@@ -56,8 +55,9 @@ add_task(async function() {
     !gridToggle.hasAttribute("disabled"),
     "Grid highlighter toggle is not disabled."
   );
-  ok(
-    gridToggle.classList.contains("active"),
+  is(
+    gridToggle.getAttribute("aria-pressed"),
+    "true",
     "Grid highlighter toggle is active."
   );
   is(highlighters.gridHighlighters.size, 1, "CSS grid highlighter is shown.");
@@ -75,8 +75,9 @@ add_task(async function() {
     !gridToggle.hasAttribute("disabled"),
     "Grid highlighter toggle is not disabled."
   );
-  ok(
-    !gridToggle.classList.contains("active"),
+  is(
+    gridToggle.getAttribute("aria-pressed"),
+    "false",
     "Grid highlighter toggle button is not active."
   );
   ok(!highlighters.gridHighlighters.size, "No CSS grid highlighter is shown.");
