@@ -4,42 +4,57 @@
 
 "use strict";
 
-const { l10n } = require("devtools/shared/inspector/css-logic");
-const { PSEUDO_CLASSES } = require("devtools/shared/css/constants");
+const { l10n } = require("resource://devtools/shared/inspector/css-logic.js");
+const {
+  PSEUDO_CLASSES,
+} = require("resource://devtools/shared/css/constants.js");
 const {
   style: { ELEMENT_STYLE },
-} = require("devtools/shared/constants");
-const Rule = require("devtools/client/inspector/rules/models/rule");
+} = require("resource://devtools/shared/constants.js");
+const Rule = require("resource://devtools/client/inspector/rules/models/rule.js");
 const {
   InplaceEditor,
   editableField,
   editableItem,
-} = require("devtools/client/shared/inplace-editor");
-const TextPropertyEditor = require("devtools/client/inspector/rules/views/text-property-editor");
+} = require("resource://devtools/client/shared/inplace-editor.js");
+const TextPropertyEditor = require("resource://devtools/client/inspector/rules/views/text-property-editor.js");
 const {
   createChild,
   blurOnMultipleProperties,
   promiseWarn,
-} = require("devtools/client/inspector/shared/utils");
+} = require("resource://devtools/client/inspector/shared/utils.js");
 const {
   parseNamedDeclarations,
   parsePseudoClassesAndAttributes,
   SELECTOR_ATTRIBUTE,
   SELECTOR_ELEMENT,
   SELECTOR_PSEUDO_CLASS,
-} = require("devtools/shared/css/parsing-utils");
-const promise = require("promise");
-const Services = require("Services");
-const EventEmitter = require("devtools/shared/event-emitter");
-const CssLogic = require("devtools/shared/inspector/css-logic");
+} = require("resource://devtools/shared/css/parsing-utils.js");
+const EventEmitter = require("resource://devtools/shared/event-emitter.js");
+const CssLogic = require("resource://devtools/shared/inspector/css-logic.js");
 
-loader.lazyRequireGetter(this, "Tools", "devtools/client/definitions", true);
+loader.lazyRequireGetter(
+  this,
+  "Tools",
+  "resource://devtools/client/definitions.js",
+  true
+);
 
 const STYLE_INSPECTOR_PROPERTIES =
   "devtools/shared/locales/styleinspector.properties";
-const { LocalizationHelper } = require("devtools/shared/l10n");
+const { LocalizationHelper } = require("resource://devtools/shared/l10n.js");
 const STYLE_INSPECTOR_L10N = new LocalizationHelper(STYLE_INSPECTOR_PROPERTIES);
 
+const COMPONENT_PROPERTIES = "devtools/client/locales/components.properties";
+const COMPONENT_L10N = new LocalizationHelper(COMPONENT_PROPERTIES);
+
+loader.lazyGetter(this, "NEW_PROPERTY_NAME_INPUT_LABEL", function () {
+  return STYLE_INSPECTOR_L10N.getStr("rule.newPropertyName.label");
+});
+
+const INDENT_SIZE = 2;
+const INDENT_STR = " ".repeat(INDENT_SIZE);
+
 /**
  * RuleEditor is responsible for the following:
  *   Owns a Rule object and creates a list of TextPropertyEditors
@@ -82,7 +97,7 @@ function RuleEditor(ruleView, rule) {
 }
 
 RuleEditor.prototype = {
-  destroy: function() {
+  destroy() {
     this.rule.domRule.off("location-changed");
     this.toolbox.off("tool-registered", this._onToolChanged);
     this.toolbox.off("tool-unregistered", this._onToolChanged);
@@ -112,7 +127,7 @@ RuleEditor.prototype = {
     return trait && !this.rule.elementStyle.element.isAnonymous;
   },
 
-  _create: function() {
+  _create() {
     this.element = this.doc.createElement("div");
     this.element.className = "ruleview-rule devtools-monospace";
     this.element.dataset.ruleId = this.rule.domRule.actorID;
@@ -130,20 +145,188 @@ RuleEditor.prototype = {
     });
     this.source.addEventListener("click", this._onSourceClick);
 
-    const sourceLabel = this.doc.createElement("span");
+    // inline style are not visible in the StyleEditor, so don't create an actual link
+    // element for their location.
+    const sourceLabel = this.doc.createElement(
+      this.rule.domRule.type === ELEMENT_STYLE ? "span" : "a"
+    );
     sourceLabel.classList.add("ruleview-rule-source-label");
     this.source.appendChild(sourceLabel);
 
     this.updateSourceLink();
 
+    if (this.rule.domRule.ancestorData.length) {
+      const ancestorsFrag = this.doc.createDocumentFragment();
+      this.rule.domRule.ancestorData.forEach((ancestorData, index) => {
+        const ancestorItem = this.doc.createElement("div");
+        ancestorItem.setAttribute("role", "listitem");
+        ancestorsFrag.append(ancestorItem);
+        ancestorItem.setAttribute("data-ancestor-index", index);
+        ancestorItem.classList.add("ruleview-rule-ancestor");
+        if (ancestorData.type) {
+          ancestorItem.classList.add(ancestorData.type);
+        }
+
+        // Indent each parent selector
+        if (index) {
+          createChild(ancestorItem, "span", {
+            class: "ruleview-rule-indent",
+            textContent: INDENT_STR.repeat(index),
+          });
+        }
+
+        const selectorContainer = createChild(ancestorItem, "span", {
+          class: "ruleview-rule-ancestor-selectorcontainer",
+        });
+
+        if (ancestorData.type == "container") {
+          ancestorItem.classList.add("container-query", "has-tooltip");
+
+          createChild(selectorContainer, "span", {
+            class: "container-query-declaration",
+            textContent: `@container${
+              ancestorData.containerName ? " " + ancestorData.containerName : ""
+            }`,
+          });
+
+          const jumpToNodeButton = createChild(selectorContainer, "button", {
+            class: "open-inspector",
+            title: l10n("rule.containerQuery.selectContainerButton.tooltip"),
+          });
+
+          let containerNodeFront;
+          const getNodeFront = async () => {
+            if (!containerNodeFront) {
+              const res = await this.rule.domRule.getQueryContainerForNode(
+                index,
+                this.rule.inherited ||
+                  this.ruleView.inspector.selection.nodeFront
+              );
+              containerNodeFront = res.node;
+            }
+            return containerNodeFront;
+          };
+
+          jumpToNodeButton.addEventListener("click", async () => {
+            const front = await getNodeFront();
+            if (!front) {
+              return;
+            }
+            this.ruleView.inspector.selection.setNodeFront(front);
+            await this.ruleView.inspector.highlighters.hideHighlighterType(
+              this.ruleView.inspector.highlighters.TYPES.BOXMODEL
+            );
+          });
+
+          ancestorItem.addEventListener("mouseenter", async () => {
+            const front = await getNodeFront();
+            if (!front) {
+              return;
+            }
+
+            await this.ruleView.inspector.highlighters.showHighlighterTypeForNode(
+              this.ruleView.inspector.highlighters.TYPES.BOXMODEL,
+              front
+            );
+          });
+          ancestorItem.addEventListener("mouseleave", async () => {
+            await this.ruleView.inspector.highlighters.hideHighlighterType(
+              this.ruleView.inspector.highlighters.TYPES.BOXMODEL
+            );
+          });
+
+          createChild(selectorContainer, "span", {
+            // Add a space between the container name (or @container if there's no name)
+            // and the query so the title, which is computed from the DOM, displays correctly.
+            textContent: " " + ancestorData.containerQuery,
+          });
+        } else if (ancestorData.type == "layer") {
+          selectorContainer.append(
+            this.doc.createTextNode(
+              `@layer${ancestorData.value ? " " + ancestorData.value : ""}`
+            )
+          );
+        } else if (ancestorData.type == "media") {
+          selectorContainer.append(
+            this.doc.createTextNode(`@media ${ancestorData.value}`)
+          );
+        } else if (ancestorData.type == "supports") {
+          selectorContainer.append(
+            this.doc.createTextNode(`@supports ${ancestorData.conditionText}`)
+          );
+        } else if (ancestorData.type == "import") {
+          selectorContainer.append(
+            this.doc.createTextNode(`@import ${ancestorData.value}`)
+          );
+        } else if (ancestorData.type == "scope") {
+          let text = `@scope`;
+          if (ancestorData.start) {
+            text += ` (${ancestorData.start})`;
+
+            if (ancestorData.end) {
+              text += ` to (${ancestorData.end})`;
+            }
+          }
+          selectorContainer.append(this.doc.createTextNode(text));
+        } else if (ancestorData.type == "starting-style") {
+          selectorContainer.append(this.doc.createTextNode(`@starting-style`));
+        } else if (ancestorData.selectors) {
+          ancestorData.selectors.forEach((selector, i) => {
+            if (i !== 0) {
+              createChild(selectorContainer, "span", {
+                class: "ruleview-selector-separator",
+                textContent: ", ",
+              });
+            }
+
+            const selectorEl = createChild(selectorContainer, "span", {
+              class: "ruleview-selector",
+              textContent: selector,
+            });
+
+            const warningsContainer = this._createWarningsElementForSelector(
+              i,
+              ancestorData.selectorWarnings
+            );
+            if (warningsContainer) {
+              selectorEl.append(warningsContainer);
+            }
+          });
+        } else {
+          // We shouldn't get here as `type` should only match to what can be set in
+          // the StyleRuleActor form, but just in case, let's return an empty string.
+          console.warn("Unknown ancestor data type:", ancestorData.type);
+          return;
+        }
+
+        createChild(ancestorItem, "span", {
+          class: "ruleview-ancestor-ruleopen",
+          textContent: " {",
+        });
+      });
+
+      // We can't use a proper "ol" as it will mess with selection copy text,
+      // adding spaces on list item instead of the one we craft (.ruleview-rule-indent)
+      this.ancestorDataEl = createChild(this.element, "div", {
+        class: "ruleview-rule-ancestor-data theme-link",
+        role: "list",
+      });
+      this.ancestorDataEl.append(ancestorsFrag);
+    }
+
     const code = createChild(this.element, "div", {
       class: "ruleview-code",
     });
 
     const header = createChild(code, "div", {});
 
+    createChild(header, "span", {
+      class: "ruleview-rule-indent",
+      textContent: INDENT_STR.repeat(this.rule.domRule.ancestorData.length),
+    });
+
     this.selectorText = createChild(header, "span", {
-      class: "ruleview-selectorcontainer",
+      class: "ruleview-selectors-container",
       tabindex: this.isSelectorEditable ? "0" : "-1",
     });
 
@@ -157,40 +340,39 @@ RuleEditor.prototype = {
         element: this.selectorText,
         done: this._onSelectorDone,
         cssProperties: this.rule.cssProperties,
+        // (Shift+)Tab will move the focus to the previous/next editable field (so property name,
+        // or new property of the previous rule).
+        focusEditableFieldAfterApply: true,
+        focusEditableFieldContainerSelector: ".ruleview-rule",
+        // We don't want Enter to trigger the next editable field, just to validate
+        // what the user entered, close the editor, and focus the span so the user can
+        // navigate with the keyboard as expected, unless the user has
+        // devtools.inspector.rule-view.focusNextOnEnter set to true
+        stopOnReturn: this.ruleView.inplaceEditorFocusNextOnEnter !== true,
       });
     }
 
     if (this.rule.domRule.type !== CSSRule.KEYFRAME_RULE) {
-      // FIXME: Avoid having this as a nested async operation. (Bug 1664511)
-      (async function() {
-        let selector;
-
-        if (this.rule.domRule.selectors) {
-          // This is a "normal" rule with a selector.
-          selector = this.rule.domRule.selectors.join(", ");
-        } else if (this.rule.inherited) {
-          // This is an inline style from an inherited rule. Need to resolve the unique
-          // selector from the node which rule this is inherited from.
-          selector = await this.rule.inherited.getUniqueSelector();
-        } else {
-          // This is an inline style from the current node.
-          selector = this.ruleView.inspector.selectionCssSelector;
-        }
-
-        const isHighlighted = this.ruleView.isSelectorHighlighted(selector);
-        // Handling of click events is delegated to CssRuleView.handleEvent()
-        createChild(header, "span", {
-          class:
-            "ruleview-selectorhighlighter js-toggle-selector-highlighter" +
-            (isHighlighted ? " highlighted" : ""),
-          "data-selector": selector,
-          title: l10n("rule.selectorHighlighter.tooltip"),
-        });
+      // This is a "normal" rule with a selector.
+      let computedSelector = "";
+      if (this.rule.domRule.selectors) {
+        computedSelector = this.rule.domRule.computedSelector;
+        // Otherwise, the rule is either inherited or inline, and selectors will
+        // be computed on demand when the highlighter is requested.
       }
-        .bind(this)()
-        .catch(error => {
-          console.error("Exception while getting unique selector", error);
-        }));
+
+      const isHighlighted =
+        this.ruleView.isSelectorHighlighted(computedSelector);
+      // Handling of click events is delegated to CssRuleView.handleEvent()
+      createChild(header, "button", {
+        class:
+          "ruleview-selectorhighlighter js-toggle-selector-highlighter" +
+          (isHighlighted ? " highlighted" : ""),
+        "aria-pressed": isHighlighted,
+        // This is used in rules.js for the selector highlighter
+        "data-computed-selector": computedSelector,
+        title: l10n("rule.selectorHighlighter.tooltip"),
+      });
     }
 
     this.openBrace = createChild(header, "span", {
@@ -198,8 +380,11 @@ RuleEditor.prototype = {
       textContent: " {",
     });
 
-    this.propertyList = createChild(code, "ul", {
+    // We can't use a proper "ol" as it will mess with selection copy text,
+    // adding spaces on list item instead of the one we craft (.ruleview-rule-indent)
+    this.propertyList = createChild(code, "div", {
       class: "ruleview-propertylist",
+      role: "list",
     });
 
     this.populate();
@@ -207,9 +392,30 @@ RuleEditor.prototype = {
     this.closeBrace = createChild(code, "div", {
       class: "ruleview-ruleclose",
       tabindex: this.isEditable ? "0" : "-1",
-      textContent: "}",
     });
 
+    if (this.rule.domRule.ancestorData.length) {
+      createChild(this.closeBrace, "span", {
+        class: "ruleview-rule-indent",
+        textContent: INDENT_STR.repeat(this.rule.domRule.ancestorData.length),
+      });
+    }
+    this.closeBrace.append(this.doc.createTextNode("}"));
+
+    if (this.rule.domRule.ancestorData.length) {
+      let closingBracketsText = "";
+      for (let i = this.rule.domRule.ancestorData.length - 1; i >= 0; i--) {
+        if (i) {
+          closingBracketsText += INDENT_STR.repeat(i);
+        }
+        closingBracketsText += "}\n";
+      }
+      createChild(code, "div", {
+        class: "ruleview-ancestor-ruleclose",
+        textContent: closingBracketsText,
+      });
+    }
+
     if (this.isEditable) {
       // A newProperty editor should only be created when no editor was
       // previously displayed. Since the editors are cleared on blur,
@@ -220,7 +426,7 @@ RuleEditor.prototype = {
         this._ruleViewIsEditing = this.ruleView.isEditing;
       });
 
-      code.addEventListener("click", event => {
+      code.addEventListener("click", () => {
         const selection = this.doc.defaultView.getSelection();
         if (selection.isCollapsed && !this._ruleViewIsEditing) {
           this.newProperty();
@@ -240,10 +446,56 @@ RuleEditor.prototype = {
     }
   },
 
+  /**
+   * Returns the selector warnings element, or null if selector at selectorIndex
+   * does not have any warning.
+   *
+   * @param {Integer} selectorIndex: The index of the selector we want to create the
+   *        warnings for
+   * @param {Array<Object>} selectorWarnings: An array of object of the following shape:
+   *        - {Integer} index: The index of the selector this applies to
+   *        - {String} kind: Identifies the warning
+   * @returns {Element|null}
+   */
+  _createWarningsElementForSelector(selectorIndex, selectorWarnings) {
+    if (!selectorWarnings) {
+      return null;
+    }
+
+    const warningKinds = [];
+    for (const { index, kind } of selectorWarnings) {
+      if (index !== selectorIndex) {
+        continue;
+      }
+      warningKinds.push(kind);
+    }
+
+    if (!warningKinds.length) {
+      return null;
+    }
+
+    const warningsContainer = this.doc.createElement("div");
+    warningsContainer.classList.add(
+      "ruleview-selector-warnings",
+      "has-tooltip"
+    );
+
+    warningsContainer.setAttribute(
+      "data-selector-warning-kind",
+      warningKinds.join(",")
+    );
+
+    if (warningKinds.includes("UnconstrainedHas")) {
+      warningsContainer.classList.add("slow");
+    }
+
+    return warningsContainer;
+  },
+
   /**
    * Called when a tool is registered or unregistered.
    */
-  _onToolChanged: function() {
+  _onToolChanged() {
     // When the source editor is registered, update the source links
     // to be clickable; and if it is unregistered, update the links to
     // be unclickable.  However, some links are never clickable, so
@@ -261,19 +513,19 @@ RuleEditor.prototype = {
    * Event handler called when a property changes on the
    * StyleRuleActor.
    */
-  _locationChanged: function() {
+  _locationChanged() {
     this.updateSourceLink();
   },
 
-  _onSourceClick: function() {
+  _onSourceClick(e) {
+    e.preventDefault();
     if (this.source.hasAttribute("unselectable")) {
       return;
     }
 
     const { inspector } = this.ruleView;
-    const target = inspector.currentTarget;
-    if (Tools.styleEditor.isTargetSupported(target)) {
-      inspector.toolbox.viewSourceInStyleEditorByFront(
+    if (Tools.styleEditor.isToolSupported(inspector.toolbox)) {
+      inspector.toolbox.viewSourceInStyleEditorByResource(
         this.rule.sheet,
         this.rule.ruleLine,
         this.rule.ruleColumn
@@ -289,52 +541,46 @@ RuleEditor.prototype = {
    * @param {Object | null} originalLocation
    *        The original position object (url/line/column) or null.
    */
-  _updateLocation: function(originalLocation) {
-    let displayURL = this.rule.sheet ? this.rule.sheet.href : null;
+  _updateLocation(originalLocation) {
+    let displayURL = this.rule.sheet?.href;
+    const constructed = this.rule.sheet?.constructed;
     let line = this.rule.ruleLine;
     if (originalLocation) {
       displayURL = originalLocation.url;
       line = originalLocation.line;
     }
 
-    let sourceTextContent = CssLogic.shortSource({ href: displayURL });
-    let title = displayURL ? displayURL : sourceTextContent;
+    let sourceTextContent = CssLogic.shortSource({
+      constructed,
+      href: displayURL,
+    });
+
+    let displayLocation = displayURL ? displayURL : sourceTextContent;
     if (line > 0) {
       sourceTextContent += ":" + line;
-      title += ":" + line;
-    }
-    if (this.rule.mediaText) {
-      sourceTextContent += " @" + this.rule.mediaText;
-      title += " @" + this.rule.mediaText;
+      displayLocation += ":" + line;
     }
+    const title = COMPONENT_L10N.getFormatStr(
+      "frame.viewsourceinstyleeditor",
+      displayLocation
+    );
 
     const sourceLabel = this.element.querySelector(
       ".ruleview-rule-source-label"
     );
     sourceLabel.setAttribute("title", title);
+    sourceLabel.setAttribute("href", displayURL);
     sourceLabel.textContent = sourceTextContent;
   },
 
-  updateSourceLink: function() {
+  updateSourceLink() {
     if (this.rule.isSystem) {
       const sourceLabel = this.element.querySelector(
         ".ruleview-rule-source-label"
       );
-      const title = this.rule.title;
-      const sourceHref =
-        this.rule.sheet && this.rule.sheet.href ? this.rule.sheet.href : title;
-
       const uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
-      sourceLabel.textContent = uaLabel + " " + title;
-
-      // Special case about:PreferenceStyleSheet, as it is generated on the
-      // fly and the URI is not registered with the about: handler.
-      // https://bugzilla.mozilla.org/show_bug.cgi?id=935803#c37
-      if (sourceHref === "about:PreferenceStyleSheet") {
-        this.source.setAttribute("unselectable", "permanent");
-        sourceLabel.textContent = uaLabel;
-        sourceLabel.removeAttribute("title");
-      }
+      sourceLabel.textContent = uaLabel + " " + this.rule.title;
+      sourceLabel.setAttribute("href", this.rule.sheet?.href);
     } else {
       this._updateLocation(null);
     }
@@ -350,7 +596,7 @@ RuleEditor.prototype = {
         this._unsubscribeSourceMap();
       }
       this._unsubscribeSourceMap = this.sourceMapURLService.subscribeByID(
-        this.rule.sheet.resourceId || this.rule.sheet.actorID,
+        this.rule.sheet.resourceId,
         this.rule.ruleLine,
         this.rule.ruleColumn,
         this._updateLocation
@@ -364,7 +610,7 @@ RuleEditor.prototype = {
       this._onToolChanged();
     }
 
-    promise.resolve().then(() => {
+    Promise.resolve().then(() => {
       this.emit("source-link-updated");
     });
   },
@@ -375,7 +621,7 @@ RuleEditor.prototype = {
    * @param {Boolean} reset
    *        True to completely reset the rule editor before populating.
    */
-  populate: function(reset) {
+  populate(reset) {
     // Clear out existing viewers.
     while (this.selectorText.hasChildNodes()) {
       this.selectorText.removeChild(this.selectorText.lastChild);
@@ -390,53 +636,19 @@ RuleEditor.prototype = {
       this.selectorText.textContent = this.rule.domRule.keyText;
     } else {
       this.rule.domRule.selectors.forEach((selector, i) => {
-        if (i !== 0) {
-          createChild(this.selectorText, "span", {
-            class: "ruleview-selector-separator",
-            textContent: ", ",
-          });
-        }
-
-        const containerClass =
-          this.rule.matchedSelectors.indexOf(selector) > -1
-            ? "ruleview-selector-matched"
-            : "ruleview-selector-unmatched";
-        const selectorContainer = createChild(this.selectorText, "span", {
-          class: containerClass,
-        });
-
-        const parsedSelector = parsePseudoClassesAndAttributes(selector);
-
-        for (const selectorText of parsedSelector) {
-          let selectorClass = "";
-
-          switch (selectorText.type) {
-            case SELECTOR_ATTRIBUTE:
-              selectorClass = "ruleview-selector-attribute";
-              break;
-            case SELECTOR_ELEMENT:
-              selectorClass = "ruleview-selector";
-              break;
-            case SELECTOR_PSEUDO_CLASS:
-              selectorClass = PSEUDO_CLASSES.some(
-                pseudo => selectorText.value === pseudo
-              )
-                ? "ruleview-selector-pseudo-class-lock"
-                : "ruleview-selector-pseudo-class";
-              break;
-            default:
-              break;
-          }
-
-          createChild(selectorContainer, "span", {
-            textContent: selectorText.value,
-            class: selectorClass,
-          });
-        }
+        this._populateSelector(selector, i);
       });
     }
 
+    let focusedElSelector;
     if (reset) {
+      // If we're going to reset the rule (i.e. if this is the `element` rule),
+      // we want to restore the focus after the rule is populated.
+      // So if this element contains the active element, retrieve its selector for later use.
+      if (this.element.contains(this.doc.activeElement)) {
+        focusedElSelector = CssLogic.findCssSelector(this.doc.activeElement);
+      }
+
       while (this.propertyList.hasChildNodes()) {
         this.propertyList.removeChild(this.propertyList.lastChild);
       }
@@ -452,6 +664,102 @@ RuleEditor.prototype = {
         this.propertyList.appendChild(prop.editor.element);
       }
     }
+
+    if (focusedElSelector) {
+      const elementToFocus = this.doc.querySelector(focusedElSelector);
+      if (elementToFocus && this.element.contains(elementToFocus)) {
+        // We need to wait for a tick for the focus to be properly set
+        setTimeout(() => {
+          elementToFocus.focus();
+          this.ruleView.emitForTests("rule-editor-focus-reset");
+        }, 0);
+      }
+    }
+  },
+
+  /**
+   * Render a given rule selector in this.selectorText element
+   *
+   * @param {String} selector: The selector text to display
+   * @param {Number} selectorIndex: Its index in the rule
+   */
+  _populateSelector(selector, selectorIndex) {
+    if (selectorIndex !== 0) {
+      createChild(this.selectorText, "span", {
+        class: "ruleview-selector-separator",
+        textContent: ", ",
+      });
+    }
+
+    let containerClass = "ruleview-selector ";
+
+    // Only add matched/unmatched class when the rule does have some matched
+    // selectors. We don't always have some (e.g. rules for pseudo elements)
+
+    if (this.rule.matchedSelectorIndexes.length) {
+      containerClass += this.rule.matchedSelectorIndexes.includes(selectorIndex)
+        ? "matched"
+        : "unmatched";
+    }
+
+    let selectorContainerTitle;
+    if (
+      typeof this.rule.selector.selectorsSpecificity?.[selectorIndex] !==
+      "undefined"
+    ) {
+      // The specificity that we get from the platform is a single number that we
+      // need to format into the common `(x,y,z)` specificity string.
+      const specificity =
+        this.rule.selector.selectorsSpecificity?.[selectorIndex];
+      const a = Math.floor(specificity / (1024 * 1024));
+      const b = Math.floor((specificity % (1024 * 1024)) / 1024);
+      const c = specificity % 1024;
+      selectorContainerTitle = STYLE_INSPECTOR_L10N.getFormatStr(
+        "rule.selectorSpecificity.title",
+        `(${a},${b},${c})`
+      );
+    }
+    const selectorContainer = createChild(this.selectorText, "span", {
+      class: containerClass,
+      title: selectorContainerTitle,
+    });
+
+    const parsedSelector = parsePseudoClassesAndAttributes(selector);
+
+    for (const selectorText of parsedSelector) {
+      let selectorClass = "";
+
+      switch (selectorText.type) {
+        case SELECTOR_ATTRIBUTE:
+          selectorClass = "ruleview-selector-attribute";
+          break;
+        case SELECTOR_ELEMENT:
+          selectorClass = "ruleview-selector-element";
+          break;
+        case SELECTOR_PSEUDO_CLASS:
+          selectorClass = PSEUDO_CLASSES.some(
+            pseudo => selectorText.value === pseudo
+          )
+            ? "ruleview-selector-pseudo-class-lock"
+            : "ruleview-selector-pseudo-class";
+          break;
+        default:
+          break;
+      }
+
+      createChild(selectorContainer, "span", {
+        textContent: selectorText.value,
+        class: selectorClass,
+      });
+    }
+
+    const warningsContainer = this._createWarningsElementForSelector(
+      selectorIndex,
+      this.rule.domRule.selectorWarnings
+    );
+    if (warningsContainer) {
+      selectorContainer.append(warningsContainer);
+    }
   },
 
   /**
@@ -470,7 +778,7 @@ RuleEditor.prototype = {
    * @return {TextProperty}
    *        The new property
    */
-  addProperty: function(name, value, priority, enabled, siblingProp) {
+  addProperty(name, value, priority, enabled, siblingProp) {
     const prop = this.rule.createProperty(
       name,
       value,
@@ -509,7 +817,7 @@ RuleEditor.prototype = {
    * @param {TextProperty} siblingProp
    *        Optional, the property next to which all new props should be added.
    */
-  addProperties: function(properties, siblingProp) {
+  addProperties(properties, siblingProp) {
     if (!properties || !properties.length) {
       return;
     }
@@ -540,7 +848,7 @@ RuleEditor.prototype = {
    * name is given, we'll create a real TextProperty and add it to the
    * rule.
    */
-  newProperty: function() {
+  newProperty() {
     // If we're already creating a new property, ignore this.
     if (!this.closeBrace.hasAttribute("tabindex")) {
       return;
@@ -551,8 +859,9 @@ RuleEditor.prototype = {
     // close brace for now.
     this.closeBrace.removeAttribute("tabindex");
 
-    this.newPropItem = createChild(this.propertyList, "li", {
+    this.newPropItem = createChild(this.propertyList, "div", {
       class: "ruleview-property ruleview-newproperty",
+      role: "listitem",
     });
 
     this.newPropSpan = createChild(this.newPropItem, "span", {
@@ -565,11 +874,17 @@ RuleEditor.prototype = {
     this.editor = new InplaceEditor({
       element: this.newPropSpan,
       done: this._onNewProperty,
+      // (Shift+)Tab will move the focus to the previous/next editable field
+      focusEditableFieldAfterApply: true,
+      focusEditableFieldContainerSelector: ".ruleview-rule",
       destroy: this._newPropertyDestroy,
       advanceChars: ":",
       contentType: InplaceEditor.CONTENT_TYPES.CSS_PROPERTY,
       popup: this.ruleView.popup,
       cssProperties: this.rule.cssProperties,
+      inputAriaLabel: NEW_PROPERTY_NAME_INPUT_LABEL,
+      getCssVariables: () =>
+        this.rule.elementStyle.getAllCustomProperties(this.rule.pseudoElement),
     });
 
     // Auto-close the input if multiple rules get pasted into new property.
@@ -587,7 +902,7 @@ RuleEditor.prototype = {
    * @param {Boolean} commit
    *        True if the value should be committed.
    */
-  _onNewProperty: function(value, commit) {
+  _onNewProperty(value, commit) {
     if (!value || !commit) {
       return;
     }
@@ -605,9 +920,7 @@ RuleEditor.prototype = {
     // the field gets destroyed (see _newPropertyDestroy)
     this.editor.input.blur();
 
-    this.telemetry.recordEvent("edit_rule", "ruleview", null, {
-      session_id: this.toolbox.sessionId,
-    });
+    this.telemetry.recordEvent("edit_rule", "ruleview");
   },
 
   /**
@@ -616,7 +929,7 @@ RuleEditor.prototype = {
    * added, since we want to wait until after the inplace editor `destroy`
    * event has been fired to keep consistent UI state.
    */
-  _newPropertyDestroy: function() {
+  _newPropertyDestroy() {
     // We're done, make the close brace focusable again.
     this.closeBrace.setAttribute("tabindex", "0");
 
@@ -661,8 +974,12 @@ RuleEditor.prototype = {
     this.isEditing = true;
 
     // Remove highlighter for the previous selector.
-    if (this.ruleView.isSelectorHighlighted(this.rule.selectorText)) {
-      await this.ruleView.toggleSelectorHighlighter(this.rule.selectorText);
+    const computedSelector = this.rule.domRule.computedSelector;
+    if (this.ruleView.isSelectorHighlighted(computedSelector)) {
+      await this.ruleView.toggleSelectorHighlighter(
+        this.rule,
+        computedSelector
+      );
     }
 
     try {
@@ -712,6 +1029,11 @@ RuleEditor.prototype = {
       // pseudo-element rules and the like.
       this.element.parentNode.replaceChild(editor.element, this.element);
 
+      // As the rules elements will be replaced, and given that the inplace-editor doesn't
+      // wait for this `done` callback to be resolved, the focus management we do there
+      // will be useless as this specific code will usually happen later (and the focused
+      // element might be replaced).
+      // Because of this, we need to handle setting the focus ourselves from here.
       editor._moveSelectorFocus(direction);
     } catch (err) {
       this.isEditing = false;
@@ -720,18 +1042,17 @@ RuleEditor.prototype = {
   },
 
   /**
-   * Handle moving the focus change after a tab or return keypress in the
-   * selector inplace editor.
+   * Handle moving the focus change after a Tab keypress in the selector inplace editor.
    *
    * @param {Number} direction
    *        The move focus direction number.
    */
-  _moveSelectorFocus: function(direction) {
+  _moveSelectorFocus(direction) {
     if (!direction || direction === Services.focus.MOVEFOCUS_BACKWARD) {
       return;
     }
 
-    if (this.rule.textProps.length > 0) {
+    if (this.rule.textProps.length) {
       this.rule.textProps[0].editor.nameSpan.click();
     } else {
       this.propertyList.click();
