@@ -4,8 +4,10 @@
 
 "use strict";
 
-const { AutoRefreshHighlighter } = require("./auto-refresh");
-const { apply } = require("devtools/shared/layout/dom-matrix-2d");
+const {
+  AutoRefreshHighlighter,
+} = require("resource://devtools/server/actors/highlighters/auto-refresh.js");
+const { apply } = require("resource://devtools/shared/layout/dom-matrix-2d.js");
 const {
   CANVAS_SIZE,
   DEFAULT_COLOR,
@@ -15,12 +17,11 @@ const {
   getCurrentMatrix,
   updateCanvasElement,
   updateCanvasPosition,
-} = require("./utils/canvas");
+} = require("resource://devtools/server/actors/highlighters/utils/canvas.js");
 const {
   CanvasFrameAnonymousContentHelper,
-  createNode,
   getComputedStyle,
-} = require("./utils/markup");
+} = require("resource://devtools/server/actors/highlighters/utils/markup.js");
 const {
   getAbsoluteScrollOffsetsForNode,
   getCurrentZoom,
@@ -28,7 +29,7 @@ const {
   getUntransformedQuad,
   getWindowDimensions,
   setIgnoreLayoutChanges,
-} = require("devtools/shared/layout/utils");
+} = require("resource://devtools/shared/layout/utils.js");
 
 const FLEXBOX_LINES_PROPERTIES = {
   edge: {
@@ -84,6 +85,7 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
       this.highlighterEnv,
       this._buildMarkup.bind(this)
     );
+    this.isReady = this.markup.initialize();
 
     this.onPageHide = this.onPageHide.bind(this);
     this.onWillNavigate = this.onWillNavigate.bind(this);
@@ -112,13 +114,13 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
   }
 
   _buildMarkup() {
-    const container = createNode(this.win, {
+    const container = this.markup.createNode({
       attributes: {
         class: "highlighter-container",
       },
     });
 
-    const root = createNode(this.win, {
+    const root = this.markup.createNode({
       parent: container,
       attributes: {
         id: "root",
@@ -130,7 +132,7 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
     // We use a <canvas> element because there is an arbitrary number of items and texts
     // to draw which wouldn't be possible with HTML or SVG without having to insert and
     // remove the whole markup on every update.
-    createNode(this.win, {
+    this.markup.createNode({
       parent: root,
       nodeType: "canvas",
       attributes: {
@@ -225,7 +227,7 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
     }
 
     // Create the diagonal lines pattern for the rendering the flexbox gaps.
-    const canvas = createNode(this.win, { nodeType: "canvas" });
+    const canvas = this.markup.createNode({ nodeType: "canvas" });
     const width = (canvas.width =
       FLEXBOX_CONTAINER_PATTERN_WIDTH * devicePixelRatio);
     const height = (canvas.height =
@@ -273,7 +275,7 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
 
     // Create the inversed diagonal lines pattern
     // for the rendering the justify content gaps.
-    const canvas = createNode(this.win, { nodeType: "canvas" });
+    const canvas = this.markup.createNode({ nodeType: "canvas" });
     const zoom = getCurrentZoom(this.win);
     const width = (canvas.width =
       FLEXBOX_JUSTIFY_CONTENT_PATTERN_WIDTH * devicePixelRatio * zoom);
@@ -300,6 +302,10 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
     return pattern;
   }
 
+  getNode(id) {
+    return this.markup.content.root.getElementById(this.ID_CLASS_PREFIX + id);
+  }
+
   /**
    * The AutoRefreshHighlighter's _hasMoved method returns true only if the
    * element's quads have changed. Override it so it also returns true if the
@@ -495,10 +501,8 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
 
     const lineWidth = getDisplayPixelRatio(this.win);
     const options = { matrix: this.currentMatrix };
-    const {
-      width: containerWidth,
-      height: containerHeight,
-    } = getUntransformedQuad(this.container, "content").getBounds();
+    const { width: containerWidth, height: containerHeight } =
+      getUntransformedQuad(this.container, "content").getBounds();
 
     this.setupCanvas({
       useContainerScrollOffsets: true,
@@ -630,6 +634,7 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
   /**
    * Clear the whole alignment container along the main axis for each flex item.
    */
+  // eslint-disable-next-line complexity
   renderJustifyContent() {
     if (
       !this.flexData ||
@@ -639,10 +644,8 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
       return;
     }
 
-    const {
-      width: containerWidth,
-      height: containerHeight,
-    } = getUntransformedQuad(this.container, "content").getBounds();
+    const { width: containerWidth, height: containerHeight } =
+      getUntransformedQuad(this.container, "content").getBounds();
 
     this.setupCanvas({
       lineDash: FLEXBOX_LINES_PROPERTIES.alignItems.lineDash,
@@ -830,12 +833,6 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
   _update() {
     setIgnoreLayoutChanges(true);
 
-    const root = this.getElement("root");
-
-    // Hide the root element and force the reflow in order to get the proper window's
-    // dimensions without increasing them.
-    root.setAttribute("style", "display: none");
-    this.win.document.documentElement.offsetWidth;
     this._winDimensions = getWindowDimensions(this.win);
     const { width, height } = this._winDimensions;
 
@@ -861,17 +858,19 @@ class FlexboxHighlighter extends AutoRefreshHighlighter {
     this.currentMatrix = currentMatrix;
     this.hasNodeTransformations = hasNodeTransformations;
 
+    if (this.prevColor != this.color) {
+      this.clearCache();
+    }
     this.renderFlexContainer();
     this.renderFlexLines();
     this.renderJustifyContent();
     this.renderFlexItems();
-
     this._showFlexbox();
+    this.prevColor = this.color;
 
-    root.setAttribute(
-      "style",
-      `position: absolute; width: ${width}px; height: ${height}px; overflow: hidden`
-    );
+    const root = this.getNode("root");
+    root.style.setProperty("width", `${width}px`);
+    root.style.setProperty("height", `${height}px`);
 
     setIgnoreLayoutChanges(false, this.highlighterEnv.document.documentElement);
     return true;
@@ -943,10 +942,7 @@ function getRectFromFlexItemValues(item, container) {
   domRect.x -= paddingLeft + scrollX;
   domRect.y -= paddingTop + scrollY;
 
-  if (
-    style.overflow === "visible" ||
-    style.overflow === "-moz-hidden-unscrollable"
-  ) {
+  if (style.overflow === "visible" || style.overflow === "clip") {
     domRect.x -= borderLeftWidth;
     domRect.y -= borderTopWidth;
   }
@@ -963,6 +959,7 @@ function getRectFromFlexItemValues(item, container) {
  *         The new Flex data object.
  * @return {Boolean} true if the flex data has changed and false otherwise.
  */
+// eslint-disable-next-line complexity
 function compareFlexData(oldFlexData, newFlexData) {
   if (!oldFlexData || !newFlexData) {
     return true;
