@@ -26,8 +26,7 @@ internal object TranslationsStateReducer {
     @Suppress("LongMethod")
     fun reduce(state: BrowserState, action: TranslationsAction): BrowserState = when (action) {
         TranslationsAction.InitTranslationsBrowserState -> {
-            // No state change on this operation
-            state
+            state.copy(translationsInitialized = true)
         }
 
         is TranslationsAction.TranslateExpectedAction -> {
@@ -295,6 +294,13 @@ internal object TranslationsStateReducer {
                 )
             }
 
+        is TranslationsAction.SetTranslateProcessingAction ->
+            state.copyWithTranslationsState(action.tabId) {
+                it.copy(
+                    isTranslateProcessing = action.isProcessing,
+                )
+            }
+
         is TranslationsAction.SetNeverTranslateSitesAction ->
             state.copy(
                 translationEngine = state.translationEngine.copy(
@@ -502,6 +508,7 @@ internal object TranslationsStateReducer {
                 ModelState.DELETION_IN_PROGRESS
             }
             val newModelState = LanguageModel.determineNewLanguageModelState(
+                appLanguage = state.locale?.language.toString(),
                 currentLanguageModels = state.translationEngine.languageModels,
                 options = action.options,
                 newStatus = processState,
