@@ -392,7 +392,7 @@ class WebExtensionActionTest {
 
         assertNull(store.state.webExtensionPromptRequest)
 
-        val promptRequest = WebExtensionPromptRequest.AfterInstallation.Permissions.Required(mock(), mock(), mock())
+        val promptRequest = WebExtensionPromptRequest.AfterInstallation.Permissions.Required(mock(), mock(), mock(), mock())
 
         store.dispatch(WebExtensionAction.UpdatePromptRequestWebExtensionAction(promptRequest))
             .joinBlocking()
@@ -402,7 +402,7 @@ class WebExtensionActionTest {
 
     @Test
     fun `WHEN ConsumePromptRequestWebExtensionAction is dispatched THEN the actual WebExtensionPromptRequest is removed from the store`() {
-        val promptRequest = WebExtensionPromptRequest.AfterInstallation.Permissions.Required(mock(), mock(), mock())
+        val promptRequest = WebExtensionPromptRequest.AfterInstallation.Permissions.Required(mock(), mock(), mock(), mock())
 
         val store = BrowserStore(
             initialState = BrowserState(
