@@ -8,13 +8,12 @@ import android.app.PendingIntent
 import android.content.Intent
 import android.graphics.Bitmap
 import android.graphics.Color
+import android.os.Build
 import android.view.ViewGroup
 import android.view.Window
 import android.widget.FrameLayout
 import android.widget.ImageButton
 import android.widget.TextView
-import androidx.appcompat.app.AppCompatDelegate
-import androidx.browser.customtabs.CustomTabsIntent
 import androidx.core.content.ContextCompat.getColor
 import androidx.core.view.forEach
 import androidx.test.ext.junit.runners.AndroidJUnit4
@@ -37,22 +36,26 @@ import mozilla.components.browser.state.state.CustomTabMenuItem
 import mozilla.components.browser.state.state.createCustomTab
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.browser.toolbar.BrowserToolbar
+import mozilla.components.browser.toolbar.display.DisplayToolbar
 import mozilla.components.concept.toolbar.Toolbar
 import mozilla.components.feature.session.SessionUseCases
 import mozilla.components.feature.tabs.CustomTabsUseCases
 import mozilla.components.support.ktx.android.content.res.resolveAttribute
 import mozilla.components.support.test.any
 import mozilla.components.support.test.argumentCaptor
+import mozilla.components.support.test.eq
 import mozilla.components.support.test.ext.joinBlocking
 import mozilla.components.support.test.middleware.CaptureActionsMiddleware
 import mozilla.components.support.test.mock
 import mozilla.components.support.test.robolectric.testContext
+import mozilla.components.support.test.whenever
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertFalse
+import org.junit.Assert.assertNotSame
+import org.junit.Assert.assertSame
 import org.junit.Assert.assertTrue
 import org.junit.Test
 import org.junit.runner.RunWith
-import org.mockito.ArgumentMatchers.anyList
 import org.mockito.Mockito.anyInt
 import org.mockito.Mockito.doNothing
 import org.mockito.Mockito.never
@@ -61,12 +64,19 @@ import org.mockito.Mockito.times
 import org.mockito.Mockito.verify
 import org.mockito.Mockito.`when`
 import org.robolectric.annotation.Config
+
 @RunWith(AndroidJUnit4::class)
 class CustomTabsToolbarFeatureTest {
     @Test
     fun `start without sessionId invokes nothing`() {
         val store = BrowserStore()
         val toolbar: BrowserToolbar = mock()
+        val display: DisplayToolbar = mock()
+        val colors: DisplayToolbar.Colors = mock()
+        whenever(toolbar.display).thenReturn(display)
+        whenever(display.colors).thenReturn(colors)
+        whenever(colors.menu).thenReturn(0)
+
         val useCases = CustomTabsUseCases(
             store = store,
             loadUrlUseCase = SessionUseCases(store).loadUrl,
@@ -126,6 +136,7 @@ class CustomTabsToolbarFeatureTest {
         assertFalse(toolbar.display.onUrlClicked.invoke())
     }
 
+    @Suppress("DEPRECATION")
     @Test
     @Config(sdk = [28])
     fun `initialize updates toolbar, window and text color on SDK 28`() {
@@ -154,6 +165,7 @@ class CustomTabsToolbarFeatureTest {
         )
         val window: Window = mock()
         `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
         val feature = CustomTabsToolbarFeature(store, toolbar, sessionId = "mozilla", useCases = useCases, window = window) {}
 
         feature.init(tab.config)
@@ -166,8 +178,10 @@ class CustomTabsToolbarFeatureTest {
         assertEquals(Color.WHITE, toolbar.display.colors.text)
     }
 
+    @Config(sdk = [Build.VERSION_CODES.UPSIDE_DOWN_CAKE])
+    @Suppress("DEPRECATION")
     @Test
-    fun `initialize updates toolbar, window and text color`() {
+    fun `initialize updates toolbar, window and text color on SDK lower than 35`() {
         val tab = createCustomTab(
             "https://www.mozilla.org",
             id = "mozilla",
@@ -193,6 +207,7 @@ class CustomTabsToolbarFeatureTest {
         )
         val window: Window = mock()
         `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
         `when`(window.insetsController).thenReturn(mock())
 
         val feature = CustomTabsToolbarFeature(store, toolbar, sessionId = "mozilla", useCases = useCases, window = window) {}
@@ -207,8 +222,54 @@ class CustomTabsToolbarFeatureTest {
         assertEquals(Color.WHITE, toolbar.display.colors.text)
     }
 
+    @Suppress("DEPRECATION")
     @Test
-    fun `initialize does not update toolbar background if flag is set`() {
+    fun `initialize updates toolbar, window and text color on SDK 35`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(
+                        toolbarColor = Color.RED,
+                        navigationBarColor = Color.BLUE,
+                    ),
+                ),
+            ),
+        )
+
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = spy(BrowserToolbar(testContext))
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        val feature = CustomTabsToolbarFeature(store, toolbar, sessionId = "mozilla", useCases = useCases, window = window) {}
+
+        feature.init(tab.config)
+
+        verify(window, never()).navigationBarColor = Color.RED
+        verify(window, never()).statusBarColor = Color.RED
+        verify(window, never()).navigationBarColor = Color.BLUE
+
+        assertEquals(Color.WHITE, toolbar.display.colors.title)
+        assertEquals(Color.WHITE, toolbar.display.colors.text)
+
+        assertEquals(window.statusBarColor, Color.TRANSPARENT)
+        assertEquals(window.navigationBarColor, Color.TRANSPARENT)
+    }
+
+    @Test
+    fun `initialize does not update toolbar colors if this functionality is disabled`() {
         val tab = createCustomTab(
             "https://www.mozilla.org",
             id = "mozilla",
@@ -231,7 +292,9 @@ class CustomTabsToolbarFeatureTest {
         )
         val window: Window = mock()
         `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
         `when`(window.insetsController).thenReturn(mock())
+        val initialDisplayToolbarColors = toolbar.display.colors
 
         run {
             val feature = CustomTabsToolbarFeature(
@@ -240,12 +303,15 @@ class CustomTabsToolbarFeatureTest {
                 sessionId = "mozilla",
                 useCases = useCases,
                 window = window,
-                updateTheme = false,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateToolbarsColor = false,
+                ),
             ) {}
 
             feature.init(tab.config)
 
             verify(toolbar, never()).setBackgroundColor(Color.RED)
+            assertSame(initialDisplayToolbarColors, toolbar.display.colors)
         }
 
         run {
@@ -255,12 +321,292 @@ class CustomTabsToolbarFeatureTest {
                 sessionId = "mozilla",
                 useCases = useCases,
                 window = window,
-                updateTheme = true,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateToolbarsColor = true,
+                ),
             ) {}
 
             feature.init(tab.config)
 
             verify(toolbar).setBackgroundColor(Color.RED)
+            assertNotSame(initialDisplayToolbarColors, toolbar.display.colors)
+        }
+    }
+
+    @Config(sdk = [Build.VERSION_CODES.UPSIDE_DOWN_CAKE])
+    @Suppress("DEPRECATION")
+    @Test
+    fun `GIVEN changing the status bar color is enabled WHEN customizing the UI for a custom tab on SDK lower than 35 THEN change the status bar color`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.GREEN),
+                ),
+            ),
+        )
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateStatusBarColor = true,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window).statusBarColor = Color.GREEN
+        }
+    }
+
+    @Suppress("DEPRECATION")
+    @Test
+    fun `GIVEN changing the status bar color is enabled WHEN customizing the UI for a custom tab on SDK 35 THEN the status bar color is not changed`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.GREEN),
+                ),
+            ),
+        )
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateStatusBarColor = true,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window, never()).navigationBarColor = Color.GREEN
+            assertEquals(window.navigationBarColor, Color.TRANSPARENT)
+        }
+    }
+
+    @Suppress("DEPRECATION")
+    @Test
+    fun `GIVEN changing the status bar color is disabled WHEN customizing the UI for a custom tab THEN don't change the status bar color`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.GREEN),
+                ),
+            ),
+        )
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateStatusBarColor = false,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window, never()).statusBarColor = Color.GREEN
+        }
+    }
+
+    @Config(sdk = [Build.VERSION_CODES.UPSIDE_DOWN_CAKE])
+    @Suppress("DEPRECATION")
+    @Test
+    fun `GIVEN changing the system navigation bar color is enabled WHEN customizing the UI for a custom tab on SDK lower than 35 THEN change the system navigation bar color`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.BLUE),
+                ),
+            ),
+        )
+
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateSystemNavigationBarColor = true,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window).navigationBarColor = Color.BLUE
+        }
+    }
+
+    @Test
+    @Suppress("DEPRECATION")
+    fun `GIVEN changing the system navigation bar color is enabled WHEN customizing the UI for a custom tab on SDK 35 THEN change the system navigation bar color is not called`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.BLUE),
+                ),
+            ),
+        )
+
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateSystemNavigationBarColor = true,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window, never()).navigationBarColor = eq(Color.BLUE)
+            assertEquals(window.navigationBarColor, Color.TRANSPARENT)
+        }
+    }
+
+    @Suppress("DEPRECATION")
+    @Test
+    fun `GIVEN changing the system navigation bar color is disabled WHEN customizing the UI for a custom tab THEN don't change the system navigation bar color`() {
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                colorSchemes = ColorSchemes(
+                    defaultColorSchemeParams = ColorSchemeParams(toolbarColor = Color.BLUE),
+                ),
+            ),
+        )
+
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val window: Window = mock()
+        `when`(window.decorView).thenReturn(mock())
+        `when`(window.context).thenReturn(testContext)
+        `when`(window.insetsController).thenReturn(mock())
+
+        run {
+            val feature = CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                window = window,
+                customTabsColorsConfig = CustomTabsColorsConfig(
+                    updateSystemNavigationBarColor = false,
+                ),
+            ) {}
+
+            feature.init(tab.config)
+
+            verify(window, never()).navigationBarColor = Color.BLUE
         }
     }
 
@@ -454,6 +800,75 @@ class CustomTabsToolbarFeatureTest {
         assertTrue(clicked)
     }
 
+    @Test
+    fun `GIVEN the default custom tabs toolbar button config and listeners THEN do not add menu button`() {
+        val tab = createCustomTab("https://www.mozilla.org", id = "mozilla", config = CustomTabConfig())
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = spy(BrowserToolbar(testContext))
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val feature = spy(
+            CustomTabsToolbarFeature(
+                store = store,
+                toolbar = toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+            ) {},
+        )
+
+        feature.start()
+
+        verify(feature, never()).addMenuButton()
+        verify(toolbar, never()).addBrowserAction(any())
+    }
+
+    @Test
+    fun `GIVEN custom tabs toolbar config to show menu with a menu listener THEN show menu button with custom menu listener`() {
+        val tab = createCustomTab("https://www.mozilla.org", id = "mozilla", config = CustomTabConfig())
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = spy(BrowserToolbar(testContext))
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        var clicked = false
+        val feature = spy(
+            CustomTabsToolbarFeature(
+                store = store,
+                toolbar = toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                customTabsToolbarButtonConfig = CustomTabsToolbarButtonConfig(
+                    showMenu = true,
+                ),
+                customTabsToolbarListeners = CustomTabsToolbarListeners(
+                    menuListener = { clicked = true },
+                ),
+            ) {},
+        )
+
+        feature.start()
+
+        verify(feature).addMenuButton()
+
+        val captor = argumentCaptor<Toolbar.ActionButton>()
+        verify(toolbar).addBrowserAction(captor.capture())
+
+        val button = captor.value.createView(FrameLayout(testContext))
+        button.performClick()
+        assertTrue(clicked)
+    }
+
     @Test
     fun `does not add share button by default`() {
         val tab = createCustomTab("https://www.mozilla.org", id = "mozilla", config = CustomTabConfig())
@@ -754,7 +1169,7 @@ class CustomTabsToolbarFeatureTest {
 
         feature.start()
 
-        verify(feature).addMenuItems(anyList(), anyInt())
+        verify(feature).addMenuItems()
     }
 
     @Test
@@ -790,7 +1205,7 @@ class CustomTabsToolbarFeatureTest {
 
         feature.start()
 
-        verify(feature).addMenuItems(anyList(), anyInt())
+        verify(feature).addMenuItems()
     }
 
     @Test
@@ -1272,6 +1687,77 @@ class CustomTabsToolbarFeatureTest {
         assertEquals(Color.WHITE, toolbar.display.colors.menu)
     }
 
+    @Test
+    fun `GIVEN the close button has enabled customization WHEN needing to show the close button THEN use the provided icon`() {
+        val customCloseIcon: Bitmap = mock()
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                closeButtonIcon = customCloseIcon,
+            ),
+        )
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val feature = spy(
+            CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+            ) {},
+        )
+
+        feature.start()
+
+        verify(feature).addCloseButton(toolbar.display.colors.menu, customCloseIcon)
+    }
+
+    @Test
+    fun `GIVEN the close button has disabled customization WHEN needing to show the close button THEN use the default icon`() {
+        val customCloseIcon: Bitmap = mock()
+        val tab = createCustomTab(
+            "https://www.mozilla.org",
+            id = "mozilla",
+            config = CustomTabConfig(
+                closeButtonIcon = customCloseIcon,
+            ),
+        )
+        val store = BrowserStore(
+            BrowserState(
+                customTabs = listOf(tab),
+            ),
+        )
+        val toolbar = BrowserToolbar(testContext)
+        val useCases = CustomTabsUseCases(
+            store = store,
+            loadUrlUseCase = SessionUseCases(store).loadUrl,
+        )
+        val feature = spy(
+            CustomTabsToolbarFeature(
+                store,
+                toolbar,
+                sessionId = "mozilla",
+                useCases = useCases,
+                customTabsToolbarButtonConfig = CustomTabsToolbarButtonConfig(
+                    allowCustomizingCloseButton = false,
+                ),
+            ) {},
+        )
+
+        feature.start()
+
+        verify(feature).addCloseButton(toolbar.display.colors.menu, null)
+    }
+
     @Test
     fun `WHEN tab is private THEN readableColor is the default private color`() {
         val tab = createCustomTab(
@@ -1297,7 +1783,7 @@ class CustomTabsToolbarFeatureTest {
                 useCases = useCases,
                 menuBuilder = BrowserMenuBuilder(listOf(mock(), mock())),
                 menuItemIndex = 4,
-                updateTheme = false,
+                customTabsColorsConfig = getPrivateCustomTabColorsConfig(),
             ) {},
         )
 
@@ -1310,237 +1796,6 @@ class CustomTabsToolbarFeatureTest {
         verify(feature).addShareButton(privateColor)
     }
 
-    @Test
-    fun `WHEN COLOR_SCHEME_SYSTEM THEN toNightMode returns MODE_NIGHT_FOLLOW_SYSTEM`() {
-        assertEquals(AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM, CustomTabsIntent.COLOR_SCHEME_SYSTEM.toNightMode())
-    }
-
-    @Test
-    fun `WHEN COLOR_SCHEME_LIGHT THEN toNightMode returns MODE_NIGHT_NO`() {
-        assertEquals(AppCompatDelegate.MODE_NIGHT_NO, CustomTabsIntent.COLOR_SCHEME_LIGHT.toNightMode())
-    }
-
-    @Test
-    fun `WHEN COLOR_SCHEME_DARK THEN toNightMode returns MODE_NIGHT_YES`() {
-        assertEquals(AppCompatDelegate.MODE_NIGHT_YES, CustomTabsIntent.COLOR_SCHEME_DARK.toNightMode())
-    }
-
-    @Test
-    fun `WHEN unknown color scheme THEN toNightMode returns null`() {
-        assertEquals(null, 100.toNightMode())
-    }
-
-    @Test
-    fun `WHEN no color scheme params set THEN getConfiguredColorSchemeParams returns null `() {
-        val customTabConfig = CustomTabConfig()
-        assertEquals(null, customTabConfig.colorSchemes?.getConfiguredColorSchemeParams())
-    }
-
-    @Test
-    fun `WHEN only default color scheme params set THEN getConfiguredColorSchemeParams returns default `() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode follow system and is light mode THEN getConfiguredColorSchemeParams returns light color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            lightColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode follow system, is light mode no light color scheme THEN getConfiguredColorSchemeParams returns default scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode follow system and is dark mode THEN getConfiguredColorSchemeParams returns dark color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            darkColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM,
-                isDarkMode = true,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode follow system, is dark mode no dark color scheme THEN getConfiguredColorSchemeParams returns default scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM,
-                isDarkMode = true,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode no THEN getConfiguredColorSchemeParams returns light color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            lightColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_NO,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode no & no light color params THEN getConfiguredColorSchemeParams returns default color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_NO,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode yes THEN getConfiguredColorSchemeParams returns dark color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            darkColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_YES,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode yes & no dark color params THEN getConfiguredColorSchemeParams returns default color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(
-                nightMode = AppCompatDelegate.MODE_NIGHT_YES,
-            ),
-        )
-    }
-
-    @Test
-    fun `WHEN night mode not set THEN getConfiguredColorSchemeParams returns default color scheme`() {
-        val customTabConfig = CustomTabConfig(
-            colorSchemes = ColorSchemes(
-                defaultColorSchemeParams = defaultColorSchemeParams,
-                lightColorSchemeParams = lightColorSchemeParams,
-                darkColorSchemeParams = darkColorSchemeParams,
-            ),
-        )
-
-        assertEquals(
-            defaultColorSchemeParams,
-            customTabConfig.colorSchemes!!.getConfiguredColorSchemeParams(),
-        )
-    }
-
-    @Test
-    fun `WHEN ColorSchemeParams has all properties THEN withDefault returns the same ColorSchemeParams`() {
-        val result = lightColorSchemeParams.withDefault(defaultColorSchemeParams)
-
-        assertEquals(lightColorSchemeParams, result)
-    }
-
-    @Test
-    fun `WHEN ColorSchemeParams has some properties THEN withDefault uses default for the missing properties`() {
-        val colorSchemeParams = ColorSchemeParams(
-            toolbarColor = Color.BLACK,
-            navigationBarDividerColor = Color.YELLOW,
-        )
-
-        val expected = ColorSchemeParams(
-            toolbarColor = colorSchemeParams.toolbarColor,
-            secondaryToolbarColor = defaultColorSchemeParams.secondaryToolbarColor,
-            navigationBarColor = defaultColorSchemeParams.navigationBarColor,
-            navigationBarDividerColor = colorSchemeParams.navigationBarDividerColor,
-        )
-
-        val result = colorSchemeParams.withDefault(defaultColorSchemeParams)
-
-        assertEquals(expected, result)
-    }
-
-    @Test
-    fun `WHEN ColorSchemeParams has no properties THEN withDefault returns all default ColorSchemeParams`() {
-        val result = ColorSchemeParams().withDefault(defaultColorSchemeParams)
-
-        assertEquals(defaultColorSchemeParams, result)
-    }
-
     @Test
     fun `show title only if not empty`() {
         val dispatcher = UnconfinedTestDispatcher()
@@ -1706,24 +1961,9 @@ class CustomTabsToolbarFeatureTest {
         return actionView
     }
 
-    private val defaultColorSchemeParams = ColorSchemeParams(
-        toolbarColor = Color.CYAN,
-        secondaryToolbarColor = Color.GREEN,
-        navigationBarColor = Color.WHITE,
-        navigationBarDividerColor = Color.MAGENTA,
-    )
-
-    private val lightColorSchemeParams = ColorSchemeParams(
-        toolbarColor = Color.BLACK,
-        secondaryToolbarColor = Color.RED,
-        navigationBarColor = Color.BLUE,
-        navigationBarDividerColor = Color.YELLOW,
-    )
-
-    private val darkColorSchemeParams = ColorSchemeParams(
-        toolbarColor = Color.DKGRAY,
-        secondaryToolbarColor = Color.LTGRAY,
-        navigationBarColor = Color.GRAY,
-        navigationBarDividerColor = Color.WHITE,
+    private fun getPrivateCustomTabColorsConfig() = CustomTabsColorsConfig(
+        updateToolbarsColor = false,
+        updateStatusBarColor = false,
+        updateSystemNavigationBarColor = false,
     )
 }
