@@ -4,8 +4,3 @@
 
 MOZ_APP_DISPLAYNAME="Firefox Beta"
 ANDROID_PACKAGE_NAME=org.mozilla.firefox_beta
-MOZ_UPDATER=
-MOZ_ANDROID_ANR_REPORTER=
-MOZ_ANDROID_SHARED_ID=org.mozilla.firefox.sharedID
-MOZ_ANDROID_GCM_SENDERID=965234145045
-MOZ_MMA_GCM_SENDERID=242693410970
\ No newline at end of file
