@@ -8,13 +8,20 @@
 use crate::context::QuirksMode;
 use crate::dom::{TDocument, TElement, TNode, TShadowRoot};
 use crate::invalidation::element::invalidation_map::Dependency;
-use crate::invalidation::element::invalidator::{DescendantInvalidationLists, Invalidation};
+use crate::invalidation::element::invalidator::{
+    DescendantInvalidationLists, Invalidation, SiblingTraversalMap,
+};
 use crate::invalidation::element::invalidator::{InvalidationProcessor, InvalidationVector};
+use crate::selector_parser::SelectorImpl;
 use crate::values::AtomIdent;
 use selectors::attr::CaseSensitivity;
-use selectors::matching::{self, MatchingContext, MatchingMode};
-use selectors::parser::{Combinator, Component, LocalName, SelectorImpl};
-use selectors::{Element, NthIndexCache, SelectorList};
+use selectors::attr::{AttrSelectorOperation, NamespaceConstraint};
+use selectors::matching::{
+    self, MatchingContext, MatchingForInvalidation, MatchingMode, NeedsSelectorFlags,
+    SelectorCaches,
+};
+use selectors::parser::{Combinator, Component, LocalName};
+use selectors::{Element, SelectorList};
 use smallvec::SmallVec;
 
 /// <https://dom.spec.whatwg.org/#dom-element-matches>
@@ -26,7 +33,16 @@ pub fn element_matches<E>(
 where
     E: Element,
 {
-    let mut context = MatchingContext::new(MatchingMode::Normal, None, None, quirks_mode);
+    let mut selector_caches = SelectorCaches::default();
+
+    let mut context = MatchingContext::new(
+        MatchingMode::Normal,
+        None,
+        &mut selector_caches,
+        quirks_mode,
+        NeedsSelectorFlags::No,
+        MatchingForInvalidation::No,
+    );
     context.scope_element = Some(element.opaque());
     context.current_host = element.containing_shadow_host().map(|e| e.opaque());
     matching::matches_selector_list(selector_list, element, &mut context)
@@ -41,13 +57,15 @@ pub fn element_closest<E>(
 where
     E: Element,
 {
-    let mut nth_index_cache = NthIndexCache::default();
+    let mut selector_caches = SelectorCaches::default();
 
     let mut context = MatchingContext::new(
         MatchingMode::Normal,
         None,
-        Some(&mut nth_index_cache),
+        &mut selector_caches,
         quirks_mode,
+        NeedsSelectorFlags::No,
+        MatchingForInvalidation::No,
     );
     context.scope_element = Some(element.opaque());
     context.current_host = element.containing_shadow_host().map(|e| e.opaque());
@@ -122,18 +140,19 @@ impl<E: TElement> SelectorQuery<E> for QueryFirst {
     }
 }
 
-struct QuerySelectorProcessor<'a, E, Q>
+struct QuerySelectorProcessor<'a, 'b, E, Q>
 where
     E: TElement + 'a,
     Q: SelectorQuery<E>,
     Q::Output: 'a,
 {
     results: &'a mut Q::Output,
-    matching_context: MatchingContext<'a, E::Impl>,
+    matching_context: MatchingContext<'b, E::Impl>,
+    traversal_map: SiblingTraversalMap<E>,
     dependencies: &'a [Dependency],
 }
 
-impl<'a, E, Q> InvalidationProcessor<'a, E> for QuerySelectorProcessor<'a, E, Q>
+impl<'a, 'b, E, Q> InvalidationProcessor<'a, 'b, E> for QuerySelectorProcessor<'a, 'b, E, Q>
 where
     E: TElement + 'a,
     Q: SelectorQuery<E>,
@@ -189,10 +208,14 @@ where
         false
     }
 
-    fn matching_context(&mut self) -> &mut MatchingContext<'a, E::Impl> {
+    fn matching_context(&mut self) -> &mut MatchingContext<'b, E::Impl> {
         &mut self.matching_context
     }
 
+    fn sibling_traversal_map(&self) -> &SiblingTraversalMap<E> {
+        &self.traversal_map
+    }
+
     fn should_process_descendants(&mut self, _: E) -> bool {
         if Q::should_stop_after_first_match() {
             return Q::is_empty(&self.results);
@@ -205,6 +228,10 @@ where
         Q::append_element(self.results, e);
     }
 
+    fn invalidated_sibling(&mut self, e: E, _of: E) {
+        Q::append_element(self.results, e);
+    }
+
     fn recursion_limit_exceeded(&mut self, _e: E) {}
     fn invalidated_descendants(&mut self, _e: E, _child: E) {}
 }
@@ -276,12 +303,11 @@ where
 fn fast_connected_elements_with_id<'a, N>(
     root: N,
     id: &AtomIdent,
-    quirks_mode: QuirksMode,
+    case_sensitivity: CaseSensitivity,
 ) -> Result<&'a [N::ConcreteElement], ()>
 where
     N: TNode + 'a,
 {
-    let case_sensitivity = quirks_mode.classes_and_ids_case_sensitivity();
     if case_sensitivity != CaseSensitivity::CaseSensitive {
         return Err(());
     }
@@ -306,20 +332,18 @@ fn collect_elements_with_id<E, Q, F>(
     root: E::ConcreteNode,
     id: &AtomIdent,
     results: &mut Q::Output,
-    quirks_mode: QuirksMode,
+    class_and_id_case_sensitivity: CaseSensitivity,
     mut filter: F,
 ) where
     E: TElement,
     Q: SelectorQuery<E>,
     F: FnMut(E) -> bool,
 {
-    let elements = match fast_connected_elements_with_id(root, id, quirks_mode) {
+    let elements = match fast_connected_elements_with_id(root, id, class_and_id_case_sensitivity) {
         Ok(elements) => elements,
         Err(()) => {
-            let case_sensitivity = quirks_mode.classes_and_ids_case_sensitivity();
-
             collect_all_elements::<E, Q, _>(root, results, |e| {
-                e.has_id(id, case_sensitivity) && filter(e)
+                e.has_id(id, class_and_id_case_sensitivity) && filter(e)
             });
 
             return;
@@ -344,6 +368,15 @@ fn collect_elements_with_id<E, Q, F>(
     }
 }
 
+fn has_attr<E>(element: E, local_name: &crate::LocalName) -> bool
+where
+    E: TElement,
+{
+    let mut found = false;
+    element.each_attr_name(|name| found |= name == local_name);
+    found
+}
+
 #[inline(always)]
 fn local_name_matches<E>(element: E, local_name: &LocalName<E::Impl>) -> bool
 where
@@ -354,7 +387,7 @@ where
         ref lower_name,
     } = *local_name;
 
-    let chosen_name = if element.is_html_element_in_html_document() {
+    let chosen_name = if name == lower_name || element.is_html_element_in_html_document() {
         lower_name
     } else {
         name
@@ -363,12 +396,51 @@ where
     element.local_name() == &**chosen_name
 }
 
+fn get_attr_name(component: &Component<SelectorImpl>) -> Option<&crate::LocalName> {
+    let (name, name_lower) = match component {
+        Component::AttributeInNoNamespace { ref local_name, .. } => return Some(local_name),
+        Component::AttributeInNoNamespaceExists {
+            ref local_name,
+            ref local_name_lower,
+            ..
+        } => (local_name, local_name_lower),
+        Component::AttributeOther(ref attr) => (&attr.local_name, &attr.local_name_lower),
+        _ => return None,
+    };
+    if name != name_lower {
+        return None; // TODO: Maybe optimize this?
+    }
+    Some(name)
+}
+
+fn get_id(component: &Component<SelectorImpl>) -> Option<&AtomIdent> {
+    use selectors::attr::AttrSelectorOperator;
+    Some(match component {
+        Component::ID(ref id) => id,
+        Component::AttributeInNoNamespace {
+            ref operator,
+            ref local_name,
+            ref value,
+            ..
+        } => {
+            if *local_name != local_name!("id") {
+                return None;
+            }
+            if *operator != AttrSelectorOperator::Equal {
+                return None;
+            }
+            AtomIdent::cast(&value.0)
+        },
+        _ => return None,
+    })
+}
+
 /// Fast paths for querySelector with a single simple selector.
 fn query_selector_single_query<E, Q>(
     root: E::ConcreteNode,
     component: &Component<E::Impl>,
     results: &mut Q::Output,
-    quirks_mode: QuirksMode,
+    class_and_id_case_sensitivity: CaseSensitivity,
 ) -> Result<(), ()>
 where
     E: TElement,
@@ -378,30 +450,70 @@ where
         Component::ExplicitUniversalType => {
             collect_all_elements::<E, Q, _>(root, results, |_| true)
         },
-        Component::ID(ref id) => {
-            collect_elements_with_id::<E, Q, _>(root, id, results, quirks_mode, |_| true);
-        },
-        Component::Class(ref class) => {
-            let case_sensitivity = quirks_mode.classes_and_ids_case_sensitivity();
-            collect_all_elements::<E, Q, _>(root, results, |element| {
-                element.has_class(class, case_sensitivity)
-            })
-        },
+        Component::Class(ref class) => collect_all_elements::<E, Q, _>(root, results, |element| {
+            element.has_class(class, class_and_id_case_sensitivity)
+        }),
         Component::LocalName(ref local_name) => {
             collect_all_elements::<E, Q, _>(root, results, |element| {
                 local_name_matches(element, local_name)
             })
         },
-        // TODO(emilio): More fast paths?
-        _ => return Err(()),
+        Component::AttributeInNoNamespaceExists {
+            ref local_name,
+            ref local_name_lower,
+        } => collect_all_elements::<E, Q, _>(root, results, |element| {
+            element.has_attr_in_no_namespace(matching::select_name(
+                &element,
+                local_name,
+                local_name_lower,
+            ))
+        }),
+        Component::AttributeInNoNamespace {
+            ref local_name,
+            ref value,
+            operator,
+            case_sensitivity,
+        } => {
+            let empty_namespace = selectors::parser::namespace_empty_string::<E::Impl>();
+            let namespace_constraint = NamespaceConstraint::Specific(&empty_namespace);
+            collect_all_elements::<E, Q, _>(root, results, |element| {
+                element.attr_matches(
+                    &namespace_constraint,
+                    local_name,
+                    &AttrSelectorOperation::WithValue {
+                        operator,
+                        case_sensitivity: matching::to_unconditional_case_sensitivity(
+                            case_sensitivity,
+                            &element,
+                        ),
+                        value,
+                    },
+                )
+            })
+        },
+        ref other => {
+            let id = match get_id(other) {
+                Some(id) => id,
+                // TODO(emilio): More fast paths?
+                None => return Err(()),
+            };
+            collect_elements_with_id::<E, Q, _>(
+                root,
+                id,
+                results,
+                class_and_id_case_sensitivity,
+                |_| true,
+            );
+        },
     }
 
     Ok(())
 }
 
-enum SimpleFilter<'a, Impl: SelectorImpl> {
+enum SimpleFilter<'a> {
     Class(&'a AtomIdent),
-    LocalName(&'a LocalName<Impl>),
+    Attr(&'a crate::LocalName),
+    LocalName(&'a LocalName<SelectorImpl>),
 }
 
 /// Fast paths for a given selector query.
@@ -425,21 +537,24 @@ where
 {
     // We need to return elements in document order, and reordering them
     // afterwards is kinda silly.
-    if selector_list.0.len() > 1 {
+    if selector_list.len() > 1 {
         return Err(());
     }
 
-    let selector = &selector_list.0[0];
-    let quirks_mode = matching_context.quirks_mode();
-
+    let selector = &selector_list.slice()[0];
+    let class_and_id_case_sensitivity = matching_context.classes_and_ids_case_sensitivity();
     // Let's just care about the easy cases for now.
     if selector.len() == 1 {
-        return query_selector_single_query::<E, Q>(
+        if query_selector_single_query::<E, Q>(
             root,
             selector.iter().next().unwrap(),
             results,
-            quirks_mode,
-        );
+            class_and_id_case_sensitivity,
+        )
+        .is_ok()
+        {
+            return Ok(());
+        }
     }
 
     let mut iter = selector.iter();
@@ -455,60 +570,6 @@ where
 
         'component_loop: for component in &mut iter {
             match *component {
-                Component::ID(ref id) => {
-                    if combinator.is_none() {
-                        // In the rightmost compound, just find descendants of
-                        // root that match the selector list with that id.
-                        collect_elements_with_id::<E, Q, _>(root, id, results, quirks_mode, |e| {
-                            matching::matches_selector_list(selector_list, &e, matching_context)
-                        });
-
-                        return Ok(());
-                    }
-
-                    let elements = fast_connected_elements_with_id(root, id, quirks_mode)?;
-                    if elements.is_empty() {
-                        return Ok(());
-                    }
-
-                    // Results need to be in document order. Let's not bother
-                    // reordering or deduplicating nodes, which we would need to
-                    // do if one element with the given id were a descendant of
-                    // another element with that given id.
-                    if !Q::should_stop_after_first_match() && elements.len() > 1 {
-                        continue;
-                    }
-
-                    for element in elements {
-                        // If the element is not a descendant of the root, then
-                        // it may have descendants that match our selector that
-                        // _are_ descendants of the root, and other descendants
-                        // that match our selector that are _not_.
-                        //
-                        // So we can't just walk over the element's descendants
-                        // and match the selector against all of them, nor can
-                        // we skip looking at this element's descendants.
-                        //
-                        // Give up on trying to optimize based on this id and
-                        // keep walking our selector.
-                        if !connected_element_is_descendant_of(*element, root) {
-                            continue 'component_loop;
-                        }
-
-                        query_selector_slow::<E, Q>(
-                            element.as_node(),
-                            selector_list,
-                            results,
-                            matching_context,
-                        );
-
-                        if Q::should_stop_after_first_match() && !Q::is_empty(&results) {
-                            break;
-                        }
-                    }
-
-                    return Ok(());
-                },
                 Component::Class(ref class) => {
                     if combinator.is_none() {
                         simple_filter = Some(SimpleFilter::Class(class));
@@ -524,7 +585,80 @@ where
                         simple_filter = Some(SimpleFilter::LocalName(local_name));
                     }
                 },
-                _ => {},
+                ref other => {
+                    if let Some(id) = get_id(other) {
+                        if combinator.is_none() {
+                            // In the rightmost compound, just find descendants of root that match
+                            // the selector list with that id.
+                            collect_elements_with_id::<E, Q, _>(
+                                root,
+                                id,
+                                results,
+                                class_and_id_case_sensitivity,
+                                |e| {
+                                    matching::matches_selector_list(
+                                        selector_list,
+                                        &e,
+                                        matching_context,
+                                    )
+                                },
+                            );
+                            return Ok(());
+                        }
+
+                        let elements = fast_connected_elements_with_id(
+                            root,
+                            id,
+                            class_and_id_case_sensitivity,
+                        )?;
+                        if elements.is_empty() {
+                            return Ok(());
+                        }
+
+                        // Results need to be in document order. Let's not bother
+                        // reordering or deduplicating nodes, which we would need to
+                        // do if one element with the given id were a descendant of
+                        // another element with that given id.
+                        if !Q::should_stop_after_first_match() && elements.len() > 1 {
+                            continue;
+                        }
+
+                        for element in elements {
+                            // If the element is not a descendant of the root, then
+                            // it may have descendants that match our selector that
+                            // _are_ descendants of the root, and other descendants
+                            // that match our selector that are _not_.
+                            //
+                            // So we can't just walk over the element's descendants
+                            // and match the selector against all of them, nor can
+                            // we skip looking at this element's descendants.
+                            //
+                            // Give up on trying to optimize based on this id and
+                            // keep walking our selector.
+                            if !connected_element_is_descendant_of(*element, root) {
+                                continue 'component_loop;
+                            }
+
+                            query_selector_slow::<E, Q>(
+                                element.as_node(),
+                                selector_list,
+                                results,
+                                matching_context,
+                            );
+
+                            if Q::should_stop_after_first_match() && !Q::is_empty(&results) {
+                                break;
+                            }
+                        }
+
+                        return Ok(());
+                    }
+                    if combinator.is_none() && simple_filter.is_none() {
+                        if let Some(attr_name) = get_attr_name(other) {
+                            simple_filter = Some(SimpleFilter::Attr(attr_name));
+                        }
+                    }
+                },
             }
         }
 
@@ -557,9 +691,8 @@ where
 
     match simple_filter {
         SimpleFilter::Class(ref class) => {
-            let case_sensitivity = quirks_mode.classes_and_ids_case_sensitivity();
             collect_all_elements::<E, Q, _>(root, results, |element| {
-                element.has_class(class, case_sensitivity) &&
+                element.has_class(class, class_and_id_case_sensitivity) &&
                     matching::matches_selector_list(selector_list, &element, matching_context)
             });
         },
@@ -569,6 +702,12 @@ where
                     matching::matches_selector_list(selector_list, &element, matching_context)
             });
         },
+        SimpleFilter::Attr(ref local_name) => {
+            collect_all_elements::<E, Q, _>(root, results, |element| {
+                has_attr(element, local_name) &&
+                    matching::matches_selector_list(selector_list, &element, matching_context)
+            });
+        },
     }
 
     Ok(())
@@ -610,16 +749,17 @@ pub fn query_selector<E, Q>(
 {
     use crate::invalidation::element::invalidator::TreeStyleInvalidator;
 
+    let mut selector_caches = SelectorCaches::default();
     let quirks_mode = root.owner_doc().quirks_mode();
 
-    let mut nth_index_cache = NthIndexCache::default();
     let mut matching_context = MatchingContext::new(
         MatchingMode::Normal,
         None,
-        Some(&mut nth_index_cache),
+        &mut selector_caches,
         quirks_mode,
+        NeedsSelectorFlags::No,
+        MatchingForInvalidation::No,
     );
-
     let root_element = root.as_element();
     matching_context.scope_element = root_element.map(|e| e.opaque());
     matching_context.current_host = match root_element {
@@ -647,19 +787,20 @@ pub fn query_selector<E, Q>(
     // A selector with a combinator needs to have a length of at least 3: A
     // simple selector, a combinator, and another simple selector.
     let invalidation_may_be_useful = may_use_invalidation == MayUseInvalidation::Yes &&
-        selector_list.0.iter().any(|s| s.len() > 2);
+        selector_list.slice().iter().any(|s| s.len() > 2);
 
     if root_element.is_some() || !invalidation_may_be_useful {
         query_selector_slow::<E, Q>(root, selector_list, results, &mut matching_context);
     } else {
         let dependencies = selector_list
-            .0
+            .slice()
             .iter()
             .map(|selector| Dependency::for_full_selector_invalidation(selector.clone()))
             .collect::<SmallVec<[_; 5]>>();
         let mut processor = QuerySelectorProcessor::<E, Q> {
             results,
             matching_context,
+            traversal_map: SiblingTraversalMap::default(),
             dependencies: &dependencies,
         };
 
