@@ -8,28 +8,25 @@
 
 use crate::error_reporting::ContextualParseError;
 use crate::parser::{Parse, ParserContext};
-#[cfg(feature = "gecko")]
 use crate::properties::longhands::font_language_override;
 use crate::shared_lock::{SharedRwLockReadGuard, ToCssWithGuard};
 use crate::str::CssStringWriter;
-use crate::values::computed::font::FamilyName;
+use crate::values::computed::font::{FamilyName, FontStretch};
 use crate::values::generics::font::FontStyle as GenericFontStyle;
-#[cfg(feature = "gecko")]
-use crate::values::specified::font::SpecifiedFontFeatureSettings;
-use crate::values::specified::font::SpecifiedFontStyle;
-#[cfg(feature = "gecko")]
-use crate::values::specified::font::SpecifiedFontVariationSettings;
-use crate::values::specified::font::{AbsoluteFontWeight, FontStretch, MetricsOverride};
+use crate::values::specified::font::{
+    AbsoluteFontWeight, FontStretch as SpecifiedFontStretch, FontFeatureSettings,
+    FontVariationSettings, MetricsOverride, SpecifiedFontStyle,
+};
 use crate::values::specified::url::SpecifiedUrl;
 use crate::values::specified::{Angle, NonNegativePercentage};
-#[cfg(feature = "gecko")]
 use cssparser::UnicodeRange;
-use cssparser::{AtRuleParser, DeclarationListParser, DeclarationParser, Parser};
-use cssparser::{CowRcStr, SourceLocation};
+use cssparser::{
+    AtRuleParser, CowRcStr, DeclarationParser, Parser, ParserState, QualifiedRuleParser,
+    RuleBodyItemParser, RuleBodyParser, SourceLocation,
+};
 use selectors::parser::SelectorParseErrorKind;
 use std::fmt::{self, Write};
-use style_traits::values::SequenceWriter;
-use style_traits::{Comma, CssWriter, OneOrMoreSeparated, ParseError};
+use style_traits::{CssWriter, ParseError};
 use style_traits::{StyleParseErrorKind, ToCss};
 
 /// A source for a font-face rule.
@@ -43,8 +40,166 @@ pub enum Source {
     Local(FamilyName),
 }
 
-impl OneOrMoreSeparated for Source {
-    type S = Comma;
+/// A list of sources for the font-face src descriptor.
+#[derive(Clone, Debug, Eq, PartialEq, ToCss, ToShmem)]
+#[css(comma)]
+pub struct SourceList(#[css(iterable)] pub Vec<Source>);
+
+// We can't just use OneOrMoreSeparated to derive Parse for the Source list,
+// because we want to filter out components that parsed as None, then fail if no
+// valid components remain. So we provide our own implementation here.
+impl Parse for SourceList {
+    fn parse<'i, 't>(
+        context: &ParserContext,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<Self, ParseError<'i>> {
+        // Parse the comma-separated list, then let filter_map discard any None items.
+        let list = input
+            .parse_comma_separated(|input| {
+                let s = input.parse_entirely(|input| Source::parse(context, input));
+                while input.next().is_ok() {}
+                Ok(s.ok())
+            })?
+            .into_iter()
+            .filter_map(|s| s)
+            .collect::<Vec<Source>>();
+        if list.is_empty() {
+            Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
+        } else {
+            Ok(SourceList(list))
+        }
+    }
+}
+
+/// Keywords for the font-face src descriptor's format() function.
+/// ('None' and 'Unknown' are for internal use in gfx, not exposed to CSS.)
+#[derive(Clone, Copy, Debug, Eq, Parse, PartialEq, ToCss, ToShmem)]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
+#[repr(u8)]
+#[allow(missing_docs)]
+pub enum FontFaceSourceFormatKeyword {
+    #[css(skip)]
+    None,
+    Collection,
+    EmbeddedOpentype,
+    Opentype,
+    Svg,
+    Truetype,
+    Woff,
+    Woff2,
+    #[css(skip)]
+    Unknown,
+}
+
+/// Flags for the @font-face tech() function, indicating font technologies
+/// required by the resource.
+#[derive(Clone, Copy, Debug, Eq, PartialEq, ToShmem)]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
+#[repr(C)]
+pub struct FontFaceSourceTechFlags(u16);
+bitflags! {
+    impl FontFaceSourceTechFlags: u16 {
+        /// Font requires OpenType feature support.
+        const FEATURES_OPENTYPE = 1 << 0;
+        /// Font requires Apple Advanced Typography support.
+        const FEATURES_AAT = 1 << 1;
+        /// Font requires Graphite shaping support.
+        const FEATURES_GRAPHITE = 1 << 2;
+        /// Font requires COLRv0 rendering support (simple list of colored layers).
+        const COLOR_COLRV0 = 1 << 3;
+        /// Font requires COLRv1 rendering support (graph of paint operations).
+        const COLOR_COLRV1 = 1 << 4;
+        /// Font requires SVG glyph rendering support.
+        const COLOR_SVG = 1 << 5;
+        /// Font has bitmap glyphs in 'sbix' format.
+        const COLOR_SBIX = 1 << 6;
+        /// Font has bitmap glyphs in 'CBDT' format.
+        const COLOR_CBDT = 1 << 7;
+        /// Font requires OpenType Variations support.
+        const VARIATIONS = 1 << 8;
+        /// Font requires CPAL palette selection support.
+        const PALETTES = 1 << 9;
+        /// Font requires support for incremental downloading.
+        const INCREMENTAL = 1 << 10;
+    }
+}
+
+impl FontFaceSourceTechFlags {
+    /// Parse a single font-technology keyword and return its flag.
+    pub fn parse_one<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
+        Ok(try_match_ident_ignore_ascii_case! { input,
+            "features-opentype" => Self::FEATURES_OPENTYPE,
+            "features-aat" => Self::FEATURES_AAT,
+            "features-graphite" => Self::FEATURES_GRAPHITE,
+            "color-colrv0" => Self::COLOR_COLRV0,
+            "color-colrv1" => Self::COLOR_COLRV1,
+            "color-svg" => Self::COLOR_SVG,
+            "color-sbix" => Self::COLOR_SBIX,
+            "color-cbdt" => Self::COLOR_CBDT,
+            "variations" => Self::VARIATIONS,
+            "palettes" => Self::PALETTES,
+            "incremental" => Self::INCREMENTAL,
+        })
+    }
+}
+
+impl Parse for FontFaceSourceTechFlags {
+    fn parse<'i, 't>(
+        _context: &ParserContext,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<Self, ParseError<'i>> {
+        let location = input.current_source_location();
+        // We don't actually care about the return value of parse_comma_separated,
+        // because we insert the flags into result as we go.
+        let mut result = Self::empty();
+        input.parse_comma_separated(|input| {
+            let flag = Self::parse_one(input)?;
+            result.insert(flag);
+            Ok(())
+        })?;
+        if !result.is_empty() {
+            Ok(result)
+        } else {
+            Err(location.new_custom_error(StyleParseErrorKind::UnspecifiedError))
+        }
+    }
+}
+
+#[allow(unused_assignments)]
+impl ToCss for FontFaceSourceTechFlags {
+    fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
+    where
+        W: fmt::Write,
+    {
+        let mut first = true;
+
+        macro_rules! write_if_flag {
+            ($s:expr => $f:ident) => {
+                if self.contains(Self::$f) {
+                    if first {
+                        first = false;
+                    } else {
+                        dest.write_str(", ")?;
+                    }
+                    dest.write_str($s)?;
+                }
+            };
+        }
+
+        write_if_flag!("features-opentype" => FEATURES_OPENTYPE);
+        write_if_flag!("features-aat" => FEATURES_AAT);
+        write_if_flag!("features-graphite" => FEATURES_GRAPHITE);
+        write_if_flag!("color-colrv0" => COLOR_COLRV0);
+        write_if_flag!("color-colrv1" => COLOR_COLRV1);
+        write_if_flag!("color-svg" => COLOR_SVG);
+        write_if_flag!("color-sbix" => COLOR_SBIX);
+        write_if_flag!("color-cbdt" => COLOR_CBDT);
+        write_if_flag!("variations" => VARIATIONS);
+        write_if_flag!("palettes" => PALETTES);
+        write_if_flag!("incremental" => INCREMENTAL);
+
+        Ok(())
+    }
 }
 
 /// A POD representation for Gecko. All pointers here are non-owned and as such
@@ -52,15 +207,27 @@ impl OneOrMoreSeparated for Source {
 ///
 /// All the strings are of course utf8.
 #[cfg(feature = "gecko")]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 #[repr(u8)]
 #[allow(missing_docs)]
 pub enum FontFaceSourceListComponent {
     Url(*const crate::gecko::url::CssUrl),
     Local(*mut crate::gecko_bindings::structs::nsAtom),
-    FormatHint {
+    FormatHintKeyword(FontFaceSourceFormatKeyword),
+    FormatHintString {
         length: usize,
         utf8_bytes: *const u8,
     },
+    TechFlags(FontFaceSourceTechFlags),
+}
+
+#[derive(Clone, Debug, Eq, PartialEq, ToCss, ToShmem)]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
+#[repr(u8)]
+#[allow(missing_docs)]
+pub enum FontFaceSourceFormat {
+    Keyword(FontFaceSourceFormatKeyword),
+    String(String),
 }
 
 /// A `UrlSource` represents a font-face source that has been specified with a
@@ -72,8 +239,10 @@ pub enum FontFaceSourceListComponent {
 pub struct UrlSource {
     /// The specified url.
     pub url: SpecifiedUrl,
-    /// The format hints specified with the `format()` function.
-    pub format_hints: Vec<String>,
+    /// The format hint specified with the `format()` function, if present.
+    pub format_hint: Option<FontFaceSourceFormat>,
+    /// The font technology flags specified with the `tech()` function, if any.
+    pub tech_flags: FontFaceSourceTechFlags,
 }
 
 impl ToCss for UrlSource {
@@ -82,14 +251,14 @@ impl ToCss for UrlSource {
         W: fmt::Write,
     {
         self.url.to_css(dest)?;
-        if !self.format_hints.is_empty() {
+        if let Some(hint) = &self.format_hint {
             dest.write_str(" format(")?;
-            {
-                let mut writer = SequenceWriter::new(dest, ", ");
-                for hint in self.format_hints.iter() {
-                    writer.item(hint)?;
-                }
-            }
+            hint.to_css(dest)?;
+            dest.write_char(')')?;
+        }
+        if !self.tech_flags.is_empty() {
+            dest.write_str(" tech(")?;
+            self.tech_flags.to_css(dest)?;
             dest.write_char(')')?;
         }
         Ok(())
@@ -134,7 +303,7 @@ macro_rules! impl_range {
             {
                 self.0.to_css(dest)?;
                 if self.0 != self.1 {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     self.1.to_css(dest)?;
                 }
                 Ok(())
@@ -170,7 +339,7 @@ fn sort_range<T: PartialOrd>(a: T, b: T) -> (T, T) {
 impl FontWeightRange {
     /// Returns a computed font-stretch range.
     pub fn compute(&self) -> ComputedFontWeightRange {
-        let (min, max) = sort_range(self.0.compute().0, self.1.compute().0);
+        let (min, max) = sort_range(self.0.compute().value(), self.1.compute().value());
         ComputedFontWeightRange(min, max)
     }
 }
@@ -179,23 +348,23 @@ impl FontWeightRange {
 ///
 /// https://drafts.csswg.org/css-fonts-4/#descdef-font-face-font-stretch
 #[derive(Clone, Debug, PartialEq, ToShmem)]
-pub struct FontStretchRange(pub FontStretch, pub FontStretch);
-impl_range!(FontStretchRange, FontStretch);
+pub struct FontStretchRange(pub SpecifiedFontStretch, pub SpecifiedFontStretch);
+impl_range!(FontStretchRange, SpecifiedFontStretch);
 
-/// The computed representation of the above, so that
-/// Gecko can read them easily.
+/// The computed representation of the above, so that Gecko can read them
+/// easily.
 #[repr(C)]
 #[allow(missing_docs)]
-pub struct ComputedFontStretchRange(f32, f32);
+pub struct ComputedFontStretchRange(FontStretch, FontStretch);
 
 impl FontStretchRange {
     /// Returns a computed font-stretch range.
     pub fn compute(&self) -> ComputedFontStretchRange {
-        fn compute_stretch(s: &FontStretch) -> f32 {
+        fn compute_stretch(s: &SpecifiedFontStretch) -> FontStretch {
             match *s {
-                FontStretch::Keyword(ref kw) => kw.compute().0,
-                FontStretch::Stretch(ref p) => p.0.get(),
-                FontStretch::System(..) => unreachable!(),
+                SpecifiedFontStretch::Keyword(ref kw) => kw.compute(),
+                SpecifiedFontStretch::Stretch(ref p) => FontStretch::from_percentage(p.0.get()),
+                SpecifiedFontStretch::System(..) => unreachable!(),
             }
         }
 
@@ -210,7 +379,6 @@ impl FontStretchRange {
 #[derive(Clone, Debug, PartialEq, ToShmem)]
 #[allow(missing_docs)]
 pub enum FontStyle {
-    Normal,
     Italic,
     Oblique(Angle, Angle),
 }
@@ -220,7 +388,6 @@ pub enum FontStyle {
 #[repr(u8)]
 #[allow(missing_docs)]
 pub enum ComputedFontStyleDescriptor {
-    Normal,
     Italic,
     Oblique(f32, f32),
 }
@@ -230,9 +397,14 @@ impl Parse for FontStyle {
         context: &ParserContext,
         input: &mut Parser<'i, 't>,
     ) -> Result<Self, ParseError<'i>> {
+        // We parse 'normal' explicitly here to distinguish it from 'oblique 0deg',
+        // because we must not accept a following angle.
+        if input.try_parse(|i| i.expect_ident_matching("normal")).is_ok() {
+            return Ok(FontStyle::Oblique(Angle::zero(), Angle::zero()));
+        }
+
         let style = SpecifiedFontStyle::parse(context, input)?;
         Ok(match style {
-            GenericFontStyle::Normal => FontStyle::Normal,
             GenericFontStyle::Italic => FontStyle::Italic,
             GenericFontStyle::Oblique(angle) => {
                 let second_angle = input
@@ -251,9 +423,13 @@ impl ToCss for FontStyle {
         W: fmt::Write,
     {
         match *self {
-            FontStyle::Normal => dest.write_str("normal"),
             FontStyle::Italic => dest.write_str("italic"),
             FontStyle::Oblique(ref first, ref second) => {
+                // Not first.is_zero() because we don't want to serialize
+                // `oblique calc(0deg)` as `normal`.
+                if *first == Angle::zero() && first == second {
+                    return dest.write_str("normal");
+                }
                 dest.write_str("oblique")?;
                 if *first != SpecifiedFontStyle::default_angle() || first != second {
                     dest.write_char(' ')?;
@@ -273,7 +449,6 @@ impl FontStyle {
     /// Returns a computed font-style descriptor.
     pub fn compute(&self) -> ComputedFontStyleDescriptor {
         match *self {
-            FontStyle::Normal => ComputedFontStyleDescriptor::Normal,
             FontStyle::Italic => ComputedFontStyleDescriptor::Italic,
             FontStyle::Oblique(ref first, ref second) => {
                 let (min, max) = sort_range(
@@ -296,11 +471,11 @@ pub fn parse_font_face_block(
 ) -> FontFaceRuleData {
     let mut rule = FontFaceRuleData::empty(location);
     {
-        let parser = FontFaceRuleParser {
-            context: context,
+        let mut parser = FontFaceRuleParser {
+            context,
             rule: &mut rule,
         };
-        let mut iter = DeclarationListParser::new(input, parser);
+        let mut iter = RuleBodyParser::new(input, &mut parser);
         while let Some(declaration) = iter.next() {
             if let Err((error, slice)) = declaration {
                 let location = error.location;
@@ -316,54 +491,6 @@ pub fn parse_font_face_block(
 #[cfg(feature = "servo")]
 pub struct FontFace<'a>(&'a FontFaceRuleData);
 
-/// A list of effective sources that we send over through IPC to the font cache.
-#[cfg(feature = "servo")]
-#[derive(Clone, Debug)]
-#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
-pub struct EffectiveSources(Vec<Source>);
-
-#[cfg(feature = "servo")]
-impl<'a> FontFace<'a> {
-    /// Returns the list of effective sources for that font-face, that is the
-    /// sources which don't list any format hint, or the ones which list at
-    /// least "truetype" or "opentype".
-    pub fn effective_sources(&self) -> EffectiveSources {
-        EffectiveSources(
-            self.sources()
-                .iter()
-                .rev()
-                .filter(|source| {
-                    if let Source::Url(ref url_source) = **source {
-                        let hints = &url_source.format_hints;
-                        // We support only opentype fonts and truetype is an alias for
-                        // that format. Sources without format hints need to be
-                        // downloaded in case we support them.
-                        hints.is_empty() ||
-                            hints.iter().any(|hint| {
-                                hint == "truetype" || hint == "opentype" || hint == "woff"
-                            })
-                    } else {
-                        true
-                    }
-                })
-                .cloned()
-                .collect(),
-        )
-    }
-}
-
-#[cfg(feature = "servo")]
-impl Iterator for EffectiveSources {
-    type Item = Source;
-    fn next(&mut self) -> Option<Source> {
-        self.0.pop()
-    }
-
-    fn size_hint(&self) -> (usize, Option<usize>) {
-        (self.0.len(), Some(self.0.len()))
-    }
-}
-
 struct FontFaceRuleParser<'a, 'b: 'a> {
     context: &'a ParserContext<'b>,
     rule: &'a mut FontFaceRuleData,
@@ -376,6 +503,23 @@ impl<'a, 'b, 'i> AtRuleParser<'i> for FontFaceRuleParser<'a, 'b> {
     type Error = StyleParseErrorKind<'i>;
 }
 
+impl<'a, 'b, 'i> QualifiedRuleParser<'i> for FontFaceRuleParser<'a, 'b> {
+    type Prelude = ();
+    type QualifiedRule = ();
+    type Error = StyleParseErrorKind<'i>;
+}
+
+impl<'a, 'b, 'i> RuleBodyItemParser<'i, (), StyleParseErrorKind<'i>>
+    for FontFaceRuleParser<'a, 'b>
+{
+    fn parse_qualified(&self) -> bool {
+        false
+    }
+    fn parse_declarations(&self) -> bool {
+        true
+    }
+}
+
 impl Parse for Source {
     fn parse<'i, 't>(
         context: &ParserContext,
@@ -393,40 +537,45 @@ impl Parse for Source {
         let url = SpecifiedUrl::parse(context, input)?;
 
         // Parsing optional format()
-        let format_hints = if input
+        let format_hint = if input
             .try_parse(|input| input.expect_function_matching("format"))
             .is_ok()
         {
             input.parse_nested_block(|input| {
-                input.parse_comma_separated(|input| Ok(input.expect_string()?.as_ref().to_owned()))
+                if let Ok(kw) = input.try_parse(FontFaceSourceFormatKeyword::parse) {
+                    Ok(Some(FontFaceSourceFormat::Keyword(kw)))
+                } else {
+                    let s = input.expect_string()?.as_ref().to_owned();
+                    Ok(Some(FontFaceSourceFormat::String(s)))
+                }
             })?
         } else {
-            vec![]
+            None
+        };
+
+        // Parse optional tech()
+        let tech_flags = if static_prefs::pref!("layout.css.font-tech.enabled") &&
+            input
+                .try_parse(|input| input.expect_function_matching("tech"))
+                .is_ok()
+        {
+            input.parse_nested_block(|input| FontFaceSourceTechFlags::parse(context, input))?
+        } else {
+            FontFaceSourceTechFlags::empty()
         };
 
         Ok(Source::Url(UrlSource {
-            url: url,
-            format_hints: format_hints,
+            url,
+            format_hint,
+            tech_flags,
         }))
     }
 }
 
 macro_rules! is_descriptor_enabled {
-    ("font-display") => {
-        static_prefs::pref!("layout.css.font-display.enabled")
-    };
     ("font-variation-settings") => {
         static_prefs::pref!("layout.css.font-variations.enabled")
     };
-    ("ascent-override") => {
-        static_prefs::pref!("layout.css.font-metrics-overrides.enabled")
-    };
-    ("descent-override") => {
-        static_prefs::pref!("layout.css.font-metrics-overrides.enabled")
-    };
-    ("line-gap-override") => {
-        static_prefs::pref!("layout.css.font-metrics-overrides.enabled")
-    };
     ("size-adjust") => {
         static_prefs::pref!("layout.css.size-adjust.enabled")
     };
@@ -468,7 +617,7 @@ macro_rules! font_face_descriptors_common {
                 $(
                     if let Some(ref value) = self.$ident {
                         dest.write_str(concat!($name, ": "))?;
-                        ToCss::to_css(value, &mut CssWriter::new(dest))?;
+                        value.to_css(&mut CssWriter::new(dest))?;
                         dest.write_str("; ")?;
                     }
                 )*
@@ -484,6 +633,7 @@ macro_rules! font_face_descriptors_common {
                &mut self,
                name: CowRcStr<'i>,
                input: &mut Parser<'i, 't>,
+               _declaration_start: &ParserState,
             ) -> Result<(), ParseError<'i>> {
                 match_ignore_ascii_case! { &*name,
                     $(
@@ -509,7 +659,7 @@ impl ToCssWithGuard for FontFaceRuleData {
     fn to_css(&self, _guard: &SharedRwLockReadGuard, dest: &mut CssStringWriter) -> fmt::Result {
         dest.write_str("@font-face { ")?;
         self.decl_to_css(dest)?;
-        dest.write_str("}")
+        dest.write_char('}')
     }
 }
 
@@ -555,14 +705,13 @@ macro_rules! font_face_descriptors {
     }
 }
 
-#[cfg(feature = "gecko")]
 font_face_descriptors! {
     mandatory descriptors = [
         /// The name of this font face
         "font-family" family / mFamily: FamilyName,
 
         /// The alternative sources for this font face.
-        "src" sources / mSrc: Vec<Source>,
+        "src" sources / mSrc: SourceList,
     ]
     optional descriptors = [
         /// The style of this font face.
@@ -581,10 +730,10 @@ font_face_descriptors! {
         "unicode-range" unicode_range / mUnicodeRange: Vec<UnicodeRange>,
 
         /// The feature settings of this font face.
-        "font-feature-settings" feature_settings / mFontFeatureSettings: SpecifiedFontFeatureSettings,
+        "font-feature-settings" feature_settings / mFontFeatureSettings: FontFeatureSettings,
 
         /// The variation settings of this font face.
-        "font-variation-settings" variation_settings / mFontVariationSettings: SpecifiedFontVariationSettings,
+        "font-variation-settings" variation_settings / mFontVariationSettings: FontVariationSettings,
 
         /// The language override of this font face.
         "font-language-override" language_override / mFontLanguageOverride: font_language_override::SpecifiedValue,
@@ -602,16 +751,3 @@ font_face_descriptors! {
         "size-adjust" size_adjust / mSizeAdjust: NonNegativePercentage,
     ]
 }
-
-#[cfg(feature = "servo")]
-font_face_descriptors! {
-    mandatory descriptors = [
-        /// The name of this font face
-        "font-family" family / mFamily: FamilyName,
-
-        /// The alternative sources for this font face.
-        "src" sources / mSrc: Vec<Source>,
-    ]
-    optional descriptors = [
-    ]
-}
