@@ -11,6 +11,7 @@ use crate::custom_properties::{
     CustomPropertiesBuilder, DeferFontRelativeCustomPropertyResolution,
 };
 use crate::dom::TElement;
+#[cfg(feature = "gecko")]
 use crate::font_metrics::FontMetricsOrientation;
 use crate::logical_geometry::WritingMode;
 use crate::properties::{
@@ -26,13 +27,13 @@ use crate::style_adjuster::StyleAdjuster;
 use crate::stylesheets::container_rule::ContainerSizeQuery;
 use crate::stylesheets::{layer_rule::LayerOrder, Origin};
 use crate::stylist::Stylist;
+#[cfg(feature = "gecko")]
 use crate::values::specified::length::FontBaseSize;
 use crate::values::{computed, specified};
 use fxhash::FxHashMap;
 use servo_arc::Arc;
 use smallvec::SmallVec;
 use std::borrow::Cow;
-use std::mem;
 
 /// Whether we're resolving a style with the purposes of reparenting for ::first-line.
 #[derive(Copy, Clone)]
@@ -305,6 +306,7 @@ where
         CascadeMode::Visited { unvisited_context } => {
             context.builder.custom_properties = unvisited_context.builder.custom_properties.clone();
             context.builder.writing_mode = unvisited_context.builder.writing_mode;
+            context.builder.color_scheme = unvisited_context.builder.color_scheme;
             // We never insert visited styles into the cache so we don't need to try looking it up.
             // It also wouldn't be super-profitable, only a handful :visited properties are
             // non-inherited.
@@ -418,12 +420,15 @@ fn tweak_when_ignoring_colors(
     }
 
     // Always honor colors if forced-color-adjust is set to none.
-    let forced = context
-        .builder
-        .get_inherited_text()
-        .clone_forced_color_adjust();
-    if forced == computed::ForcedColorAdjust::None {
-        return;
+    #[cfg(feature = "gecko")]
+    {
+        let forced = context
+            .builder
+            .get_inherited_text()
+            .clone_forced_color_adjust();
+        if forced == computed::ForcedColorAdjust::None {
+            return;
+        }
     }
 
     // Don't override background-color on ::-moz-color-swatch. It is set as an
@@ -574,7 +579,7 @@ struct Declarations<'a> {
     /// Whether we have any prioritary property. This is just a minor optimization.
     has_prioritary_properties: bool,
     /// A list of all the applicable longhand declarations.
-    longhand_declarations: SmallVec<[Declaration<'a>; 32]>,
+    longhand_declarations: SmallVec<[Declaration<'a>; 64]>,
     /// The prioritary property position data.
     prioritary_positions: [PrioritaryDeclarationPosition; property_counts::PRIORITARY],
 }
@@ -756,13 +761,19 @@ impl<'b> Cascade<'b> {
             return;
         }
 
-        let has_writing_mode = apply!(WritingMode) | apply!(Direction) | apply!(TextOrientation);
+        let has_writing_mode = apply!(WritingMode) | apply!(Direction);
+        #[cfg(feature = "gecko")]
+        let has_writing_mode = has_writing_mode | apply!(TextOrientation);
+
         if has_writing_mode {
-            self.compute_writing_mode(context);
+            context.builder.writing_mode = WritingMode::new(context.builder.get_inherited_box())
         }
 
         if apply!(Zoom) {
-            self.compute_zoom(context);
+            context.builder.effective_zoom = context
+                .builder
+                .inherited_effective_zoom()
+                .compute_effective(context.builder.specified_zoom());
             // NOTE(emilio): This is a bit of a hack, but matches the shipped WebKit and Blink
             // behavior for now. Ideally, in the future, we have a pass over all
             // implicitly-or-explicitly-inherited properties that can contain lengths and
@@ -773,41 +784,56 @@ impl<'b> Cascade<'b> {
         // Compute font-family.
         let has_font_family = apply!(FontFamily);
         let has_lang = apply!(XLang);
-        if has_lang {
-            self.recompute_initial_font_family_if_needed(&mut context.builder);
-        }
-        if has_font_family {
-            self.prioritize_user_fonts_if_needed(&mut context.builder);
+        #[cfg(feature = "gecko")]
+        {
+            if has_lang {
+                self.recompute_initial_font_family_if_needed(&mut context.builder);
+            }
+            if has_font_family {
+                self.prioritize_user_fonts_if_needed(&mut context.builder);
+            }
+
+            // Compute font-size.
+            if apply!(XTextScale) {
+                self.unzoom_fonts_if_needed(&mut context.builder);
+            }
+            let has_font_size = apply!(FontSize);
+            let has_math_depth = apply!(MathDepth);
+            let has_min_font_size_ratio = apply!(MozMinFontSizeRatio);
+
+            if has_math_depth && has_font_size {
+                self.recompute_math_font_size_if_needed(context);
+            }
+            if has_lang || has_font_family {
+                self.recompute_keyword_font_size_if_needed(context);
+            }
+            if has_font_size || has_min_font_size_ratio || has_lang || has_font_family {
+                self.constrain_font_size_if_needed(&mut context.builder);
+            }
         }
 
-        // Compute font-size.
-        if apply!(XTextScale) {
-            self.unzoom_fonts_if_needed(&mut context.builder);
-        }
-        let has_font_size = apply!(FontSize);
-        let has_math_depth = apply!(MathDepth);
-        let has_min_font_size_ratio = apply!(MozMinFontSizeRatio);
-
-        if has_math_depth && has_font_size {
-            self.recompute_math_font_size_if_needed(context);
-        }
-        if has_lang || has_font_family {
-            self.recompute_keyword_font_size_if_needed(context);
-        }
-        if has_font_size || has_min_font_size_ratio || has_lang || has_font_family {
-            self.constrain_font_size_if_needed(&mut context.builder);
+        #[cfg(feature = "servo")]
+        {
+            apply!(FontSize);
+            if has_lang || has_font_family {
+                self.recompute_keyword_font_size_if_needed(context);
+            }
         }
 
         // Compute the rest of the first-available-font-affecting properties.
         apply!(FontWeight);
         apply!(FontStretch);
         apply!(FontStyle);
+        #[cfg(feature = "gecko")]
         apply!(FontSizeAdjust);
 
-        apply!(ColorScheme);
+        #[cfg(feature = "gecko")]
         apply!(ForcedColorAdjust);
-
-        // Compute the line height.
+        // color-scheme needs to be after forced-color-adjust, since it's one of the "skipped in
+        // forced-colors-mode" properties.
+        if apply!(ColorScheme) {
+            context.builder.color_scheme = context.builder.get_inherited_ui().color_scheme_bits();
+        }
         apply!(LineHeight);
     }
 
@@ -938,17 +964,6 @@ impl<'b> Cascade<'b> {
         (CASCADE_PROPERTY[longhand_id as usize])(&declaration, context);
     }
 
-    fn compute_zoom(&self, context: &mut computed::Context) {
-        context.builder.effective_zoom = context
-            .builder
-            .inherited_effective_zoom()
-            .compute_effective(context.builder.specified_zoom());
-    }
-
-    fn compute_writing_mode(&self, context: &mut computed::Context) {
-        context.builder.writing_mode = WritingMode::new(context.builder.get_inherited_box())
-    }
-
     fn compute_visited_style_if_needed<E>(
         &self,
         context: &mut computed::Context,
@@ -1022,6 +1037,12 @@ impl<'b> Cascade<'b> {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_FONT_FAMILY);
         }
 
+        if self.author_specified.contains_any(LonghandIdSet::margin_properties()) &&
+           self.author_specified.contains(LonghandId::FontSize)
+        {
+            builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_MARGIN_AND_FONT_SIZE);
+        }
+
         if self.author_specified.contains(LonghandId::Color) {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_TEXT_COLOR);
         }
@@ -1034,6 +1055,7 @@ impl<'b> Cascade<'b> {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_WORD_SPACING);
         }
 
+        #[cfg(feature = "gecko")]
         if self
             .author_specified
             .contains(LonghandId::FontSynthesisWeight)
@@ -1041,6 +1063,7 @@ impl<'b> Cascade<'b> {
             builder.add_flags(ComputedValueFlags::HAS_AUTHOR_SPECIFIED_FONT_SYNTHESIS_WEIGHT);
         }
 
+        #[cfg(feature = "gecko")]
         if self
             .author_specified
             .contains(LonghandId::FontSynthesisStyle)
@@ -1176,7 +1199,6 @@ impl<'b> Cascade<'b> {
     }
 
     /// Some keyword sizes depend on the font family and language.
-    #[cfg(feature = "gecko")]
     fn recompute_keyword_font_size_if_needed(&self, context: &mut computed::Context) {
         use crate::values::computed::ToComputedValue;
 
@@ -1195,6 +1217,7 @@ impl<'b> Cascade<'b> {
                 },
             };
 
+            #[cfg(feature = "gecko")]
             if font.mScriptUnconstrainedSize == new_size.computed_size {
                 return;
             }
@@ -1247,7 +1270,7 @@ impl<'b> Cascade<'b> {
         );
         debug_assert!(
             !text_scale.text_zoom_enabled(),
-            "We only ever disable text zoom (in svg:text), never enable it"
+            "We only ever disable text zoom never enable it"
         );
         let device = builder.device;
         builder.mutate_font().unzoom_fonts(device);
@@ -1257,9 +1280,8 @@ impl<'b> Cascade<'b> {
         debug_assert!(self.seen.contains(LonghandId::Zoom));
         // NOTE(emilio): Intentionally not using the effective zoom here, since all the inherited
         // zooms are already applied.
-        let zoom = builder.get_box().clone_zoom();
         let old_size = builder.get_font().clone_font_size();
-        let new_size = old_size.zoom(zoom);
+        let new_size = old_size.zoom(builder.resolved_specified_zoom());
         if old_size == new_size {
             return;
         }
@@ -1310,7 +1332,7 @@ impl<'b> Cascade<'b> {
                 return s;
             }
             if b < a {
-                mem::swap(&mut a, &mut b);
+                std::mem::swap(&mut a, &mut b);
                 invert_scale_factor = true;
             }
             let mut e = b - a;
@@ -1333,6 +1355,8 @@ impl<'b> Cascade<'b> {
         }
 
         let (new_size, new_unconstrained_size) = {
+            use crate::values::specified::font::QueryFontMetricsFlags;
+
             let builder = &context.builder;
             let font = builder.get_font();
             let parent_font = builder.get_parent_font();
@@ -1354,7 +1378,7 @@ impl<'b> Cascade<'b> {
                 let font_metrics = context.query_font_metrics(
                     FontBaseSize::InheritedStyle,
                     FontMetricsOrientation::Horizontal,
-                    /* retrieve_math_scales = */ true,
+                    QueryFontMetricsFlags::NEEDS_MATH_SCALES,
                 );
                 scale_factor_for_math_depth_change(
                     parent_font.mMathDepth as i32,
