@@ -10,7 +10,6 @@ ${helpers.predefined_type(
     "computed_value::T::currentcolor()",
     engines="gecko servo",
     initial_specified_value="specified::Color::currentcolor()",
-    animation_value_type="AnimatedColor",
     ignored_when_colors_disabled=True,
     spec="https://drafts.csswg.org/css-ui/#propdef-outline-color",
     affects="paint",
@@ -22,7 +21,7 @@ ${helpers.predefined_type(
     "computed::OutlineStyle::none()",
     engines="gecko servo",
     initial_specified_value="specified::OutlineStyle::none()",
-    animation_value_type="discrete",
+    animation_type="discrete",
     spec="https://drafts.csswg.org/css-ui/#propdef-outline-style",
     affects="overflow",
 )}
@@ -33,7 +32,6 @@ ${helpers.predefined_type(
     "app_units::Au::from_px(3)",
     engines="gecko servo",
     initial_specified_value="specified::BorderSideWidth::medium()",
-    animation_value_type="NonNegativeLength",
     spec="https://drafts.csswg.org/css-ui/#propdef-outline-width",
     affects="overflow",
 )}
@@ -43,7 +41,6 @@ ${helpers.predefined_type(
     "Length",
     "crate::values::computed::Length::new(0.)",
     engines="gecko servo",
-    animation_value_type="ComputedValue",
     spec="https://drafts.csswg.org/css-ui/#propdef-outline-offset",
     affects="overflow",
 )}
