@@ -1,16 +1,18 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 //! A centralized set of stylesheets for a document.
 
-use dom::TElement;
-use invalidation::stylesheets::StylesheetInvalidationSet;
-use media_queries::Device;
-use selector_parser::SnapshotMap;
-use shared_lock::SharedRwLockReadGuard;
+use crate::dom::TElement;
+use crate::invalidation::stylesheets::{RuleChangeKind, StylesheetInvalidationSet};
+use crate::media_queries::Device;
+use crate::selector_parser::SnapshotMap;
+use crate::shared_lock::SharedRwLockReadGuard;
+use crate::stylesheets::{
+    CssRule, Origin, OriginSet, OriginSetIterator, PerOrigin, StylesheetInDocument,
+};
 use std::{mem, slice};
-use stylesheets::{Origin, OriginSet, OriginSetIterator, PerOrigin, StylesheetInDocument};
 
 /// Entry for a StylesheetSet.
 #[derive(MallocSizeOf)]
@@ -30,7 +32,10 @@ where
     S: StylesheetInDocument + PartialEq + 'static,
 {
     fn new(sheet: S) -> Self {
-        Self { sheet, committed: false }
+        Self {
+            sheet,
+            committed: false,
+        }
     }
 }
 
@@ -48,7 +53,6 @@ where
     }
 }
 
-
 impl<'a, S> Iterator for StylesheetCollectionIterator<'a, S>
 where
     S: StylesheetInDocument + PartialEq + 'static,
@@ -86,14 +90,16 @@ where
             if self.current.is_none() {
                 let next_origin = self.origins.next()?;
 
-                self.current =
-                    Some((next_origin, self.collections.borrow_for_origin(&next_origin).iter()));
+                self.current = Some((
+                    next_origin,
+                    self.collections.borrow_for_origin(&next_origin).iter(),
+                ));
             }
 
             {
                 let (origin, ref mut iter) = *self.current.as_mut().unwrap();
                 if let Some(s) = iter.next() {
-                    return Some((s, origin))
+                    return Some((s, origin));
                 }
             }
 
@@ -176,9 +182,11 @@ where
 /// appropriate stylesheets that need work.
 pub struct SheetCollectionFlusher<'a, S>
 where
-    S: StylesheetInDocument + PartialEq + 'static
+    S: StylesheetInDocument + PartialEq + 'static,
 {
-    iter: slice::IterMut<'a, StylesheetSetEntry<S>>,
+    // TODO: This can be made an iterator again once
+    // https://github.com/rust-lang/rust/pull/82771 lands on stable.
+    entries: &'a mut [StylesheetSetEntry<S>],
     validity: DataValidity,
     dirty: bool,
 }
@@ -198,34 +206,42 @@ where
     pub fn data_validity(&self) -> DataValidity {
         self.validity
     }
+
+    /// Returns an iterator over the remaining list of sheets to consume.
+    pub fn sheets<'b>(&'b self) -> impl Iterator<Item = &'b S> {
+        self.entries.iter().map(|entry| &entry.sheet)
+    }
 }
 
-impl<'a, S> Iterator for SheetCollectionFlusher<'a, S>
+impl<'a, S> SheetCollectionFlusher<'a, S>
 where
     S: StylesheetInDocument + PartialEq + 'static,
 {
-    type Item = (&'a S, SheetRebuildKind);
-
-    fn next(&mut self) -> Option<Self::Item> {
-        use std::mem;
-
-        loop {
-            let potential_sheet = self.iter.next()?;
-
+    /// Iterates over all sheets and values that we have to invalidate.
+    ///
+    /// TODO(emilio): This would be nicer as an iterator but we can't do that
+    /// until https://github.com/rust-lang/rust/pull/82771 stabilizes.
+    ///
+    /// Since we don't have a good use-case for partial iteration, this does the
+    /// trick for now.
+    pub fn each(self, mut callback: impl FnMut(usize, &S, SheetRebuildKind) -> bool) {
+        for (index, potential_sheet) in self.entries.iter_mut().enumerate() {
             let committed = mem::replace(&mut potential_sheet.committed, true);
-            if !committed {
+            let rebuild_kind = if !committed {
                 // If the sheet was uncommitted, we need to do a full rebuild
                 // anyway.
-                return Some((&potential_sheet.sheet, SheetRebuildKind::Full))
-            }
-
-            let rebuild_kind = match self.validity {
-                DataValidity::Valid => continue,
-                DataValidity::CascadeInvalid => SheetRebuildKind::CascadeOnly,
-                DataValidity::FullyInvalid => SheetRebuildKind::Full,
+                SheetRebuildKind::Full
+            } else {
+                match self.validity {
+                    DataValidity::Valid => continue,
+                    DataValidity::CascadeInvalid => SheetRebuildKind::CascadeOnly,
+                    DataValidity::FullyInvalid => SheetRebuildKind::Full,
+                }
             };
 
-            return Some((&potential_sheet.sheet, rebuild_kind));
+            if !callback(index, &potential_sheet.sheet, rebuild_kind) {
+                return;
+            }
         }
     }
 }
@@ -263,7 +279,7 @@ where
         Self {
             entries: vec![],
             data_validity: DataValidity::Valid,
-            dirty: true,
+            dirty: false,
         }
     }
 }
@@ -282,10 +298,13 @@ where
         self.entries.get(index).map(|e| &e.sheet)
     }
 
+    fn find_sheet_index(&self, sheet: &S) -> Option<usize> {
+        let rev_pos = self.entries.iter().rev().position(|entry| entry.sheet == *sheet);
+        rev_pos.map(|i| self.entries.len() - i - 1)
+    }
+
     fn remove(&mut self, sheet: &S) {
-        let index = self.entries.iter().position(|entry| {
-            entry.sheet == *sheet
-        });
+        let index = self.find_sheet_index(sheet);
         if cfg!(feature = "gecko") && index.is_none() {
             // FIXME(emilio): Make Gecko's PresShell::AddUserSheet not suck.
             return;
@@ -294,7 +313,9 @@ where
         // Removing sheets makes us tear down the whole cascade and invalidation
         // data, but only if the sheet has been involved in at least one flush.
         // Checking whether the sheet has been committed allows us to avoid
-        // rebuilding the world when sites quickly append and remove a stylesheet.
+        // rebuilding the world when sites quickly append and remove a
+        // stylesheet.
+        //
         // See bug 1434756.
         if sheet.committed {
             self.set_data_validity_at_least(DataValidity::FullyInvalid);
@@ -322,9 +343,8 @@ where
     fn insert_before(&mut self, sheet: S, before_sheet: &S) {
         debug_assert!(!self.contains(&sheet));
 
-        let index = self.entries.iter().position(|entry| {
-            entry.sheet == *before_sheet
-        }).expect("`before_sheet` stylesheet not found");
+        let index = self.find_sheet_index(before_sheet)
+            .expect("`before_sheet` stylesheet not found");
 
         // Inserting stylesheets somewhere but at the end changes the validity
         // of the cascade data, but not the invalidation data.
@@ -341,14 +361,6 @@ where
         self.data_validity = cmp::max(validity, self.data_validity);
     }
 
-    fn prepend(&mut self, sheet: S) {
-        debug_assert!(!self.contains(&sheet));
-        // Inserting stylesheets somewhere but at the end changes the validity
-        // of the cascade data, but not the invalidation data.
-        self.set_data_validity_at_least(DataValidity::CascadeInvalid);
-        self.entries.insert(0, StylesheetSetEntry::new(sheet));
-    }
-
     /// Returns an iterator over the current list of stylesheets.
     fn iter(&self) -> StylesheetCollectionIterator<S> {
         StylesheetCollectionIterator(self.entries.iter())
@@ -359,7 +371,7 @@ where
         let validity = mem::replace(&mut self.data_validity, DataValidity::Valid);
 
         SheetCollectionFlusher {
-            iter: self.entries.iter_mut(),
+            entries: &mut self.entries,
             dirty,
             validity,
         }
@@ -394,7 +406,8 @@ macro_rules! sheet_set_methods {
             guard: &SharedRwLockReadGuard,
         ) {
             if let Some(device) = device {
-                self.invalidations.collect_invalidations_for(device, sheet, guard);
+                self.invalidations
+                    .collect_invalidations_for(device, sheet, guard);
             }
         }
 
@@ -409,24 +422,10 @@ macro_rules! sheet_set_methods {
         ) {
             debug!(concat!($set_name, "::append_stylesheet"));
             self.collect_invalidations_for(device, &sheet, guard);
-            let collection = self.collection_for(&sheet, guard);
+            let collection = self.collection_for(&sheet);
             collection.append(sheet);
         }
 
-        /// Prepend a new stylesheet to the current set.
-        pub fn prepend_stylesheet(
-            &mut self,
-            device: Option<&Device>,
-            sheet: S,
-            guard: &SharedRwLockReadGuard,
-        ) {
-            debug!(concat!($set_name, "::prepend_stylesheet"));
-            self.collect_invalidations_for(device, &sheet, guard);
-
-            let collection = self.collection_for(&sheet, guard);
-            collection.prepend(sheet);
-        }
-
         /// Insert a given stylesheet before another stylesheet in the document.
         pub fn insert_stylesheet_before(
             &mut self,
@@ -438,7 +437,7 @@ macro_rules! sheet_set_methods {
             debug!(concat!($set_name, "::insert_stylesheet_before"));
             self.collect_invalidations_for(device, &sheet, guard);
 
-            let collection = self.collection_for(&sheet, guard);
+            let collection = self.collection_for(&sheet);
             collection.insert_before(sheet, &before_sheet);
         }
 
@@ -452,10 +451,60 @@ macro_rules! sheet_set_methods {
             debug!(concat!($set_name, "::remove_stylesheet"));
             self.collect_invalidations_for(device, &sheet, guard);
 
-            let collection = self.collection_for(&sheet, guard);
+            let collection = self.collection_for(&sheet);
             collection.remove(&sheet)
         }
-    }
+
+        /// Notify the set that a rule from a given stylesheet has changed
+        /// somehow.
+        pub fn rule_changed(
+            &mut self,
+            device: Option<&Device>,
+            sheet: &S,
+            rule: &CssRule,
+            guard: &SharedRwLockReadGuard,
+            change_kind: RuleChangeKind,
+        ) {
+            if let Some(device) = device {
+                let quirks_mode = device.quirks_mode();
+                self.invalidations.rule_changed(
+                    sheet,
+                    rule,
+                    guard,
+                    device,
+                    quirks_mode,
+                    change_kind,
+                );
+            }
+
+            let validity = match change_kind {
+                // Insertion / Removals need to rebuild both the cascade and
+                // invalidation data. For generic changes this is conservative,
+                // could be optimized on a per-case basis.
+                RuleChangeKind::Generic | RuleChangeKind::Insertion | RuleChangeKind::Removal => {
+                    DataValidity::FullyInvalid
+                },
+                // TODO(emilio): This, in theory, doesn't need to invalidate
+                // style data, if the rule we're modifying is actually in the
+                // CascadeData already.
+                //
+                // But this is actually a bit tricky to prove, because when we
+                // copy-on-write a stylesheet we don't bother doing a rebuild,
+                // so we may still have rules from the original stylesheet
+                // instead of the cloned one that we're modifying. So don't
+                // bother for now and unconditionally rebuild, it's no worse
+                // than what we were already doing anyway.
+                //
+                // Maybe we could record whether we saw a clone in this flush,
+                // and if so do the conservative thing, otherwise just
+                // early-return.
+                RuleChangeKind::StyleRuleDeclarations => DataValidity::FullyInvalid,
+            };
+
+            let collection = self.collection_for(&sheet);
+            collection.set_data_validity_at_least(validity);
+        }
+    };
 }
 
 impl<S> DocumentStylesheetSet<S>
@@ -470,12 +519,8 @@ where
         }
     }
 
-    fn collection_for(
-        &mut self,
-        sheet: &S,
-        guard: &SharedRwLockReadGuard,
-    ) -> &mut SheetCollection<S> {
-        let origin = sheet.contents(guard).origin;
+    fn collection_for(&mut self, sheet: &S) -> &mut SheetCollection<S> {
+        let origin = sheet.contents().origin;
         self.collections.borrow_mut_for_origin(&origin)
     }
 
@@ -483,17 +528,29 @@ where
 
     /// Returns the number of stylesheets in the set.
     pub fn len(&self) -> usize {
-        self.collections.iter_origins().fold(0, |s, (item, _)| s + item.len())
+        self.collections
+            .iter_origins()
+            .fold(0, |s, (item, _)| s + item.len())
+    }
+
+    /// Returns the count of stylesheets for a given origin.
+    #[inline]
+    pub fn sheet_count(&self, origin: Origin) -> usize {
+        self.collections.borrow_for_origin(&origin).len()
     }
 
     /// Returns the `index`th stylesheet in the set for the given origin.
+    #[inline]
     pub fn get(&self, origin: Origin, index: usize) -> Option<&S> {
         self.collections.borrow_for_origin(&origin).get(index)
     }
 
     /// Returns whether the given set has changed from the last flush.
     pub fn has_changed(&self) -> bool {
-        self.collections.iter_origins().any(|(collection, _)| collection.dirty)
+        !self.invalidations.is_empty() ||
+            self.collections
+                .iter_origins()
+                .any(|(collection, _)| collection.dirty)
     }
 
     /// Flush the current set, unmarking it as dirty, and returns a
@@ -508,8 +565,7 @@ where
     {
         debug!("DocumentStylesheetSet::flush");
 
-        let had_invalidations =
-            self.invalidations.flush(document_element, snapshots);
+        let had_invalidations = self.invalidations.flush(document_element, snapshots);
 
         DocumentStylesheetFlusher {
             collections: &mut self.collections,
@@ -537,7 +593,7 @@ where
     /// Return an iterator over the flattened view of all the stylesheets.
     pub fn iter(&self) -> StylesheetIterator<S> {
         StylesheetIterator {
-            origins: OriginSet::all().iter(),
+            origins: OriginSet::all().iter_origins(),
             collections: &self.collections,
             current: None,
         }
@@ -547,7 +603,7 @@ where
     /// something external may have invalidated it.
     pub fn force_dirty(&mut self, origins: OriginSet) {
         self.invalidations.invalidate_fully();
-        for origin in origins.iter() {
+        for origin in origins.iter_origins() {
             // We don't know what happened, assume the worse.
             self.collections
                 .borrow_mut_for_origin(&origin)
@@ -556,7 +612,7 @@ where
     }
 }
 
-/// The set of stylesheets effective for a given XBL binding or Shadow Root.
+/// The set of stylesheets effective for a given Shadow Root.
 #[derive(MallocSizeOf)]
 pub struct AuthorStylesheetSet<S>
 where
@@ -602,11 +658,17 @@ where
         self.collection.len() == 0
     }
 
-    fn collection_for(
-        &mut self,
-        _sheet: &S,
-        _guard: &SharedRwLockReadGuard,
-    ) -> &mut SheetCollection<S> {
+    /// Returns the `index`th stylesheet in the collection of author styles if present.
+    pub fn get(&self, index: usize) -> Option<&S> {
+        self.collection.get(index)
+    }
+
+    /// Returns the number of author stylesheets.
+    pub fn len(&self) -> usize {
+        self.collection.len()
+    }
+
+    fn collection_for(&mut self, _sheet: &S) -> &mut SheetCollection<S> {
         &mut self.collection
     }
 
@@ -620,7 +682,8 @@ where
     /// Mark the sheet set dirty, as appropriate.
     pub fn force_dirty(&mut self) {
         self.invalidations.invalidate_fully();
-        self.collection.set_data_validity_at_least(DataValidity::FullyInvalid);
+        self.collection
+            .set_data_validity_at_least(DataValidity::FullyInvalid);
     }
 
     /// Flush the stylesheets for this author set.
