@@ -1,30 +1,38 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 //! Keyframes: https://drafts.csswg.org/css-animations/#keyframes
 
-use cssparser::{AtRuleParser, Parser, QualifiedRuleParser, RuleListParser, ParserInput, CowRcStr};
-use cssparser::{DeclarationListParser, DeclarationParser, parse_one_rule, SourceLocation, Token};
-use error_reporting::{NullReporter, ContextualParseError, ParseErrorReporter};
-use parser::{ParserContext, ParserErrorContext};
-use properties::{DeclarationSource, Importance, PropertyDeclaration, PropertyDeclarationBlock, PropertyId};
-use properties::{PropertyDeclarationId, LonghandId, SourcePropertyDeclaration};
-use properties::LonghandIdSet;
-use properties::longhands::transition_timing_function::single_value::SpecifiedValue as SpecifiedTimingFunction;
+use crate::error_reporting::ContextualParseError;
+use crate::parser::ParserContext;
+use crate::properties::{
+    longhands::{
+        animation_composition::single_value::SpecifiedValue as SpecifiedComposition,
+        transition_timing_function::single_value::SpecifiedValue as SpecifiedTimingFunction,
+    },
+    parse_property_declaration_list, LonghandId, PropertyDeclaration, PropertyDeclarationBlock,
+    PropertyDeclarationId, PropertyDeclarationIdSet,
+};
+use crate::shared_lock::{DeepCloneWithLock, SharedRwLock, SharedRwLockReadGuard};
+use crate::shared_lock::{Locked, ToCssWithGuard};
+use crate::str::CssStringWriter;
+use crate::stylesheets::rule_parser::VendorPrefix;
+use crate::stylesheets::{CssRuleType, StylesheetContents};
+use crate::values::{serialize_percentage, KeyframesName};
+use cssparser::{
+    parse_one_rule, AtRuleParser, DeclarationParser, Parser, ParserInput, ParserState,
+    QualifiedRuleParser, RuleBodyItemParser, RuleBodyParser, SourceLocation, Token,
+};
 use servo_arc::Arc;
-use shared_lock::{DeepCloneParams, DeepCloneWithLock, SharedRwLock, SharedRwLockReadGuard, Locked, ToCssWithGuard};
+use std::borrow::Cow;
 use std::fmt::{self, Write};
-use str::CssStringWriter;
 use style_traits::{CssWriter, ParseError, ParsingMode, StyleParseErrorKind, ToCss};
-use stylesheets::{CssRuleType, StylesheetContents};
-use stylesheets::rule_parser::VendorPrefix;
-use values::{KeyframesName, serialize_percentage};
 
 /// A [`@keyframes`][keyframes] rule.
 ///
 /// [keyframes]: https://drafts.csswg.org/css-animations/#keyframes
-#[derive(Debug)]
+#[derive(Debug, ToShmem)]
 pub struct KeyframesRule {
     /// The name of the current animation.
     pub name: KeyframesName,
@@ -76,15 +84,16 @@ impl DeepCloneWithLock for KeyframesRule {
         &self,
         lock: &SharedRwLock,
         guard: &SharedRwLockReadGuard,
-        params: &DeepCloneParams,
     ) -> Self {
         KeyframesRule {
             name: self.name.clone(),
-            keyframes: self.keyframes.iter()
+            keyframes: self
+                .keyframes
+                .iter()
                 .map(|x| {
-                    Arc::new(lock.wrap(
-                        x.read_with(guard).deep_clone_with_lock(lock, guard, params)
-                    ))
+                    Arc::new(
+                        lock.wrap(x.read_with(guard).deep_clone_with_lock(lock, guard)),
+                    )
                 })
                 .collect(),
             vendor_prefix: self.vendor_prefix.clone(),
@@ -95,7 +104,7 @@ impl DeepCloneWithLock for KeyframesRule {
 
 /// A number from 0 to 1, indicating the percentage of the animation when this
 /// keyframe should run.
-#[derive(Clone, Copy, Debug, MallocSizeOf, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, MallocSizeOf, PartialEq, PartialOrd, ToShmem)]
 pub struct KeyframePercentage(pub f32);
 
 impl ::std::cmp::Ord for KeyframePercentage {
@@ -106,7 +115,7 @@ impl ::std::cmp::Ord for KeyframePercentage {
     }
 }
 
-impl ::std::cmp::Eq for KeyframePercentage { }
+impl ::std::cmp::Eq for KeyframePercentage {}
 
 impl ToCss for KeyframePercentage {
     fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
@@ -134,20 +143,19 @@ impl KeyframePercentage {
             Token::Ident(ref identifier) if identifier.as_ref().eq_ignore_ascii_case("to") => {
                 Ok(KeyframePercentage::new(1.))
             },
-            Token::Percentage { unit_value: percentage, .. } if percentage >= 0. && percentage <= 1. => {
-                Ok(KeyframePercentage::new(percentage))
-            },
-            _ => {
-                Err(input.new_unexpected_token_error(token))
-            },
+            Token::Percentage {
+                unit_value: percentage,
+                ..
+            } if percentage >= 0. && percentage <= 1. => Ok(KeyframePercentage::new(percentage)),
+            _ => Err(input.new_unexpected_token_error(token)),
         }
     }
 }
 
 /// A keyframes selector is a list of percentages or from/to symbols, which are
 /// converted at parse time to percentages.
+#[derive(Clone, Debug, Eq, PartialEq, ToCss, ToShmem)]
 #[css(comma)]
-#[derive(Clone, Debug, Eq, PartialEq, ToCss)]
 pub struct KeyframeSelector(#[css(iterable)] Vec<KeyframePercentage>);
 
 impl KeyframeSelector {
@@ -164,13 +172,14 @@ impl KeyframeSelector {
 
     /// Parse a keyframe selector from CSS input.
     pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
-        input.parse_comma_separated(KeyframePercentage::parse)
-             .map(KeyframeSelector)
+        input
+            .parse_comma_separated(KeyframePercentage::parse)
+            .map(KeyframeSelector)
     }
 }
 
 /// A keyframe.
-#[derive(Debug)]
+#[derive(Debug, ToShmem)]
 pub struct Keyframe {
     /// The selector this keyframe was specified from.
     pub selector: KeyframeSelector,
@@ -203,26 +212,23 @@ impl Keyframe {
         lock: &SharedRwLock,
     ) -> Result<Arc<Locked<Self>>, ParseError<'i>> {
         let url_data = parent_stylesheet_contents.url_data.read();
-        let error_reporter = NullReporter;
         let namespaces = parent_stylesheet_contents.namespaces.read();
         let mut context = ParserContext::new(
             parent_stylesheet_contents.origin,
             &url_data,
             Some(CssRuleType::Keyframe),
             ParsingMode::DEFAULT,
-            parent_stylesheet_contents.quirks_mode
+            parent_stylesheet_contents.quirks_mode,
+            Cow::Borrowed(&*namespaces),
+            None,
+            None,
         );
-        let error_context = ParserErrorContext { error_reporter: &error_reporter };
-        context.namespaces = Some(&*namespaces);
         let mut input = ParserInput::new(css);
         let mut input = Parser::new(&mut input);
 
-        let mut declarations = SourcePropertyDeclaration::new();
         let mut rule_parser = KeyframeListParser {
-            context: &context,
-            error_context: &error_context,
+            context: &mut context,
             shared_lock: &lock,
-            declarations: &mut declarations,
         };
         parse_one_rule(&mut input, &mut rule_parser)
     }
@@ -234,7 +240,6 @@ impl DeepCloneWithLock for Keyframe {
         &self,
         lock: &SharedRwLock,
         guard: &SharedRwLockReadGuard,
-        _params: &DeepCloneParams,
     ) -> Keyframe {
         Keyframe {
             selector: self.selector.clone(),
@@ -249,15 +254,17 @@ impl DeepCloneWithLock for Keyframe {
 /// declarations to apply.
 ///
 /// TODO: Find a better name for this?
-#[derive(Debug, MallocSizeOf)]
+#[derive(Clone, Debug, MallocSizeOf)]
 pub enum KeyframesStepValue {
     /// A step formed by a declaration block specified by the CSS.
     Declarations {
         /// The declaration block per se.
-        #[cfg_attr(feature = "gecko",
-                   ignore_malloc_size_of = "XXX: Primary ref, measure if DMD says it's worthwhile")]
+        #[cfg_attr(
+            feature = "gecko",
+            ignore_malloc_size_of = "XXX: Primary ref, measure if DMD says it's worthwhile"
+        )]
         #[cfg_attr(feature = "servo", ignore_malloc_size_of = "Arc")]
-        block: Arc<Locked<PropertyDeclarationBlock>>
+        block: Arc<Locked<PropertyDeclarationBlock>>,
     },
     /// A synthetic step computed from the current computed values at the time
     /// of the animation.
@@ -265,82 +272,140 @@ pub enum KeyframesStepValue {
 }
 
 /// A single step from a keyframe animation.
-#[derive(Debug, MallocSizeOf)]
+#[derive(Clone, Debug, MallocSizeOf)]
 pub struct KeyframesStep {
     /// The percentage of the animation duration when this step starts.
     pub start_percentage: KeyframePercentage,
     /// Declarations that will determine the final style during the step, or
     /// `ComputedValues` if this is an autogenerated step.
     pub value: KeyframesStepValue,
-    /// Wether a animation-timing-function declaration exists in the list of
+    /// Whether an animation-timing-function declaration exists in the list of
     /// declarations.
     ///
     /// This is used to know when to override the keyframe animation style.
     pub declared_timing_function: bool,
+    /// Whether an animation-composition declaration exists in the list of
+    /// declarations.
+    ///
+    /// This is used to know when to override the keyframe animation style.
+    pub declared_composition: bool,
 }
 
 impl KeyframesStep {
     #[inline]
-    fn new(percentage: KeyframePercentage,
-           value: KeyframesStepValue,
-           guard: &SharedRwLockReadGuard) -> Self {
-        let declared_timing_function = match value {
-            KeyframesStepValue::Declarations { ref block } => {
-                block.read_with(guard).declarations().iter().any(|prop_decl| {
-                    match *prop_decl {
-                        PropertyDeclaration::AnimationTimingFunction(..) => true,
-                        _ => false,
-                    }
-                })
+    fn new(
+        start_percentage: KeyframePercentage,
+        value: KeyframesStepValue,
+        guard: &SharedRwLockReadGuard,
+    ) -> Self {
+        let mut declared_timing_function = false;
+        let mut declared_composition = false;
+        if let KeyframesStepValue::Declarations { ref block } = value {
+            for prop_decl in block.read_with(guard).declarations().iter() {
+                match *prop_decl {
+                    PropertyDeclaration::AnimationTimingFunction(..) => {
+                        declared_timing_function = true;
+                    },
+                    PropertyDeclaration::AnimationComposition(..) => {
+                        declared_composition = true;
+                    },
+                    _ => continue,
+                }
+                // Don't need to continue the loop if both are found.
+                if declared_timing_function && declared_composition {
+                    break;
+                }
             }
-            _ => false,
-        };
+        }
 
         KeyframesStep {
-            start_percentage: percentage,
-            value: value,
-            declared_timing_function: declared_timing_function,
+            start_percentage,
+            value,
+            declared_timing_function,
+            declared_composition,
         }
     }
 
-    /// Return specified TransitionTimingFunction if this KeyframesSteps has 'animation-timing-function'.
-    pub fn get_animation_timing_function(&self, guard: &SharedRwLockReadGuard)
-                                         -> Option<SpecifiedTimingFunction> {
-        if !self.declared_timing_function {
-            return None;
-        }
+    /// Return specified PropertyDeclaration.
+    #[inline]
+    fn get_declared_property<'a>(
+        &'a self,
+        guard: &'a SharedRwLockReadGuard,
+        property: LonghandId,
+    ) -> Option<&'a PropertyDeclaration> {
         match self.value {
             KeyframesStepValue::Declarations { ref block } => {
                 let guard = block.read_with(guard);
-                let (declaration, _) =
-                    guard.get(PropertyDeclarationId::Longhand(LonghandId::AnimationTimingFunction)).unwrap();
+                let (declaration, _) = guard
+                    .get(PropertyDeclarationId::Longhand(property))
+                    .unwrap();
                 match *declaration {
-                    PropertyDeclaration::AnimationTimingFunction(ref value) => {
-                        // Use the first value.
-                        Some(value.0[0])
-                    },
                     PropertyDeclaration::CSSWideKeyword(..) => None,
+                    // FIXME: Bug 1710735: Support css variable in @keyframes rule.
                     PropertyDeclaration::WithVariables(..) => None,
-                    _ => panic!(),
+                    _ => Some(declaration),
                 }
             },
             KeyframesStepValue::ComputedValues => {
-                panic!("Shouldn't happen to set animation-timing-function in missing keyframes")
+                panic!("Shouldn't happen to set this property in missing keyframes")
             },
         }
     }
+
+    /// Return specified TransitionTimingFunction if this KeyframesSteps has
+    /// 'animation-timing-function'.
+    pub fn get_animation_timing_function(
+        &self,
+        guard: &SharedRwLockReadGuard,
+    ) -> Option<SpecifiedTimingFunction> {
+        if !self.declared_timing_function {
+            return None;
+        }
+
+        self.get_declared_property(guard, LonghandId::AnimationTimingFunction)
+            .map(|decl| {
+                match *decl {
+                    PropertyDeclaration::AnimationTimingFunction(ref value) => {
+                        // Use the first value
+                        value.0[0].clone()
+                    },
+                    _ => unreachable!("Unexpected PropertyDeclaration"),
+                }
+            })
+    }
+
+    /// Return CompositeOperation if this KeyframesSteps has 'animation-composition'.
+    pub fn get_animation_composition(
+        &self,
+        guard: &SharedRwLockReadGuard,
+    ) -> Option<SpecifiedComposition> {
+        if !self.declared_composition {
+            return None;
+        }
+
+        self.get_declared_property(guard, LonghandId::AnimationComposition)
+            .map(|decl| {
+                match *decl {
+                    PropertyDeclaration::AnimationComposition(ref value) => {
+                        // Use the first value
+                        value.0[0].clone()
+                    },
+                    _ => unreachable!("Unexpected PropertyDeclaration"),
+                }
+            })
+    }
 }
 
 /// This structure represents a list of animation steps computed from the list
 /// of keyframes, in order.
 ///
 /// It only takes into account animable properties.
-#[derive(Debug, MallocSizeOf)]
+#[derive(Clone, Debug, MallocSizeOf)]
 pub struct KeyframesAnimation {
     /// The difference steps of the animation.
     pub steps: Vec<KeyframesStep>,
     /// The properties that change in this animation.
-    pub properties_changed: LonghandIdSet,
+    pub properties_changed: PropertyDeclarationIdSet,
     /// Vendor prefix type the @keyframes has.
     pub vendor_prefix: Option<VendorPrefix>,
 }
@@ -348,9 +413,9 @@ pub struct KeyframesAnimation {
 /// Get all the animated properties in a keyframes animation.
 fn get_animated_properties(
     keyframes: &[Arc<Locked<Keyframe>>],
-    guard: &SharedRwLockReadGuard
-) -> LonghandIdSet {
-    let mut ret = LonghandIdSet::new();
+    guard: &SharedRwLockReadGuard,
+) -> PropertyDeclarationIdSet {
+    let mut ret = PropertyDeclarationIdSet::default();
     // NB: declarations are already deduplicated, so we don't have to check for
     // it here.
     for keyframe in keyframes {
@@ -363,20 +428,17 @@ fn get_animated_properties(
         // be properties with !important in keyframe rules here.
         // See the spec issue https://github.com/w3c/csswg-drafts/issues/1824
         for declaration in block.normal_declaration_iter() {
-            let longhand_id = match declaration.id() {
-                PropertyDeclarationId::Longhand(id) => id,
-                _ => continue,
-            };
+            let declaration_id = declaration.id();
 
-            if longhand_id == LonghandId::Display {
+            if declaration_id == PropertyDeclarationId::Longhand(LonghandId::Display) {
                 continue;
             }
 
-            if !longhand_id.is_animatable() {
+            if !declaration_id.is_animatable() {
                 continue;
             }
 
-            ret.insert(longhand_id);
+            ret.insert(declaration_id);
         }
     }
 
@@ -399,7 +461,7 @@ impl KeyframesAnimation {
     ) -> Self {
         let mut result = KeyframesAnimation {
             steps: vec![],
-            properties_changed: LonghandIdSet::new(),
+            properties_changed: PropertyDeclarationIdSet::default(),
             vendor_prefix,
         };
 
@@ -415,9 +477,13 @@ impl KeyframesAnimation {
         for keyframe in keyframes {
             let keyframe = keyframe.read_with(&guard);
             for percentage in keyframe.selector.0.iter() {
-                result.steps.push(KeyframesStep::new(*percentage, KeyframesStepValue::Declarations {
-                    block: keyframe.block.clone(),
-                }, guard));
+                result.steps.push(KeyframesStep::new(
+                    *percentage,
+                    KeyframesStepValue::Declarations {
+                        block: keyframe.block.clone(),
+                    },
+                    guard,
+                ));
             }
         }
 
@@ -426,15 +492,22 @@ impl KeyframesAnimation {
 
         // Prepend autogenerated keyframes if appropriate.
         if result.steps[0].start_percentage.0 != 0. {
-            result.steps.insert(0, KeyframesStep::new(KeyframePercentage::new(0.),
-                                                      KeyframesStepValue::ComputedValues,
-                                                      guard));
+            result.steps.insert(
+                0,
+                KeyframesStep::new(
+                    KeyframePercentage::new(0.),
+                    KeyframesStepValue::ComputedValues,
+                    guard,
+                ),
+            );
         }
 
         if result.steps.last().unwrap().start_percentage.0 != 1. {
-            result.steps.push(KeyframesStep::new(KeyframePercentage::new(1.),
-                                                 KeyframesStepValue::ComputedValues,
-                                                 guard));
+            result.steps.push(KeyframesStep::new(
+                KeyframePercentage::new(1.),
+                KeyframesStepValue::ComputedValues,
+                guard,
+            ));
         }
 
         result
@@ -449,145 +522,82 @@ impl KeyframesAnimation {
 /// 40%, 60%, 100% {
 ///     width: 100%;
 /// }
-struct KeyframeListParser<'a, R: 'a> {
-    context: &'a ParserContext<'a>,
-    error_context: &'a ParserErrorContext<'a, R>,
+struct KeyframeListParser<'a, 'b> {
+    context: &'a mut ParserContext<'b>,
     shared_lock: &'a SharedRwLock,
-    declarations: &'a mut SourcePropertyDeclaration,
 }
 
 /// Parses a keyframe list from CSS input.
-pub fn parse_keyframe_list<R>(
-    context: &ParserContext,
-    error_context: &ParserErrorContext<R>,
+pub fn parse_keyframe_list<'a>(
+    context: &mut ParserContext<'a>,
     input: &mut Parser,
-    shared_lock: &SharedRwLock
-) -> Vec<Arc<Locked<Keyframe>>>
-    where R: ParseErrorReporter
-{
-    debug_assert!(context.namespaces.is_some(),
-                  "Parsing a keyframe list from a context without namespaces?");
-
-    let mut declarations = SourcePropertyDeclaration::new();
-    RuleListParser::new_for_nested_rule(input, KeyframeListParser {
-        context: context,
-        error_context: error_context,
-        shared_lock: shared_lock,
-        declarations: &mut declarations,
-    }).filter_map(Result::ok).collect()
+    shared_lock: &SharedRwLock,
+) -> Vec<Arc<Locked<Keyframe>>> {
+    let mut parser = KeyframeListParser {
+        context,
+        shared_lock,
+    };
+    RuleBodyParser::new(input, &mut parser)
+        .filter_map(Result::ok)
+        .collect()
 }
 
-impl<'a, 'i, R> AtRuleParser<'i> for KeyframeListParser<'a, R> {
-    type PreludeNoBlock = ();
-    type PreludeBlock = ();
+impl<'a, 'b, 'i> AtRuleParser<'i> for KeyframeListParser<'a, 'b> {
+    type Prelude = ();
     type AtRule = Arc<Locked<Keyframe>>;
     type Error = StyleParseErrorKind<'i>;
 }
 
-/// A wrapper to wraps the KeyframeSelector with its source location
-struct KeyframeSelectorParserPrelude {
-    selector: KeyframeSelector,
-    source_location: SourceLocation,
+impl<'a, 'b, 'i> DeclarationParser<'i> for KeyframeListParser<'a, 'b> {
+    type Declaration = Arc<Locked<Keyframe>>;
+    type Error = StyleParseErrorKind<'i>;
 }
 
-impl<'a, 'i, R: ParseErrorReporter> QualifiedRuleParser<'i> for KeyframeListParser<'a, R> {
-    type Prelude = KeyframeSelectorParserPrelude;
+impl<'a, 'b, 'i> QualifiedRuleParser<'i> for KeyframeListParser<'a, 'b> {
+    type Prelude = KeyframeSelector;
     type QualifiedRule = Arc<Locked<Keyframe>>;
     type Error = StyleParseErrorKind<'i>;
 
-    fn parse_prelude<'t>(&mut self, input: &mut Parser<'i, 't>) -> Result<Self::Prelude, ParseError<'i>> {
+    fn parse_prelude<'t>(
+        &mut self,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<Self::Prelude, ParseError<'i>> {
         let start_position = input.position();
-        let start_location = input.current_source_location();
-        match KeyframeSelector::parse(input) {
-            Ok(sel) => {
-                Ok(KeyframeSelectorParserPrelude {
-                    selector: sel,
-                    source_location: start_location,
-                })
-            },
-            Err(e) => {
-                let location = e.location;
-                let error = ContextualParseError::InvalidKeyframeRule(input.slice_from(start_position), e.clone());
-                self.context.log_css_error(self.error_context, location, error);
-                Err(e)
-            }
-        }
+        KeyframeSelector::parse(input).map_err(|e| {
+            let location = e.location;
+            let error = ContextualParseError::InvalidKeyframeRule(
+                input.slice_from(start_position),
+                e.clone(),
+            );
+            self.context.log_css_error(location, error);
+            e
+        })
     }
 
-    fn parse_block<'t>(&mut self, prelude: Self::Prelude, input: &mut Parser<'i, 't>)
-                       -> Result<Self::QualifiedRule, ParseError<'i>> {
-        let context =
-            ParserContext::new_with_rule_type(
-                self.context,
-                CssRuleType::Keyframe,
-                self.context.namespaces.unwrap(),
-            );
-
-        let parser = KeyframeDeclarationParser {
-            context: &context,
-            declarations: self.declarations,
-        };
-        let mut iter = DeclarationListParser::new(input, parser);
-        let mut block = PropertyDeclarationBlock::new();
-        while let Some(declaration) = iter.next() {
-            match declaration {
-                Ok(()) => {
-                    block.extend(
-                        iter.parser.declarations.drain(),
-                        Importance::Normal,
-                        DeclarationSource::Parsing,
-                    );
-                }
-                Err((error, slice)) => {
-                    iter.parser.declarations.clear();
-                    let location = error.location;
-                    let error = ContextualParseError::UnsupportedKeyframePropertyDeclaration(slice, error);
-                    context.log_css_error(self.error_context, location, error);
-                }
-            }
-            // `parse_important` is not called here, `!important` is not allowed in keyframe blocks.
-        }
+    fn parse_block<'t>(
+        &mut self,
+        selector: Self::Prelude,
+        start: &ParserState,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<Self::QualifiedRule, ParseError<'i>> {
+        let block = self.context.nest_for_rule(CssRuleType::Keyframe, |p| {
+            parse_property_declaration_list(&p, input, &[])
+        });
         Ok(Arc::new(self.shared_lock.wrap(Keyframe {
-            selector: prelude.selector,
+            selector,
             block: Arc::new(self.shared_lock.wrap(block)),
-            source_location: prelude.source_location,
+            source_location: start.source_location(),
         })))
     }
 }
 
-struct KeyframeDeclarationParser<'a, 'b: 'a> {
-    context: &'a ParserContext<'b>,
-    declarations: &'a mut SourcePropertyDeclaration,
-}
-
-/// Default methods reject all at rules.
-impl<'a, 'b, 'i> AtRuleParser<'i> for KeyframeDeclarationParser<'a, 'b> {
-    type PreludeNoBlock = ();
-    type PreludeBlock = ();
-    type AtRule = ();
-    type Error = StyleParseErrorKind<'i>;
-}
-
-impl<'a, 'b, 'i> DeclarationParser<'i> for KeyframeDeclarationParser<'a, 'b> {
-    type Declaration = ();
-    type Error = StyleParseErrorKind<'i>;
-
-    fn parse_value<'t>(
-        &mut self,
-        name: CowRcStr<'i>,
-        input: &mut Parser<'i, 't>,
-    ) -> Result<(), ParseError<'i>> {
-        let id = PropertyId::parse(&name).map_err(|()| {
-            input.new_custom_error(StyleParseErrorKind::UnknownProperty(name.clone()))
-        })?;
-
-        // TODO(emilio): Shouldn't this use parse_entirely?
-        PropertyDeclaration::parse_into(self.declarations, id, name, self.context, input)?;
-
-        // In case there is still unparsed text in the declaration, we should
-        // roll back.
-        input.expect_exhausted()?;
-
-        Ok(())
+impl<'a, 'b, 'i> RuleBodyItemParser<'i, Arc<Locked<Keyframe>>, StyleParseErrorKind<'i>>
+    for KeyframeListParser<'a, 'b>
+{
+    fn parse_qualified(&self) -> bool {
+        true
+    }
+    fn parse_declarations(&self) -> bool {
+        false
     }
 }
