@@ -6,7 +6,7 @@ use crate::context::QuirksMode;
 use crate::error_reporting::{ContextualParseError, ParseErrorReporter};
 use crate::media_queries::{Device, MediaList};
 use crate::parser::ParserContext;
-use crate::shared_lock::{DeepCloneParams, DeepCloneWithLock, Locked};
+use crate::shared_lock::{DeepCloneWithLock, Locked};
 use crate::shared_lock::{SharedRwLock, SharedRwLockReadGuard};
 use crate::stylesheets::loader::StylesheetLoader;
 use crate::stylesheets::rule_parser::{State, TopLevelRuleParser};
@@ -15,16 +15,17 @@ use crate::stylesheets::rules_iterator::{NestedRuleIterationCondition, RulesIter
 use crate::stylesheets::{CssRule, CssRules, Origin, UrlExtraData};
 use crate::use_counters::UseCounters;
 use crate::{Namespace, Prefix};
-use cssparser::{Parser, ParserInput, RuleListParser};
+use cssparser::{Parser, ParserInput, StyleSheetParser};
 use fxhash::FxHashMap;
 #[cfg(feature = "gecko")]
 use malloc_size_of::{MallocSizeOfOps, MallocUnconditionalShallowSizeOf};
 use parking_lot::RwLock;
 use servo_arc::Arc;
-use std::mem;
 use std::sync::atomic::{AtomicBool, Ordering};
 use style_traits::ParsingMode;
 
+use super::scope_rule::ImplicitScopeRoot;
+
 /// This structure holds the user-agent and user stylesheets.
 pub struct UserAgentStylesheets {
     /// The lock used for user-agent stylesheets.
@@ -64,6 +65,8 @@ pub struct StylesheetContents {
     pub source_map_url: RwLock<Option<String>>,
     /// This stylesheet's source URL.
     pub source_url: RwLock<Option<String>>,
+    /// The use counters of the original stylesheet.
+    pub use_counters: UseCounters,
 
     /// We don't want to allow construction outside of this file, to guarantee
     /// that all contents are created with Arc<>.
@@ -81,23 +84,19 @@ impl StylesheetContents {
         stylesheet_loader: Option<&dyn StylesheetLoader>,
         error_reporter: Option<&dyn ParseErrorReporter>,
         quirks_mode: QuirksMode,
-        line_number_offset: u32,
-        use_counters: Option<&UseCounters>,
         allow_import_rules: AllowImportRules,
         sanitization_data: Option<&mut SanitizationData>,
     ) -> Arc<Self> {
-        let namespaces = RwLock::new(Namespaces::default());
-        let (rules, source_map_url, source_url) = Stylesheet::parse_rules(
+        let use_counters = UseCounters::default();
+        let (namespaces, rules, source_map_url, source_url) = Stylesheet::parse_rules(
             css,
             &url_data,
             origin,
-            &mut *namespaces.write(),
             &shared_lock,
             stylesheet_loader,
             error_reporter,
             quirks_mode,
-            line_number_offset,
-            use_counters,
+            Some(&use_counters),
             allow_import_rules,
             sanitization_data,
         );
@@ -106,10 +105,11 @@ impl StylesheetContents {
             rules: CssRules::new(rules, &shared_lock),
             origin,
             url_data: RwLock::new(url_data),
-            namespaces,
+            namespaces: RwLock::new(namespaces),
             quirks_mode,
             source_map_url: RwLock::new(source_map_url),
             source_url: RwLock::new(source_url),
+            use_counters,
             _forbid_construction: (),
         })
     }
@@ -140,6 +140,7 @@ impl StylesheetContents {
             quirks_mode,
             source_map_url: RwLock::new(None),
             source_url: RwLock::new(None),
+            use_counters: UseCounters::default(),
             _forbid_construction: (),
         })
     }
@@ -167,13 +168,12 @@ impl DeepCloneWithLock for StylesheetContents {
         &self,
         lock: &SharedRwLock,
         guard: &SharedRwLockReadGuard,
-        params: &DeepCloneParams,
     ) -> Self {
         // Make a deep clone of the rules, using the new lock.
         let rules = self
             .rules
             .read_with(guard)
-            .deep_clone_with_lock(lock, guard, params);
+            .deep_clone_with_lock(lock, guard);
 
         Self {
             rules: Arc::new(lock.wrap(rules)),
@@ -183,6 +183,7 @@ impl DeepCloneWithLock for StylesheetContents {
             namespaces: RwLock::new((*self.namespaces.read()).clone()),
             source_map_url: RwLock::new((*self.source_map_url.read()).clone()),
             source_url: RwLock::new((*self.source_url.read()).clone()),
+            use_counters: self.use_counters.clone(),
             _forbid_construction: (),
         }
     }
@@ -201,26 +202,6 @@ pub struct Stylesheet {
     pub disabled: AtomicBool,
 }
 
-macro_rules! rule_filter {
-    ($( $method: ident($variant:ident => $rule_type: ident), )+) => {
-        $(
-            #[allow(missing_docs)]
-            fn $method<F>(&self, device: &Device, guard: &SharedRwLockReadGuard, mut f: F)
-                where F: FnMut(&crate::stylesheets::$rule_type),
-            {
-                use crate::stylesheets::CssRule;
-
-                for rule in self.effective_rules(device, guard) {
-                    if let CssRule::$variant(ref lock) = *rule {
-                        let rule = lock.read_with(guard);
-                        f(&rule)
-                    }
-                }
-            }
-        )+
-    }
-}
-
 /// A trait to represent a given stylesheet in a document.
 pub trait StylesheetInDocument: ::std::fmt::Debug {
     /// Get whether this stylesheet is enabled.
@@ -275,10 +256,8 @@ pub trait StylesheetInDocument: ::std::fmt::Debug {
         self.iter_rules::<EffectiveRules>(device, guard)
     }
 
-    rule_filter! {
-        effective_style_rules(Style => StyleRule),
-        effective_viewport_rules(Viewport => ViewportRule),
-    }
+    /// Return the implicit scope root for this stylesheet, if one exists.
+    fn implicit_scope_root(&self) -> Option<ImplicitScopeRoot>;
 }
 
 impl StylesheetInDocument for Stylesheet {
@@ -294,6 +273,10 @@ impl StylesheetInDocument for Stylesheet {
     fn contents(&self) -> &StylesheetContents {
         &self.contents
     }
+
+    fn implicit_scope_root(&self) -> Option<ImplicitScopeRoot> {
+        None
+    }
 }
 
 /// A simple wrapper over an `Arc<Stylesheet>`, with pointer comparison, and
@@ -323,6 +306,10 @@ impl StylesheetInDocument for DocumentStyleSheet {
     fn contents(&self) -> &StylesheetContents {
         self.0.contents()
     }
+
+    fn implicit_scope_root(&self) -> Option<ImplicitScopeRoot> {
+        None
+    }
 }
 
 /// The kind of sanitization to use when parsing a stylesheet.
@@ -363,16 +350,25 @@ impl SanitizationKind {
             // TODO(emilio): Perhaps Layer should not be always sanitized? But
             // we sanitize @media and co, so this seems safer for now.
             CssRule::LayerStatement(..) |
-            CssRule::LayerBlock(..) => false,
+            CssRule::LayerBlock(..) |
+            // TODO(dshin): Same comment as Layer applies - shouldn't give away
+            // something like display size - erring on the side of "safe" for now.
+            CssRule::Scope(..) |
+            CssRule::StartingStyle(..) => false,
 
-            CssRule::FontFace(..) | CssRule::Namespace(..) | CssRule::Style(..) => true,
+            CssRule::FontFace(..) |
+            CssRule::Namespace(..) |
+            CssRule::Style(..) |
+            CssRule::NestedDeclarations(..) |
+            CssRule::PositionTry(..) => true,
 
             CssRule::Keyframes(..) |
             CssRule::Page(..) |
+            CssRule::Margin(..) |
+            CssRule::Property(..) |
             CssRule::FontFeatureValues(..) |
-            CssRule::Viewport(..) |
-            CssRule::CounterStyle(..) |
-            CssRule::ScrollTimeline(..) => !is_standard,
+            CssRule::FontPaletteValues(..) |
+            CssRule::CounterStyle(..) => !is_standard,
         }
     }
 }
@@ -412,56 +408,46 @@ impl Stylesheet {
         url_data: UrlExtraData,
         stylesheet_loader: Option<&dyn StylesheetLoader>,
         error_reporter: Option<&dyn ParseErrorReporter>,
-        line_number_offset: u32,
         allow_import_rules: AllowImportRules,
     ) {
-        let namespaces = RwLock::new(Namespaces::default());
-
-        // FIXME: Consider adding use counters to Servo?
-        let (rules, source_map_url, source_url) = Self::parse_rules(
+        let use_counters = UseCounters::default();
+        let (namespaces, rules, source_map_url, source_url) = Self::parse_rules(
             css,
             &url_data,
             existing.contents.origin,
-            &mut *namespaces.write(),
             &existing.shared_lock,
             stylesheet_loader,
             error_reporter,
             existing.contents.quirks_mode,
-            line_number_offset,
-            /* use_counters = */ None,
+            Some(&use_counters),
             allow_import_rules,
             /* sanitization_data = */ None,
         );
 
         *existing.contents.url_data.write() = url_data;
-        mem::swap(
-            &mut *existing.contents.namespaces.write(),
-            &mut *namespaces.write(),
-        );
+        *existing.contents.namespaces.write() = namespaces;
 
         // Acquire the lock *after* parsing, to minimize the exclusive section.
         let mut guard = existing.shared_lock.write();
         *existing.contents.rules.write_with(&mut guard) = CssRules(rules);
         *existing.contents.source_map_url.write() = source_map_url;
         *existing.contents.source_url.write() = source_url;
+        existing.contents.use_counters.merge(&use_counters);
     }
 
     fn parse_rules(
         css: &str,
         url_data: &UrlExtraData,
         origin: Origin,
-        namespaces: &mut Namespaces,
         shared_lock: &SharedRwLock,
         stylesheet_loader: Option<&dyn StylesheetLoader>,
         error_reporter: Option<&dyn ParseErrorReporter>,
         quirks_mode: QuirksMode,
-        line_number_offset: u32,
         use_counters: Option<&UseCounters>,
         allow_import_rules: AllowImportRules,
         mut sanitization_data: Option<&mut SanitizationData>,
-    ) -> (Vec<CssRule>, Option<String>, Option<String>) {
-        let mut rules = Vec::new();
-        let mut input = ParserInput::new_with_line_number_offset(css, line_number_offset);
+    ) -> (Namespaces, Vec<CssRule>, Option<String>, Option<String>) {
+        let mut input = ParserInput::new(css);
         let mut input = Parser::new(&mut input);
 
         let context = ParserContext::new(
@@ -470,45 +456,42 @@ impl Stylesheet {
             None,
             ParsingMode::DEFAULT,
             quirks_mode,
+            /* namespaces = */ Default::default(),
             error_reporter,
             use_counters,
         );
 
-        let rule_parser = TopLevelRuleParser {
+        let mut rule_parser = TopLevelRuleParser {
             shared_lock,
             loader: stylesheet_loader,
             context,
             state: State::Start,
             dom_error: None,
             insert_rule_context: None,
-            namespaces,
             allow_import_rules,
+            declaration_parser_state: Default::default(),
+            first_declaration_block: Default::default(),
+            wants_first_declaration_block: false,
+            error_reporting_state: Default::default(),
+            rules: Vec::new(),
         };
 
         {
-            let mut iter = RuleListParser::new_for_stylesheet(&mut input, rule_parser);
-
-            loop {
-                let result = match iter.next() {
-                    Some(result) => result,
-                    None => break,
-                };
+            let mut iter = StyleSheetParser::new(&mut input, &mut rule_parser);
+            while let Some(result) = iter.next() {
                 match result {
-                    Ok((rule_start, rule)) => {
+                    Ok(rule_start) => {
+                        // TODO(emilio, nesting): sanitize nested CSS rules, probably?
                         if let Some(ref mut data) = sanitization_data {
-                            if !data.kind.allows(&rule) {
-                                continue;
+                            if let Some(ref rule) = iter.parser.rules.last() {
+                                if !data.kind.allows(rule) {
+                                    iter.parser.rules.pop();
+                                    continue;
+                                }
                             }
                             let end = iter.input.position().byte_index();
                             data.output.push_str(&css[rule_start.byte_index()..end]);
                         }
-                        // Use a fallible push here, and if it fails, just fall
-                        // out of the loop.  This will cause the page to be
-                        // shown incorrectly, but it's better than OOMing.
-                        if rules.try_reserve(1).is_err() {
-                            break;
-                        }
-                        rules.push(rule);
                     },
                     Err((error, slice)) => {
                         let location = error.location;
@@ -521,7 +504,12 @@ impl Stylesheet {
 
         let source_map_url = input.current_source_map_url().map(String::from);
         let source_url = input.current_source_url().map(String::from);
-        (rules, source_map_url, source_url)
+        (
+            rule_parser.context.namespaces.into_owned(),
+            rule_parser.rules,
+            source_map_url,
+            source_url,
+        )
     }
 
     /// Creates an empty stylesheet and parses it with a given base url, origin
@@ -538,7 +526,6 @@ impl Stylesheet {
         stylesheet_loader: Option<&dyn StylesheetLoader>,
         error_reporter: Option<&dyn ParseErrorReporter>,
         quirks_mode: QuirksMode,
-        line_number_offset: u32,
         allow_import_rules: AllowImportRules,
     ) -> Self {
         // FIXME: Consider adding use counters to Servo?
@@ -550,8 +537,6 @@ impl Stylesheet {
             stylesheet_loader,
             error_reporter,
             quirks_mode,
-            line_number_offset,
-            /* use_counters = */ None,
             allow_import_rules,
             /* sanitized_output = */ None,
         );
@@ -592,15 +577,11 @@ impl Clone for Stylesheet {
         // Make a deep clone of the media, using the new lock.
         let media = self.media.read_with(&guard).clone();
         let media = Arc::new(lock.wrap(media));
-        let contents = Arc::new(self.contents.deep_clone_with_lock(
-            &lock,
-            &guard,
-            &DeepCloneParams,
-        ));
+        let contents = Arc::new(self.contents.deep_clone_with_lock(&lock, &guard));
 
         Stylesheet {
             contents,
-            media: media,
+            media,
             shared_lock: lock,
             disabled: AtomicBool::new(self.disabled.load(Ordering::SeqCst)),
         }
