@@ -8,14 +8,15 @@
 //! [position]: https://drafts.csswg.org/css-backgrounds-3/#position
 
 use crate::values::computed::{Integer, LengthPercentage, NonNegativeNumber, Percentage};
-use crate::values::generics::position::AspectRatio as GenericAspectRatio;
+use crate::values::generics::position::GenericAnchorFunction;
 use crate::values::generics::position::Position as GenericPosition;
 use crate::values::generics::position::PositionComponent as GenericPositionComponent;
 use crate::values::generics::position::PositionOrAuto as GenericPositionOrAuto;
 use crate::values::generics::position::ZIndex as GenericZIndex;
+use crate::values::generics::position::{AspectRatio as GenericAspectRatio, GenericInset};
 pub use crate::values::specified::position::{
-    AnchorName, AnchorScope, DashedIdentAndOrTryTactic, InsetArea, InsetAreaKeyword,
-    PositionAnchor, PositionTryOptions, PositionTryOrder, PositionVisibility,
+    AnchorName, AnchorScope, DashedIdentAndOrTryTactic, PositionAnchor, PositionArea,
+    PositionAreaKeyword, PositionTryFallbacks, PositionTryOrder, PositionVisibility,
 };
 pub use crate::values::specified::position::{GridAutoFlow, GridTemplateAreas, MasonryAutoFlow};
 use crate::Zero;
@@ -34,6 +35,12 @@ pub type HorizontalPosition = LengthPercentage;
 /// The computed value of a CSS vertical position.
 pub type VerticalPosition = LengthPercentage;
 
+/// The computed value of an `anchor()` function.
+pub type AnchorFunction = GenericAnchorFunction<Percentage, LengthPercentage>;
+
+/// A computed type for `inset` properties.
+pub type Inset = GenericInset<Percentage, LengthPercentage>;
+
 impl Position {
     /// `50% 50%`
     #[inline]
