@@ -6,6 +6,8 @@
 //!
 //! [calc]: https://drafts.csswg.org/css-values/#calc-notation
 
+use crate::values::generics::length::GenericAnchorSizeFunction;
+use crate::values::generics::position::{AnchorSide, GenericAnchorFunction};
 use num_traits::Zero;
 use smallvec::SmallVec;
 use std::fmt::{self, Write};
@@ -158,9 +160,16 @@ pub enum SortKey {
     Vmax,
     Vmin,
     Vw,
+    ColorComponent,
     Other,
 }
 
+/// `anchor()` function used in math functions.
+pub type GenericCalcAnchorFunction<L> =
+    GenericAnchorFunction<Box<GenericCalcNode<L>>, Box<GenericCalcNode<L>>>;
+/// `anchor-size()` function used in math functions.
+pub type GenericCalcAnchorSizeFunction<L> = GenericAnchorSizeFunction<Box<GenericCalcNode<L>>>;
+
 /// A generic node in a calc expression.
 ///
 /// FIXME: This would be much more elegant if we used `Self` in the types below,
@@ -233,6 +242,10 @@ pub enum GenericCalcNode<L> {
     Abs(Box<GenericCalcNode<L>>),
     /// A `sign()` function.
     Sign(Box<GenericCalcNode<L>>),
+    /// An `anchor()` function.
+    Anchor(Box<GenericCalcAnchorFunction<L>>),
+    /// An `anchor-size()` function.
+    AnchorSize(Box<GenericCalcAnchorSizeFunction<L>>),
 }
 
 pub use self::GenericCalcNode as CalcNode;
@@ -543,6 +556,9 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                 let _ = child.unit()?;
                 CalcUnits::empty()
             },
+            CalcNode::Anchor(..) | CalcNode::AnchorSize(..) => {
+                CalcUnits::LENGTH_PERCENTAGE
+            }
         })
     }
 
@@ -645,12 +661,16 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
             CalcNode::Sign(ref mut child) => {
                 child.negate();
             },
+            CalcNode::Anchor(_) | CalcNode::AnchorSize(_) => {
+                wrap_self_in_negate(self);
+            },
         }
     }
 
     fn sort_key(&self) -> SortKey {
         match *self {
             Self::Leaf(ref l) => l.sort_key(),
+            Self::Anchor(..) | Self::AnchorSize(..) => SortKey::Px,
             _ => SortKey::Other,
         }
     }
@@ -766,6 +786,9 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                     Ok(())
                 },
                 CalcNode::Abs(child) | CalcNode::Sign(child) => map_internal(child, op),
+                // It is invalid to treat inner `CalcNode`s here - `anchor(--foo 50%) / 2` != `anchor(--foo 25%)`.
+                // Same applies to fallback, as we don't know if it will be used. Similar reasoning applies to `anchor-size()`.
+                CalcNode::Anchor(_) | CalcNode::AnchorSize(_) => Err(())
             }
         }
 
@@ -847,6 +870,23 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
             Self::Hypot(ref c) => CalcNode::Hypot(map_children(c, map)),
             Self::Abs(ref c) => CalcNode::Abs(Box::new(c.map_leaves_internal(map))),
             Self::Sign(ref c) => CalcNode::Sign(Box::new(c.map_leaves_internal(map))),
+            Self::Anchor(ref f) => CalcNode::Anchor(Box::new(
+                GenericAnchorFunction {
+                    target_element: f.target_element.clone(),
+                    side: match &f.side {
+                        AnchorSide::Keyword(k) => AnchorSide::Keyword(*k),
+                        AnchorSide::Percentage(p) => AnchorSide::Percentage(Box::new(p.map_leaves_internal(map))),
+                    },
+                    fallback: f.fallback.as_ref().map(|fb| Box::new(fb.map_leaves_internal(map))).into(),
+                }
+            )),
+            Self::AnchorSize(ref f) => CalcNode::AnchorSize(Box::new(
+                GenericAnchorSizeFunction {
+                    target_element: f.target_element.clone(),
+                    size: f.size,
+                    fallback: f.fallback.as_ref().map(|fb| Box::new(fb.map_leaves_internal(map))).into(),
+                }
+            )),
         }
     }
 
@@ -929,7 +969,7 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                 for child in children.iter().skip(1) {
                     let candidate = child.resolve_internal(leaf_to_output_fn)?;
 
-                    // Leave types must match for each child.
+                    // Leaf types must match for each child.
                     if !result.is_same_unit_as(&candidate) {
                         return Err(());
                     }
@@ -1117,9 +1157,56 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                 let result = c.resolve_internal(leaf_to_output_fn)?;
                 Ok(L::sign_from(&result)?)
             },
+            Self::Anchor(_) | Self::AnchorSize(_) => Err(()),
         }
     }
 
+    /// Mutate nodes within this calc node tree using given the mapping function.
+    pub fn map_node<F>(&mut self, mut mapping_fn: F) -> Result<(), ()>
+    where
+        F: FnMut(&CalcNode<L>) -> Result<Option<CalcNode<L>>, ()>,
+    {
+        self.map_node_internal(&mut mapping_fn)
+    }
+
+    fn map_node_internal<F>(&mut self, mapping_fn: &mut F) -> Result<(), ()>
+    where
+        F: FnMut(&CalcNode<L>) -> Result<Option<CalcNode<L>>, ()>,
+    {
+        if let Some(node) = mapping_fn(self)? {
+            *self = node;
+            // Assume that any sub-nodes don't need to be mutated.
+            return Ok(());
+        }
+        match self {
+            Self::Leaf(_) | Self::Anchor(_) | Self::AnchorSize(_) => (),
+            Self::Negate(child) | Self::Invert(child) | Self::Abs(child) | Self::Sign(child) => {
+                child.map_node_internal(mapping_fn)?;
+            },
+            Self::Sum(children) | Self::Product(children) | Self::Hypot(children) | Self::MinMax(children, _) => {
+                for child in children.iter_mut() {
+                    child.map_node_internal(mapping_fn)?;
+                }
+            },
+            Self::Clamp { min, center, max } => {
+                min.map_node_internal(mapping_fn)?;
+                center.map_node_internal(mapping_fn)?;
+                max.map_node_internal(mapping_fn)?;
+            },
+            Self::Round { value, step, .. } => {
+                value.map_node_internal(mapping_fn)?;
+                step.map_node_internal(mapping_fn)?;
+            },
+            Self::ModRem {
+                dividend, divisor, ..
+            } => {
+                dividend.map_node_internal(mapping_fn)?;
+                divisor.map_node_internal(mapping_fn)?;
+            },
+        };
+        Ok(())
+    }
+
     fn is_negative_leaf(&self) -> Result<bool, ()> {
         Ok(match *self {
             Self::Leaf(ref l) => l.is_negative()?,
@@ -1141,6 +1228,13 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
         })
     }
 
+    fn is_nan_leaf(&self) -> Result<bool, ()> {
+        Ok(match *self {
+            Self::Leaf(ref l) => l.is_nan()?,
+            _ => false,
+        })
+    }
+
     /// Visits all the nodes in this calculation tree recursively, starting by
     /// the leaves and bubbling all the way up.
     ///
@@ -1191,7 +1285,7 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
             Self::Abs(ref mut value) | Self::Sign(ref mut value) => {
                 value.visit_depth_first_internal(f);
             },
-            Self::Leaf(..) => {},
+            Self::Leaf(..) | Self::Anchor(..) | Self::AnchorSize(..) => {},
         }
         f(self);
     }
@@ -1253,14 +1347,7 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                     // `max(min, max)`.
                     let max_cmp_min = match max.compare(&min, PositivePercentageBasis::Unknown) {
                         Some(o) => o,
-                        None => {
-                            debug_assert!(
-                                false,
-                                "We compared center with min and max, how are \
-                                 min / max not comparable with each other?"
-                            );
-                            return;
-                        },
+                        None => return,
                     };
 
                     if matches!(max_cmp_min, cmp::Ordering::Less) {
@@ -1420,8 +1507,17 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                     MinMaxOp::Max => cmp::Ordering::Greater,
                 };
 
+                if value_or_stop!(children[0].is_nan_leaf()) {
+                    replace_self_with!(&mut children[0]);
+                    return;
+                }
+
                 let mut result = 0;
                 for i in 1..children.len() {
+                    if value_or_stop!(children[i].is_nan_leaf()) {
+                        replace_self_with!(&mut children[i]);
+                        return;
+                    }
                     let o = match children[i]
                         .compare(&children[result], PositivePercentageBasis::Unknown)
                     {
@@ -1533,6 +1629,9 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
 
                 debug_assert!(children.len() >= 2, "Should still have multiple kids!");
 
+                // Sort by spec order.
+                children.sort_unstable_by_key(|c| c.sort_key());
+
                 // NOTE: if the function returns true, by the docs of dedup_by,
                 // a is removed.
                 children.dedup_by(|right, left| left.try_product_in_place(right));
@@ -1610,6 +1709,19 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
             Self::Leaf(ref mut l) => {
                 l.simplify();
             },
+            Self::Anchor(ref mut f) => {
+                if let AnchorSide::Percentage(ref mut n) =  f.side {
+                    n.simplify_and_sort();
+                }
+                if let Some(fallback) = f.fallback.as_mut() {
+                    fallback.simplify_and_sort();
+                }
+            },
+            Self::AnchorSize(ref mut f) => {
+                if let Some(fallback) = f.fallback.as_mut() {
+                    fallback.simplify_and_sort();
+                }
+            }
         }
     }
 
@@ -1676,6 +1788,9 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                 true
             },
             Self::Invert(_) => {
+                if matches!(level, ArgumentLevel::CalculationRoot) {
+                    dest.write_str("calc")?;
+                }
                 dest.write_str("(1 / ")?;
                 true
             },
@@ -1690,7 +1805,7 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                     true
                 },
             },
-            Self::Leaf(_) => match level {
+            Self::Leaf(_) | Self::Anchor(_) | Self::AnchorSize(_) => match level {
                 ArgumentLevel::CalculationRoot => {
                     dest.write_str("calc(")?;
                     true
@@ -1799,6 +1914,8 @@ impl<L: CalcNodeLeaf> CalcNode<L> {
                 v.to_css_impl(dest, ArgumentLevel::ArgumentRoot)?
             },
             Self::Leaf(ref l) => l.to_css(dest)?,
+            Self::Anchor(ref f) => f.to_css(dest)?,
+            Self::AnchorSize(ref f) => f.to_css(dest)?,
         }
 
         if write_closing_paren {
