@@ -6,44 +6,47 @@
 //!
 //! [images]: https://drafts.csswg.org/css-images/#image-values
 
+use crate::color::mix::ColorInterpolationMethod;
 use crate::custom_properties;
-use crate::values::generics::position::PositionComponent;
+use crate::values::generics::{position::PositionComponent, color::GenericLightDark, Optional};
 use crate::values::serialize_atom_identifier;
 use crate::Atom;
 use crate::Zero;
 use servo_arc::Arc;
 use std::fmt::{self, Write};
 use style_traits::{CssWriter, ToCss};
-
 /// An `<image> | none` value.
 ///
 /// https://drafts.csswg.org/css-images/#image-values
 #[derive(
-    Clone, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToComputedValue, ToResolvedValue, ToShmem,
+    Clone, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToResolvedValue, ToShmem,
 )]
 #[repr(C, u8)]
-pub enum GenericImage<G, MozImageRect, ImageUrl, Color, Percentage, Resolution> {
+pub enum GenericImage<G, ImageUrl, Color, Percentage, Resolution> {
     /// `none` variant.
     None,
+
     /// A `<url()>` image.
     Url(ImageUrl),
 
     /// A `<gradient>` image.  Gradients are rather large, and not nearly as
     /// common as urls, so we box them here to keep the size of this enum sane.
     Gradient(Box<G>),
-    /// A `-moz-image-rect` image.  Also fairly large and rare.
-    // not cfg’ed out on non-Gecko to avoid `error[E0392]: parameter `MozImageRect` is never used`
-    // Instead we make MozImageRect an empty enum
-    Rect(Box<MozImageRect>),
 
     /// A `-moz-element(# <element-id>)`
     #[cfg(feature = "gecko")]
     #[css(function = "-moz-element")]
     Element(Atom),
 
+    /// A `-moz-symbolic-icon(<icon-id>)`
+    /// NOTE(emilio): #[css(skip)] only really affects SpecifiedValueInfo, which we want because
+    /// this is chrome-only.
+    #[css(function, skip)]
+    MozSymbolicIcon(Atom),
+
     /// A paint worklet image.
     /// <https://drafts.css-houdini.org/css-paint-api/>
-    #[cfg(feature = "servo-layout-2013")]
+    #[cfg(feature = "servo")]
     PaintWorklet(PaintWorklet),
 
     /// A `<cross-fade()>` image. Storing this directly inside of
@@ -53,7 +56,12 @@ pub enum GenericImage<G, MozImageRect, ImageUrl, Color, Percentage, Resolution>
     CrossFade(Box<GenericCrossFade<Self, Color, Percentage>>),
 
     /// An `image-set()` function.
-    ImageSet(#[compute(field_bound)] Box<GenericImageSet<Self, Resolution>>),
+    ImageSet(Box<GenericImageSet<Self, Resolution>>),
+
+    /// A `light-dark()` function.
+    /// NOTE(emilio): #[css(skip)] only affects SpecifiedValueInfo. Remove or make conditional
+    /// if/when shipping light-dark() for content.
+    LightDark(#[css(skip)] Box<GenericLightDark<Self>>),
 }
 
 pub use self::GenericImage as Image;
@@ -71,20 +79,6 @@ pub struct GenericCrossFade<Image, Color, Percentage> {
     pub elements: crate::OwnedSlice<GenericCrossFadeElement<Image, Color, Percentage>>,
 }
 
-/// A `<percent> | none` value. Represents optional percentage values
-/// assosicated with cross-fade images.
-#[derive(
-    Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem, ToCss,
-)]
-#[repr(C, u8)]
-pub enum PercentOrNone<Percentage> {
-    /// `none` variant.
-    #[css(skip)]
-    None,
-    /// A percentage variant.
-    Percent(Percentage),
-}
-
 /// An optional percent and a cross fade image.
 #[derive(
     Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem, ToCss,
@@ -92,7 +86,7 @@ pub enum PercentOrNone<Percentage> {
 #[repr(C)]
 pub struct GenericCrossFadeElement<Image, Color, Percentage> {
     /// The percent of the final image that `image` will be.
-    pub percent: PercentOrNone<Percentage>,
+    pub percent: Optional<Percentage>,
     /// A color or image that will be blended when cross-fade is
     /// evaluated.
     pub image: GenericCrossFadeImage<Image, Color>,
@@ -121,7 +115,7 @@ pub use self::GenericCrossFadeImage as CrossFadeImage;
 #[css(comma, function = "image-set")]
 #[repr(C)]
 pub struct GenericImageSet<Image, Resolution> {
-    /// The index of the selected candidate. Zero for specified values.
+    /// The index of the selected candidate. usize::MAX for specified values or invalid images.
     #[css(skip)]
     pub selected_index: usize,
 
@@ -131,9 +125,7 @@ pub struct GenericImageSet<Image, Resolution> {
 }
 
 /// An optional percent and a cross fade image.
-#[derive(
-    Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem,
-)]
+#[derive(Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem)]
 #[repr(C)]
 pub struct GenericImageSetItem<Image, Resolution> {
     /// `<image>`. `<string>` is converted to `Image::Url` at parse time.
@@ -151,21 +143,20 @@ pub struct GenericImageSetItem<Image, Resolution> {
     pub has_mime_type: bool,
 }
 
-impl<I: style_traits::ToCss, R: style_traits::ToCss> ToCss for GenericImageSetItem<I, R>
-{
+impl<I: style_traits::ToCss, R: style_traits::ToCss> ToCss for GenericImageSetItem<I, R> {
     fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
     where
         W: fmt::Write,
     {
         self.image.to_css(dest)?;
-        dest.write_str(" ")?;
+        dest.write_char(' ')?;
         self.resolution.to_css(dest)?;
 
         if self.has_mime_type {
-            dest.write_str(" ")?;
+            dest.write_char(' ')?;
             dest.write_str("type(")?;
             self.mime_type.to_css(dest)?;
-            dest.write_str(")")?;
+            dest.write_char(')')?;
         }
         Ok(())
     }
@@ -174,6 +165,21 @@ impl<I: style_traits::ToCss, R: style_traits::ToCss> ToCss for GenericImageSetIt
 pub use self::GenericImageSet as ImageSet;
 pub use self::GenericImageSetItem as ImageSetItem;
 
+/// State flags stored on each variant of a Gradient.
+#[derive(
+    Clone, Copy, Debug, Default, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem,
+)]
+#[repr(C)]
+pub struct GradientFlags(u8);
+bitflags! {
+    impl GradientFlags: u8 {
+        /// Set if this is a repeating gradient.
+        const REPEATING = 1 << 0;
+        /// Set if the color interpolation method matches the default for the items.
+        const HAS_DEFAULT_COLOR_INTERPOLATION_METHOD = 1 << 1;
+    }
+}
+
 /// A CSS gradient.
 /// <https://drafts.csswg.org/css-images/#gradients>
 #[derive(Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue, ToResolvedValue, ToShmem)]
@@ -192,10 +198,12 @@ pub enum GenericGradient<
     Linear {
         /// Line direction
         direction: LineDirection,
+        /// Method to use for color interpolation.
+        color_interpolation_method: ColorInterpolationMethod,
         /// The color stops and interpolation hints.
         items: crate::OwnedSlice<GenericGradientItem<Color, LengthPercentage>>,
-        /// True if this is a repeating gradient.
-        repeating: bool,
+        /// State flags for the gradient.
+        flags: GradientFlags,
         /// Compatibility mode.
         compat_mode: GradientCompatMode,
     },
@@ -205,10 +213,12 @@ pub enum GenericGradient<
         shape: GenericEndingShape<NonNegativeLength, NonNegativeLengthPercentage>,
         /// Center of gradient
         position: Position,
+        /// Method to use for color interpolation.
+        color_interpolation_method: ColorInterpolationMethod,
         /// The color stops and interpolation hints.
         items: crate::OwnedSlice<GenericGradientItem<Color, LengthPercentage>>,
-        /// True if this is a repeating gradient.
-        repeating: bool,
+        /// State flags for the gradient.
+        flags: GradientFlags,
         /// Compatibility mode.
         compat_mode: GradientCompatMode,
     },
@@ -218,10 +228,12 @@ pub enum GenericGradient<
         angle: Angle,
         /// Center of gradient
         position: Position,
+        /// Method to use for color interpolation.
+        color_interpolation_method: ColorInterpolationMethod,
         /// The color stops and interpolation hints.
         items: crate::OwnedSlice<GenericGradientItem<Color, AngleOrPercentage>>,
-        /// True if this is a repeating gradient.
-        repeating: bool,
+        /// State flags for the gradient.
+        flags: GradientFlags,
     },
 }
 
@@ -385,50 +397,22 @@ impl ToCss for PaintWorklet {
             dest.write_str(", ")?;
             argument.to_css(dest)?;
         }
-        dest.write_str(")")
+        dest.write_char(')')
     }
 }
 
-/// Values for `moz-image-rect`.
-///
-/// `-moz-image-rect(<uri>, top, right, bottom, left);`
-#[allow(missing_docs)]
-#[derive(
-    Clone,
-    Debug,
-    MallocSizeOf,
-    PartialEq,
-    SpecifiedValueInfo,
-    ToComputedValue,
-    ToCss,
-    ToResolvedValue,
-    ToShmem,
-)]
-#[css(comma, function = "-moz-image-rect")]
-#[repr(C)]
-pub struct GenericMozImageRect<NumberOrPercentage, MozImageRectUrl> {
-    pub url: MozImageRectUrl,
-    pub top: NumberOrPercentage,
-    pub right: NumberOrPercentage,
-    pub bottom: NumberOrPercentage,
-    pub left: NumberOrPercentage,
-}
-
-pub use self::GenericMozImageRect as MozImageRect;
-
-impl<G, R, U, C, P, Resolution> fmt::Debug for Image<G, R, U, C, P, Resolution>
+impl<G, U, C, P, Resolution> fmt::Debug for Image<G, U, C, P, Resolution>
 where
-    Image<G, R, U, C, P, Resolution>: ToCss,
+    Image<G, U, C, P, Resolution>: ToCss,
 {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         self.to_css(&mut CssWriter::new(f))
     }
 }
 
-impl<G, R, U, C, P, Resolution> ToCss for Image<G, R, U, C, P, Resolution>
+impl<G, U, C, P, Resolution> ToCss for Image<G, U, C, P, Resolution>
 where
     G: ToCss,
-    R: ToCss,
     U: ToCss,
     C: ToCss,
     P: ToCss,
@@ -442,17 +426,22 @@ where
             Image::None => dest.write_str("none"),
             Image::Url(ref url) => url.to_css(dest),
             Image::Gradient(ref gradient) => gradient.to_css(dest),
-            Image::Rect(ref rect) => rect.to_css(dest),
-            #[cfg(feature = "servo-layout-2013")]
+            #[cfg(feature = "servo")]
             Image::PaintWorklet(ref paint_worklet) => paint_worklet.to_css(dest),
             #[cfg(feature = "gecko")]
             Image::Element(ref selector) => {
                 dest.write_str("-moz-element(#")?;
                 serialize_atom_identifier(selector, dest)?;
-                dest.write_str(")")
+                dest.write_char(')')
+            },
+            Image::MozSymbolicIcon(ref id) => {
+                dest.write_str("-moz-symbolic-icon(")?;
+                serialize_atom_identifier(id, dest)?;
+                dest.write_char(')')
             },
             Image::ImageSet(ref is) => is.to_css(dest),
             Image::CrossFade(ref cf) => cf.to_css(dest),
+            Image::LightDark(ref ld) => ld.to_css(dest),
         }
     }
 }
@@ -472,18 +461,22 @@ where
     where
         W: Write,
     {
-        let (compat_mode, repeating) = match *self {
+        let (compat_mode, repeating, has_default_color_interpolation_method) = match *self {
             Gradient::Linear {
-                compat_mode,
-                repeating,
-                ..
-            } => (compat_mode, repeating),
+                compat_mode, flags, ..
+            } |
             Gradient::Radial {
+                compat_mode, flags, ..
+            } => (
                 compat_mode,
-                repeating,
-                ..
-            } => (compat_mode, repeating),
-            Gradient::Conic { repeating, .. } => (GradientCompatMode::Modern, repeating),
+                flags.contains(GradientFlags::REPEATING),
+                flags.contains(GradientFlags::HAS_DEFAULT_COLOR_INTERPOLATION_METHOD),
+            ),
+            Gradient::Conic { flags, .. } => (
+                GradientCompatMode::Modern,
+                flags.contains(GradientFlags::REPEATING),
+                flags.contains(GradientFlags::HAS_DEFAULT_COLOR_INTERPOLATION_METHOD),
+            ),
         };
 
         match compat_mode {
@@ -499,17 +492,24 @@ where
         match *self {
             Gradient::Linear {
                 ref direction,
+                ref color_interpolation_method,
                 ref items,
                 compat_mode,
                 ..
             } => {
                 dest.write_str("linear-gradient(")?;
-                let mut skip_comma = if !direction.points_downwards(compat_mode) {
+                let mut skip_comma = true;
+                if !direction.points_downwards(compat_mode) {
                     direction.to_css(dest, compat_mode)?;
-                    false
-                } else {
-                    true
-                };
+                    skip_comma = false;
+                }
+                if !has_default_color_interpolation_method {
+                    if !skip_comma {
+                        dest.write_char(' ')?;
+                    }
+                    color_interpolation_method.to_css(dest)?;
+                    skip_comma = false;
+                }
                 for item in &**items {
                     if !skip_comma {
                         dest.write_str(", ")?;
@@ -521,6 +521,7 @@ where
             Gradient::Radial {
                 ref shape,
                 ref position,
+                ref color_interpolation_method,
                 ref items,
                 compat_mode,
                 ..
@@ -536,7 +537,7 @@ where
                     if !omit_shape {
                         shape.to_css(dest)?;
                         if !omit_position {
-                            dest.write_str(" ")?;
+                            dest.write_char(' ')?;
                         }
                     }
                     if !omit_position {
@@ -554,7 +555,15 @@ where
                         shape.to_css(dest)?;
                     }
                 }
-                let mut skip_comma = omit_shape && omit_position;
+                if !has_default_color_interpolation_method {
+                    if !omit_shape || !omit_position {
+                        dest.write_char(' ')?;
+                    }
+                    color_interpolation_method.to_css(dest)?;
+                }
+
+                let mut skip_comma =
+                    omit_shape && omit_position && has_default_color_interpolation_method;
                 for item in &**items {
                     if !skip_comma {
                         dest.write_str(", ")?;
@@ -566,6 +575,7 @@ where
             Gradient::Conic {
                 ref angle,
                 ref position,
+                ref color_interpolation_method,
                 ref items,
                 ..
             } => {
@@ -576,14 +586,21 @@ where
                     dest.write_str("from ")?;
                     angle.to_css(dest)?;
                     if !omit_position {
-                        dest.write_str(" ")?;
+                        dest.write_char(' ')?;
                     }
                 }
                 if !omit_position {
                     dest.write_str("at ")?;
                     position.to_css(dest)?;
                 }
-                let mut skip_comma = omit_angle && omit_position;
+                if !has_default_color_interpolation_method {
+                    if !omit_angle || !omit_position {
+                        dest.write_char(' ')?;
+                    }
+                    color_interpolation_method.to_css(dest)?;
+                }
+                let mut skip_comma =
+                    omit_angle && omit_position && has_default_color_interpolation_method;
                 for item in &**items {
                     if !skip_comma {
                         dest.write_str(", ")?;
@@ -593,7 +610,7 @@ where
                 }
             },
         }
-        dest.write_str(")")
+        dest.write_char(')')
     }
 }
 
