@@ -13,53 +13,62 @@ use super::generics::grid::{GridLine as GenericGridLine, TrackBreadth as Generic
 use super::generics::grid::{TrackList as GenericTrackList, TrackSize as GenericTrackSize};
 use super::generics::transform::IsParallelTo;
 use super::generics::{self, GreaterThanOrEqualToOne, NonNegative};
-use super::{CSSFloat, CSSInteger, Either, None_};
+use super::{CSSFloat, CSSInteger};
 use crate::context::QuirksMode;
 use crate::parser::{Parse, ParserContext};
-use crate::values::serialize_atom_identifier;
 use crate::values::specified::calc::CalcNode;
+use crate::values::{serialize_atom_identifier, serialize_number, AtomString};
 use crate::{Atom, Namespace, One, Prefix, Zero};
 use cssparser::{Parser, Token};
-use std::f32;
 use std::fmt::{self, Write};
 use std::ops::Add;
 use style_traits::values::specified::AllowedNumericType;
 use style_traits::{CssWriter, ParseError, SpecifiedValueInfo, StyleParseErrorKind, ToCss};
 
-#[cfg(feature = "gecko")]
-pub use self::align::{AlignContent, AlignItems, AlignSelf, AlignTracks, ContentDistribution};
-#[cfg(feature = "gecko")]
-pub use self::align::{JustifyContent, JustifyItems, JustifySelf, JustifyTracks, SelfAlignment};
+pub use self::align::{AlignContent, AlignItems, AlignSelf, ContentDistribution};
+pub use self::align::{JustifyContent, JustifyItems, JustifySelf, SelfAlignment};
 pub use self::angle::{AllowUnitlessZeroAngle, Angle};
+pub use self::animation::{
+    AnimationComposition, AnimationDirection, AnimationDuration, AnimationFillMode,
+    AnimationIterationCount, AnimationName, AnimationPlayState, AnimationTimeline, ScrollAxis,
+    TimelineName, TransitionBehavior, TransitionProperty, ViewTimelineInset, ViewTransitionClass,
+    ViewTransitionName,
+};
 pub use self::background::{BackgroundRepeat, BackgroundSize};
 pub use self::basic_shape::FillRule;
-pub use self::border::{BorderCornerRadius, BorderImageSlice, BorderImageWidth};
-pub use self::border::{BorderImageRepeat, BorderImageSideWidth};
-pub use self::border::{BorderRadius, BorderSideWidth, BorderSpacing, BorderStyle};
-pub use self::box_::{AnimationIterationCount, AnimationName, AnimationTimeline, Contain, Display};
-pub use self::box_::{Appearance, BreakBetween, BreakWithin, ContainerName, ContainerType};
-pub use self::box_::{Clear, ContentVisibility, Float, Overflow, OverflowAnchor};
-pub use self::box_::{OverflowClipBox, OverscrollBehavior, Perspective, Resize, ScrollbarGutter};
-pub use self::box_::{ScrollSnapAlign, ScrollSnapAxis, ScrollSnapStrictness, ScrollSnapType};
-pub use self::box_::{TouchAction, TransitionProperty, VerticalAlign, WillChange};
-pub use self::color::{Color, ColorOrAuto, ColorPropertyValue, ColorScheme, PrintColorAdjust};
+pub use self::border::{
+    BorderCornerRadius, BorderImageRepeat, BorderImageSideWidth, BorderImageSlice,
+    BorderImageWidth, BorderRadius, BorderSideWidth, BorderSpacing, BorderStyle, LineWidth,
+};
+pub use self::box_::{
+    Appearance, BaselineSource, BreakBetween, BreakWithin, Clear, Contain, ContainIntrinsicSize,
+    ContainerName, ContainerType, ContentVisibility, Display, Float, LineClamp, Overflow,
+    OverflowAnchor, OverflowClipBox, OverscrollBehavior, Perspective, PositionProperty, Resize,
+    ScrollSnapAlign, ScrollSnapAxis, ScrollSnapStop, ScrollSnapStrictness, ScrollSnapType,
+    ScrollbarGutter, TouchAction, VerticalAlign, WillChange, WillChangeBits, WritingModeProperty,
+    Zoom,
+};
+pub use self::color::{
+    Color, ColorOrAuto, ColorPropertyValue, ColorScheme, ForcedColorAdjust, PrintColorAdjust,
+};
 pub use self::column::ColumnCount;
 pub use self::counters::{Content, ContentItem, CounterIncrement, CounterReset, CounterSet};
 pub use self::easing::TimingFunction;
 pub use self::effects::{BoxShadow, Filter, SimpleShadow};
 pub use self::flex::FlexBasis;
-pub use self::font::{FontFamily, FontLanguageOverride, FontStyle};
+pub use self::font::{FontFamily, FontLanguageOverride, FontPalette, FontStyle};
 pub use self::font::{FontFeatureSettings, FontVariantLigatures, FontVariantNumeric};
-pub use self::font::{FontSize, FontSizeAdjust, FontSizeKeyword, FontStretch, FontSynthesis};
+pub use self::font::{
+    FontSize, FontSizeAdjust, FontSizeAdjustFactor, FontSizeKeyword, FontStretch, FontSynthesis, FontSynthesisStyle,
+};
 pub use self::font::{FontVariantAlternates, FontWeight};
-pub use self::font::{FontVariantEastAsian, FontVariationSettings};
-pub use self::font::{MathDepth, MozScriptMinSize, MozScriptSizeMultiplier, XLang, XTextZoom};
-pub use self::image::{EndingShape as GradientEndingShape, Gradient};
-pub use self::image::{Image, ImageRendering, MozImageRect};
-pub use self::length::{AbsoluteLength, CalcLengthPercentage, CharacterWidth};
+pub use self::font::{FontVariantEastAsian, FontVariationSettings, LineHeight};
+pub use self::font::{MathDepth, MozScriptMinSize, MozScriptSizeMultiplier, XLang, XTextScale};
+pub use self::image::{EndingShape as GradientEndingShape, Gradient, Image, ImageRendering};
+pub use self::length::{AbsoluteLength, AnchorSizeFunction, CalcLengthPercentage, CharacterWidth};
 pub use self::length::{FontRelativeLength, Length, LengthOrNumber, NonNegativeLengthOrNumber};
 pub use self::length::{LengthOrAuto, LengthPercentage, LengthPercentageOrAuto};
-pub use self::length::{MaxSize, Size};
+pub use self::length::{Margin, MaxSize, Size};
 pub use self::length::{NoCalcLength, ViewportPercentageLength, ViewportVariant};
 pub use self::length::{
     NonNegativeLength, NonNegativeLengthPercentage, NonNegativeLengthPercentageOrAuto,
@@ -67,41 +76,52 @@ pub use self::length::{
 #[cfg(feature = "gecko")]
 pub use self::list::ListStyleType;
 pub use self::list::Quotes;
-pub use self::motion::{OffsetPath, OffsetRotate};
+pub use self::motion::{OffsetPath, OffsetPosition, OffsetRotate};
 pub use self::outline::OutlineStyle;
-pub use self::page::{PageOrientation, PageName, PageSize, PaperSize};
+pub use self::page::{PageName, PageOrientation, PageSize, PageSizeOrientation, PaperSize};
 pub use self::percentage::{NonNegativePercentage, Percentage};
+pub use self::position::AnchorFunction;
+pub use self::position::AnchorName;
+pub use self::position::AnchorScope;
 pub use self::position::AspectRatio;
-pub use self::position::{
-    GridAutoFlow, GridTemplateAreas, MasonryAutoFlow, Position, PositionOrAuto,
-};
+pub use self::position::Inset;
+pub use self::position::PositionAnchor;
+pub use self::position::PositionTryFallbacks;
+pub use self::position::PositionTryOrder;
+pub use self::position::PositionVisibility;
+pub use self::position::{GridAutoFlow, GridTemplateAreas, Position, PositionOrAuto};
+pub use self::position::{MasonryAutoFlow, MasonryItemOrder, MasonryPlacement};
+pub use self::position::{PositionArea, PositionAreaKeyword};
 pub use self::position::{PositionComponent, ZIndex};
 pub use self::ratio::Ratio;
 pub use self::rect::NonNegativeLengthOrNumberRect;
 pub use self::resolution::Resolution;
 pub use self::svg::{DProperty, MozContextProperties};
 pub use self::svg::{SVGLength, SVGOpacity, SVGPaint};
-pub use self::svg::{SVGPaintOrder, SVGStrokeDashArray, SVGWidth};
+pub use self::svg::{SVGPaintOrder, SVGStrokeDashArray, SVGWidth, VectorEffect};
 pub use self::svg_path::SVGPathData;
-pub use self::text::HyphenateCharacter;
+pub use self::text::{HyphenateCharacter, HyphenateLimitChars};
 pub use self::text::RubyPosition;
 pub use self::text::TextAlignLast;
 pub use self::text::TextUnderlinePosition;
-pub use self::text::{InitialLetter, LetterSpacing, LineBreak, LineHeight, TextAlign};
+pub use self::text::{InitialLetter, LetterSpacing, LineBreak, TextAlign, TextIndent};
 pub use self::text::{OverflowWrap, TextEmphasisPosition, TextEmphasisStyle, WordBreak};
 pub use self::text::{TextAlignKeyword, TextDecorationLine, TextOverflow, WordSpacing};
 pub use self::text::{TextDecorationLength, TextDecorationSkipInk, TextJustify, TextTransform};
 pub use self::time::Time;
 pub use self::transform::{Rotate, Scale, Transform};
-pub use self::transform::{TransformOrigin, TransformStyle, Translate};
+pub use self::transform::{TransformBox, TransformOrigin, TransformStyle, Translate};
 #[cfg(feature = "gecko")]
 pub use self::ui::CursorImage;
-pub use self::ui::{Cursor, MozForceBrokenImageIcon, UserSelect};
+pub use self::ui::{
+    BoolInteger, Cursor, Inert, MozTheme, PointerEvents, ScrollbarColor, UserFocus, UserInput,
+    UserSelect,
+};
 pub use super::generics::grid::GridTemplateComponent as GenericGridTemplateComponent;
 
-#[cfg(feature = "gecko")]
 pub mod align;
 pub mod angle;
+pub mod animation;
 pub mod background;
 pub mod basic_shape;
 pub mod border;
@@ -115,10 +135,9 @@ pub mod easing;
 pub mod effects;
 pub mod flex;
 pub mod font;
-#[cfg(feature = "gecko")]
-pub mod gecko;
 pub mod grid;
 pub mod image;
+pub mod intersection_observer;
 pub mod length;
 pub mod list;
 pub mod motion;
@@ -190,15 +209,15 @@ fn parse_number_with_clamping_mode<'i, 't>(
     match *input.next()? {
         Token::Number { value, .. } if clamping_mode.is_ok(context.parsing_mode, value) => {
             Ok(Number {
-                value: value.min(f32::MAX).max(f32::MIN),
+                value,
                 calc_clamping_mode: None,
             })
         },
         Token::Function(ref name) => {
-            let function = CalcNode::math_function(name, location)?;
-            let result = CalcNode::parse_number(context, input, function)?;
+            let function = CalcNode::math_function(context, name, location)?;
+            let value = CalcNode::parse_number(context, input, function)?;
             Ok(Number {
-                value: result.min(f32::MAX).max(f32::MIN),
+                value,
                 calc_clamping_mode: Some(clamping_mode),
             })
         },
@@ -209,7 +228,7 @@ fn parse_number_with_clamping_mode<'i, 't>(
 /// A CSS `<number>` specified value.
 ///
 /// https://drafts.csswg.org/css-values-3/#number-value
-#[derive(Clone, Copy, Debug, MallocSizeOf, PartialEq, PartialOrd, ToShmem)]
+#[derive(Clone, Copy, Debug, MallocSizeOf, PartialOrd, ToShmem)]
 pub struct Number {
     /// The numeric value itself.
     value: CSSFloat,
@@ -227,8 +246,19 @@ impl Parse for Number {
     }
 }
 
+impl PartialEq<Number> for Number {
+    fn eq(&self, other: &Number) -> bool {
+        if self.calc_clamping_mode != other.calc_clamping_mode {
+            return false;
+        }
+
+        self.value == other.value || (self.value.is_nan() && other.value.is_nan())
+    }
+}
+
 impl Number {
     /// Returns a new number with the value `val`.
+    #[inline]
     fn new_with_clamping_mode(
         value: CSSFloat,
         calc_clamping_mode: Option<AllowedNumericType>,
@@ -245,6 +275,7 @@ impl Number {
     }
 
     /// Returns a new number with the value `val`.
+    #[inline]
     pub fn new(val: CSSFloat) -> Self {
         Self::new_with_clamping_mode(val, None)
     }
@@ -258,8 +289,12 @@ impl Number {
     /// Returns the numeric value, clamped if needed.
     #[inline]
     pub fn get(&self) -> f32 {
-        self.calc_clamping_mode
-            .map_or(self.value, |mode| mode.clamp(self.value))
+        crate::values::normalize(
+            self.calc_clamping_mode
+                .map_or(self.value, |mode| mode.clamp(self.value)),
+        )
+        .min(f32::MAX)
+        .max(f32::MIN)
     }
 
     #[allow(missing_docs)]
@@ -310,14 +345,7 @@ impl ToCss for Number {
     where
         W: Write,
     {
-        if self.calc_clamping_mode.is_some() {
-            dest.write_str("calc(")?;
-        }
-        self.value.to_css(dest)?;
-        if self.calc_clamping_mode.is_some() {
-            dest.write_str(")")?;
-        }
-        Ok(())
+        serialize_number(self.value, self.calc_clamping_mode.is_some(), dest)
     }
 }
 
@@ -408,6 +436,18 @@ impl NonNegativeNumber {
     }
 }
 
+/// An Integer which is >= 0.
+pub type NonNegativeInteger = NonNegative<Integer>;
+
+impl Parse for NonNegativeInteger {
+    fn parse<'i, 't>(
+        context: &ParserContext,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<Self, ParseError<'i>> {
+        Ok(NonNegative(Integer::parse_non_negative(context, input)?))
+    }
+}
+
 /// A Number which is >= 1.0.
 pub type GreaterThanOrEqualToOneNumber = GreaterThanOrEqualToOne<Number>;
 
@@ -489,6 +529,12 @@ impl NonNegativeNumberOrPercentage {
     pub fn hundred_percent() -> Self {
         NonNegative(NumberOrPercentage::Percentage(Percentage::hundred()))
     }
+
+    /// Return a particular number.
+    #[inline]
+    pub fn new_number(n: f32) -> Self {
+        NonNegative(NumberOrPercentage::Number(Number::new(n)))
+    }
 }
 
 impl Parse for NonNegativeNumberOrPercentage {
@@ -544,13 +590,17 @@ impl ToComputedValue for Opacity {
     }
 }
 
-/// A specified `<integer>`, optionally coming from a `calc()` expression.
+/// A specified `<integer>`, either a simple integer value or a calc expression.
+/// Note that a calc expression may not actually be an integer; it will be rounded
+/// at computed-value time.
 ///
 /// <https://drafts.csswg.org/css-values/#integers>
-#[derive(Clone, Copy, Debug, Eq, MallocSizeOf, PartialEq, PartialOrd, ToShmem)]
-pub struct Integer {
-    value: CSSInteger,
-    was_calc: bool,
+#[derive(Clone, Copy, Debug, MallocSizeOf, PartialEq, PartialOrd, ToShmem)]
+pub enum Integer {
+    /// A literal integer value.
+    Literal(CSSInteger),
+    /// A calc expression, whose value will be rounded later if necessary.
+    Calc(CSSFloat),
 }
 
 impl Zero for Integer {
@@ -561,7 +611,7 @@ impl Zero for Integer {
 
     #[inline]
     fn is_zero(&self) -> bool {
-        self.value() == 0
+        *self == 0
     }
 }
 
@@ -573,7 +623,7 @@ impl One for Integer {
 
     #[inline]
     fn is_one(&self) -> bool {
-        self.value() == 1
+        *self == 1
     }
 }
 
@@ -586,23 +636,20 @@ impl PartialEq<i32> for Integer {
 impl Integer {
     /// Trivially constructs a new `Integer` value.
     pub fn new(val: CSSInteger) -> Self {
-        Integer {
-            value: val,
-            was_calc: false,
-        }
+        Self::Literal(val)
     }
 
-    /// Returns the integer value associated with this value.
+    /// Returns the (rounded) integer value associated with this value.
     pub fn value(&self) -> CSSInteger {
-        self.value
+        match *self {
+            Self::Literal(i) => i,
+            Self::Calc(n) => (n + 0.5).floor() as CSSInteger,
+        }
     }
 
     /// Trivially constructs a new integer value from a `calc()` expression.
-    fn from_calc(val: CSSInteger) -> Self {
-        Integer {
-            value: val,
-            was_calc: true,
-        }
+    fn from_calc(val: CSSFloat) -> Self {
+        Self::Calc(val)
     }
 }
 
@@ -617,8 +664,8 @@ impl Parse for Integer {
                 int_value: Some(v), ..
             } => Ok(Integer::new(v)),
             Token::Function(ref name) => {
-                let function = CalcNode::math_function(name, location)?;
-                let result = CalcNode::parse_integer(context, input, function)?;
+                let function = CalcNode::math_function(context, name, location)?;
+                let result = CalcNode::parse_number(context, input, function)?;
                 Ok(Integer::from_calc(result))
             },
             ref t => Err(location.new_unexpected_token_error(t.clone())),
@@ -667,7 +714,7 @@ impl ToComputedValue for Integer {
 
     #[inline]
     fn to_computed_value(&self, _: &Context) -> i32 {
-        self.value
+        self.value()
     }
 
     #[inline]
@@ -681,14 +728,14 @@ impl ToCss for Integer {
     where
         W: Write,
     {
-        if self.was_calc {
-            dest.write_str("calc(")?;
+        match *self {
+            Integer::Literal(i) => i.to_css(dest),
+            Integer::Calc(n) => {
+                dest.write_str("calc(")?;
+                n.to_css(dest)?;
+                dest.write_char(')')
+            }
         }
-        self.value.to_css(dest)?;
-        if self.was_calc {
-            dest.write_str(")")?;
-        }
-        Ok(())
     }
 }
 
@@ -707,9 +754,6 @@ impl Parse for PositiveInteger {
     }
 }
 
-/// A specified positive `<integer>` value or `none`.
-pub type PositiveIntegerOrNone = Either<PositiveInteger, None_>;
-
 /// The specified value of a grid `<track-breadth>`
 pub type TrackBreadth = GenericTrackBreadth<LengthPercentage>;
 
@@ -849,6 +893,8 @@ pub struct Attr {
     pub namespace_url: Namespace,
     /// Attribute name
     pub attribute: Atom,
+    /// Fallback value
+    pub fallback: AtomString,
 }
 
 impl Parse for Attr {
@@ -863,12 +909,33 @@ impl Parse for Attr {
 
 /// Get the Namespace for a given prefix from the namespace map.
 fn get_namespace_for_prefix(prefix: &Prefix, context: &ParserContext) -> Option<Namespace> {
-    context
-        .namespaces
-        .as_ref()?
-        .prefixes
-        .get(prefix)
-        .map(|x| x.clone())
+    context.namespaces.prefixes.get(prefix).cloned()
+}
+
+/// Try to parse a namespace and return it if parsed, or none if there was not one present
+fn parse_namespace<'i, 't>(
+    context: &ParserContext,
+    input: &mut Parser<'i, 't>,
+) -> Result<(Prefix, Namespace), ParseError<'i>> {
+    let ns_prefix = match input.next()? {
+        Token::Ident(ref prefix) => Some(Prefix::from(prefix.as_ref())),
+        Token::Delim('|') => None,
+        _ => return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError)),
+    };
+
+    if ns_prefix.is_some() && !matches!(*input.next_including_whitespace()?, Token::Delim('|')) {
+        return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError));
+    }
+
+    if let Some(prefix) = ns_prefix {
+        let ns = match get_namespace_for_prefix(&prefix, context) {
+            Some(ns) => ns,
+            None => return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError)),
+        };
+        Ok((prefix, ns))
+    } else {
+        Ok((Prefix::default(), Namespace::default()))
+    }
 }
 
 impl Attr {
@@ -878,54 +945,39 @@ impl Attr {
         context: &ParserContext,
         input: &mut Parser<'i, 't>,
     ) -> Result<Attr, ParseError<'i>> {
-        // Syntax is `[namespace? `|`]? ident`
-        // no spaces allowed
-        let first = input.try_parse(|i| i.expect_ident_cloned()).ok();
-        if let Ok(token) = input.try_parse(|i| i.next_including_whitespace().map(|t| t.clone())) {
-            match token {
-                Token::Delim('|') => {
-                    let location = input.current_source_location();
-                    // must be followed by an ident
-                    let second_token = match *input.next_including_whitespace()? {
-                        Token::Ident(ref second) => second,
-                        ref t => return Err(location.new_unexpected_token_error(t.clone())),
-                    };
+        // Syntax is `[namespace? '|']? ident [',' fallback]?`
+        let namespace = input
+            .try_parse(|input| parse_namespace(context, input))
+            .ok();
+        let namespace_is_some = namespace.is_some();
+        let (namespace_prefix, namespace_url) = namespace.unwrap_or_default();
 
-                    let (namespace_prefix, namespace_url) = if let Some(ns) = first {
-                        let prefix = Prefix::from(ns.as_ref());
-                        let ns = match get_namespace_for_prefix(&prefix, context) {
-                            Some(ns) => ns,
-                            None => {
-                                return Err(location
-                                    .new_custom_error(StyleParseErrorKind::UnspecifiedError));
-                            },
-                        };
-                        (prefix, ns)
-                    } else {
-                        (Prefix::default(), Namespace::default())
-                    };
-                    return Ok(Attr {
-                        namespace_prefix,
-                        namespace_url,
-                        attribute: Atom::from(second_token.as_ref()),
-                    });
-                },
-                // In the case of attr(foobar    ) we don't want to error out
-                // because of the trailing whitespace.
-                Token::WhiteSpace(..) => {},
-                ref t => return Err(input.new_unexpected_token_error(t.clone())),
+        // If there is a namespace, ensure no whitespace following '|'
+        let attribute = Atom::from(if namespace_is_some {
+            let location = input.current_source_location();
+            match *input.next_including_whitespace()? {
+                Token::Ident(ref ident) => ident.as_ref(),
+                ref t => return Err(location.new_unexpected_token_error(t.clone())),
             }
-        }
-
-        if let Some(first) = first {
-            Ok(Attr {
-                namespace_prefix: Prefix::default(),
-                namespace_url: Namespace::default(),
-                attribute: Atom::from(first.as_ref()),
-            })
         } else {
-            Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
-        }
+            input.expect_ident()?.as_ref()
+        });
+
+        // Fallback will always be a string value for now as we do not support
+        // attr() types yet.
+        let fallback = input
+            .try_parse(|input| -> Result<AtomString, ParseError<'i>> {
+                input.expect_comma()?;
+                Ok(input.expect_string()?.as_ref().into())
+            })
+            .unwrap_or_default();
+
+        Ok(Attr {
+            namespace_prefix,
+            namespace_url,
+            attribute,
+            fallback,
+        })
     }
 }
 
@@ -937,9 +989,15 @@ impl ToCss for Attr {
         dest.write_str("attr(")?;
         if !self.namespace_prefix.is_empty() {
             serialize_atom_identifier(&self.namespace_prefix, dest)?;
-            dest.write_str("|")?;
+            dest.write_char('|')?;
         }
         serialize_atom_identifier(&self.attribute, dest)?;
-        dest.write_str(")")
+
+        if !self.fallback.is_empty() {
+            dest.write_str(", ")?;
+            self.fallback.to_css(dest)?;
+        }
+
+        dest.write_char(')')
     }
 }
