@@ -7,22 +7,18 @@ Templates provide a way of modifying the task definition of selected tasks.
 They are added to 'try_task_config.json' and processed by the transforms.
 """
 
-
 import json
 import os
 import pathlib
 import subprocess
 import sys
-from abc import ABCMeta, abstractmethod, abstractproperty
+from abc import ABCMeta, abstractmethod
 from argparse import SUPPRESS, Action
-from contextlib import contextmanager
 from textwrap import dedent
 
 import mozpack.path as mozpath
 import requests
-import six
 from mozbuild.base import BuildEnvironmentNotFoundException, MozbuildObject
-from mozversioncontrol import Repository
 from taskgraph.util import taskcluster
 
 from .tasks import resolve_tests_by_suite
@@ -32,24 +28,6 @@ here = pathlib.Path(__file__).parent
 build = MozbuildObject.from_environment(cwd=str(here))
 
 
-@contextmanager
-def try_config_commit(vcs: Repository, commit_message: str):
-    """Context manager that creates and removes a try config commit."""
-    # Add the `try_task_config.json` file if it exists.
-    try_task_config_path = pathlib.Path(build.topsrcdir) / "try_task_config.json"
-    if try_task_config_path.exists():
-        vcs.add_remove_files("try_task_config.json")
-
-    try:
-        # Create a try config commit.
-        vcs.create_try_commit(commit_message)
-
-        yield
-    finally:
-        # Revert the try config commit.
-        vcs.remove_current_commit()
-
-
 class ParameterConfig:
     __metaclass__ = ABCMeta
 
@@ -61,7 +39,8 @@ class ParameterConfig:
             action = parser.add_argument(*cli, **kwargs)
             self.dests.add(action.dest)
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def arguments(self):
         pass
 
@@ -73,6 +52,19 @@ class ParameterConfig:
         pass
 
 
+class TargetTasksMethod(ParameterConfig):
+    arguments = [
+        [
+            ["--target-tasks-method"],
+            {"help": "Custom target tasks method to use."},
+        ],
+    ]
+
+    def get_parameters(self, target_tasks_method: str, **kwargs):
+        if target_tasks_method:
+            return {"target_tasks_method": target_tasks_method}
+
+
 class TryConfig(ParameterConfig):
     @abstractmethod
     def try_config(self, **kwargs) -> dict:
@@ -163,14 +155,12 @@ class Pernosco(TryConfig):
                 if not address.endswith("@mozilla.com"):
                     print(
                         dedent(
-                            """\
+                            f"""\
                         Pernosco requires a Mozilla e-mail address to view its reports. Please
                         push to try with an @mozilla.com address to use --pernosco.
 
-                            Current user: {}
-                    """.format(
-                                address
-                            )
+                            Current user: {address}
+                    """
                         )
                     )
                     sys.exit(1)
@@ -224,7 +214,7 @@ class Path(TryConfig):
 
         for p in paths:
             if not os.path.exists(p):
-                print("error: '{}' is not a valid path.".format(p), file=sys.stderr)
+                print(f"error: '{p}' is not a valid path.", file=sys.stderr)
                 sys.exit(1)
 
         paths = [
@@ -233,9 +223,30 @@ class Path(TryConfig):
         ]
         return {
             "env": {
-                "MOZHARNESS_TEST_PATHS": six.ensure_text(
-                    json.dumps(resolve_tests_by_suite(paths))
-                ),
+                "MOZHARNESS_TEST_PATHS": json.dumps(resolve_tests_by_suite(paths)),
+            }
+        }
+
+
+class Tag(TryConfig):
+    arguments = [
+        [
+            ["--tag"],
+            {
+                "action": "append",
+                "default": [],
+                "help": "Run tests matching the specified tag.",
+            },
+        ],
+    ]
+
+    def try_config(self, tag, **kwargs):
+        if not tag:
+            return
+
+        return {
+            "env": {
+                "MOZHARNESS_TEST_TAG": json.dumps(tag),
             }
         }
 
@@ -329,15 +340,15 @@ class RangeAction(Action):
     def __init__(self, min, max, *args, **kwargs):
         self.min = min
         self.max = max
-        kwargs["metavar"] = "[{}-{}]".format(self.min, self.max)
+        kwargs["metavar"] = f"[{self.min}-{self.max}]"
         super().__init__(*args, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
         name = option_string or self.dest
         if values < self.min:
-            parser.error("{} can not be less than {}".format(name, self.min))
+            parser.error(f"{name} can not be less than {self.min}")
         if values > self.max:
-            parser.error("{} can not be more than {}".format(name, self.max))
+            parser.error(f"{name} can not be more than {self.max}")
         setattr(namespace, self.dest, values)
 
 
@@ -422,7 +433,10 @@ class GeckoProfile(TryConfig):
                 "dest": "profile",
                 "action": "store_true",
                 "default": False,
-                "help": "Create and upload a gecko profile during talos/raptor tasks.",
+                "help": (
+                    "Create and upload a gecko profile during talos/raptor tasks. "
+                    "Copy paste the parameters used in this profiling run directly from about:profiling in Nightly."
+                ),
             },
         ],
         [
@@ -598,41 +612,65 @@ class WorkerOverrides(TryConfig):
         from gecko_taskgraph.util.workertypes import get_worker_type
         from taskgraph.config import load_graph_config
 
+        root = build.topsrcdir
+        root = os.path.join(root, "taskcluster")
+        graph_config = load_graph_config(root)
+
         overrides = {}
         if worker_overrides:
             for override in worker_overrides:
                 alias, worker_pool = override.split("=", 1)
                 if alias in overrides:
                     print(
-                        "Can't override worker alias {alias} more than once. "
-                        "Already set to use {previous}, but also asked to use {new}.".format(
-                            alias=alias, previous=overrides[alias], new=worker_pool
-                        )
+                        f"Can't override worker alias {alias} more than once. "
+                        f"Already set to use {overrides[alias]}, but also asked to use {worker_pool}."
                     )
                     sys.exit(1)
                 overrides[alias] = worker_pool
 
+                try:
+                    provisioner, worker_type = get_worker_type(
+                        graph_config, worker_type=alias, parameters={"level": "1"}
+                    )
+                except KeyError:
+                    print(
+                        f"Invalid worker type {alias}, use a value that matches below (limited to b-*, t-*, win*):"
+                    )
+                    root_alias = alias.strip("gecko-")
+                    root_alias = root_alias.strip("t-")
+                    root_alias = root_alias.split("-")[0]
+                    possible_matches = []
+                    for item in graph_config["workers"]["aliases"]:
+                        if root_alias in item:
+                            possible_matches.append(item)
+                        if (
+                            item.startswith("t-")
+                            or item.startswith("b-")
+                            or item.startswith("win")
+                        ):
+                            print(f"{item}")
+
+                    print("")
+                    if len(possible_matches) == 1:
+                        print(f"did you mean: {possible_matches[0]}")
+                    elif len(possible_matches) > 1:
+                        print(f"did you mean one of these {possible_matches}")
+                    sys.exit(1)
+
         if worker_suffixes:
-            root = build.topsrcdir
-            root = os.path.join(root, "taskcluster")
-            graph_config = load_graph_config(root)
             for worker_suffix in worker_suffixes:
                 alias, suffix = worker_suffix.split("=", 1)
                 if alias in overrides:
                     print(
-                        "Can't override worker alias {alias} more than once. "
-                        "Already set to use {previous}, but also asked "
-                        "to add suffix {suffix}.".format(
-                            alias=alias, previous=overrides[alias], suffix=suffix
-                        )
+                        f"Can't override worker alias {alias} more than once. "
+                        f"Already set to use {overrides[alias]}, but also asked "
+                        f"to add suffix {suffix}."
                     )
                     sys.exit(1)
                 provisioner, worker_type = get_worker_type(
                     graph_config, worker_type=alias, parameters={"level": "1"}
                 )
-                overrides[alias] = "{provisioner}/{worker_type}{suffix}".format(
-                    provisioner=provisioner, worker_type=worker_type, suffix=suffix
-                )
+                overrides[alias] = f"{provisioner}/{worker_type}{suffix}"
 
         retVal = {}
         if worker_types:
@@ -653,8 +691,10 @@ all_task_configs = {
     "gecko-profile": GeckoProfile,
     "new-test-config": NewConfig,
     "path": Path,
+    "test-tag": Tag,
     "pernosco": Pernosco,
     "rebuild": Rebuild,
     "routes": Routes,
+    "target-tasks-method": TargetTasksMethod,
     "worker-overrides": WorkerOverrides,
 }
