@@ -2,8 +2,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import, print_function
-
 import os.path
 import sys
 
@@ -19,23 +17,23 @@ if (len(cert_pairs) % 2) != 0:
     print("Certs must be provided in pairs")
     sys.exit(1)
 
-for find_cert, replace_cert in zip(*[iter(cert_pairs)]*2):
+for find_cert, replace_cert in zip(*[iter(cert_pairs)] * 2):
     find = open(os.path.join(cert_dir, find_cert), "rb").read()
     replace = open(os.path.join(cert_dir, replace_cert), "rb").read()
-    print("Looking for {}...".format(find_cert))
+    print(f"Looking for {find_cert}...")
     if find in new_updater:
-        print("Replacing {} with {}".format(find_cert, replace_cert))
+        print(f"Replacing {find_cert} with {replace_cert}")
         new_updater = new_updater.replace(find, replace)
     else:
-        print("Didn't find {}...".format(find_cert))
+        print(f"Didn't find {find_cert}...")
 
 if len(updater_data) != len(new_updater):
     print(
-        "WARNING: new updater is not the same length as the old one (old: {}, new: {})".format(
-            len(updater_data), len(new_updater)))
+        f"WARNING: new updater is not the same length as the old one (old: {len(updater_data)}, new: {len(new_updater)})"
+    )
 
 if updater_data == new_updater:
     print("WARNING: updater is unchanged")
 
-with open(outfile, 'wb+') as f:
+with open(outfile, "wb+") as f:
     f.write(new_updater)
