@@ -7,17 +7,25 @@
 #ifndef NSEXPIRATIONTRACKER_H_
 #define NSEXPIRATIONTRACKER_H_
 
-#include "mozilla/Logging.h"
+#include <cstring>
+#include "MainThreadUtils.h"
+#include "nsAlgorithm.h"
+#include "nsDebug.h"
 #include "nsTArray.h"
 #include "nsITimer.h"
 #include "nsCOMPtr.h"
-#include "nsAutoPtr.h"
-#include "nsComponentManagerUtils.h"
 #include "nsIEventTarget.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
+#include "nsISupports.h"
+#include "nsIThread.h"
 #include "nsThreadUtils.h"
-#include "mozilla/Attributes.h"
+#include "nscore.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/Likely.h"
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/RefCountType.h"
+#include "mozilla/RefPtr.h"
 #include "mozilla/Services.h"
 
 /**
@@ -31,7 +39,7 @@ struct nsExpirationState {
     MAX_INDEX_IN_GENERATION = (1U << 28) - 1
   };
 
-  nsExpirationState() : mGeneration(NOT_TRACKED) {}
+  nsExpirationState() : mGeneration(NOT_TRACKED), mIndexInGeneration(0) {}
   bool IsTracked() { return mGeneration != NOT_TRACKED; }
 
   /**
@@ -86,6 +94,9 @@ struct nsExpirationState {
  * For creating a thread-safe tracker, you can define a subclass inheriting this
  * base class and specialize the Mutex and AutoLock to be used.
  *
+ * For an example of using ExpirationTrackerImpl with a DataMutex
+ * @see mozilla::gfx::GradientCache.
+ *
  */
 template <typename T, uint32_t K, typename Mutex, typename AutoLock>
 class ExpirationTrackerImpl {
@@ -110,28 +121,22 @@ class ExpirationTrackerImpl {
         mEventTarget(aEventTarget) {
     static_assert(K >= 2 && K <= nsExpirationState::NOT_TRACKED,
                   "Unsupported number of generations (must be 2 <= K <= 15)");
-    MOZ_ASSERT(NS_IsMainThread());
-    if (mEventTarget) {
-      bool current = false;
-      // NOTE: The following check+crash could be condensed into a
-      // MOZ_RELEASE_ASSERT, but that triggers a segfault during compilation in
-      // clang 3.8. Once we don't have to care about clang 3.8 anymore, though,
-      // we can convert to MOZ_RELEASE_ASSERT here.
-      if (MOZ_UNLIKELY(NS_FAILED(mEventTarget->IsOnCurrentThread(&current)) ||
-                       !current)) {
-        MOZ_CRASH("Provided event target must be on the main thread");
-      }
+    // If we are not initialized on the main thread, the owner is responsible
+    // for dealing with memory pressure events.
+    if (NS_IsMainThread()) {
+      mObserver = new ExpirationTrackerObserver();
+      mObserver->Init(this);
     }
-    mObserver = new ExpirationTrackerObserver();
-    mObserver->Init(this);
   }
 
   virtual ~ExpirationTrackerImpl() {
-    MOZ_ASSERT(NS_IsMainThread());
     if (mTimer) {
       mTimer->Cancel();
     }
-    mObserver->Destroy();
+    if (mObserver) {
+      MOZ_ASSERT(NS_IsMainThread());
+      mObserver->Destroy();
+    }
   }
 
   /**
@@ -141,13 +146,12 @@ class ExpirationTrackerImpl {
    */
   nsresult AddObjectLocked(T* aObj, const AutoLock& aAutoLock) {
     if (NS_WARN_IF(!aObj)) {
-      MOZ_DIAGNOSTIC_ASSERT(false, "Invalid object to add");
+      MOZ_DIAGNOSTIC_CRASH("Invalid object to add");
       return NS_ERROR_UNEXPECTED;
     }
     nsExpirationState* state = aObj->GetExpirationState();
     if (NS_WARN_IF(state->IsTracked())) {
-      MOZ_DIAGNOSTIC_ASSERT(false,
-                            "Tried to add an object that's already tracked");
+      MOZ_DIAGNOSTIC_CRASH("Tried to add an object that's already tracked");
       return NS_ERROR_UNEXPECTED;
     }
     nsTArray<T*>& generation = mGenerations[mNewestGeneration];
@@ -163,9 +167,9 @@ class ExpirationTrackerImpl {
         return rv;
       }
     }
-    if (!generation.AppendElement(aObj)) {
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
+    // XXX(Bug 1631371) Check if this should use a fallible operation as it
+    // pretended earlier.
+    generation.AppendElement(aObj);
     state->mGeneration = mNewestGeneration;
     state->mIndexInGeneration = index;
     return NS_OK;
@@ -176,13 +180,12 @@ class ExpirationTrackerImpl {
    */
   void RemoveObjectLocked(T* aObj, const AutoLock& aAutoLock) {
     if (NS_WARN_IF(!aObj)) {
-      MOZ_DIAGNOSTIC_ASSERT(false, "Invalid object to remove");
+      MOZ_DIAGNOSTIC_CRASH("Invalid object to remove");
       return;
     }
     nsExpirationState* state = aObj->GetExpirationState();
     if (NS_WARN_IF(!state->IsTracked())) {
-      MOZ_DIAGNOSTIC_ASSERT(false,
-                            "Tried to remove an object that's not tracked");
+      MOZ_DIAGNOSTIC_CRASH("Tried to remove an object that's not tracked");
       return;
     }
     nsTArray<T*>& generation = mGenerations[state->mGeneration];
@@ -190,12 +193,13 @@ class ExpirationTrackerImpl {
     MOZ_ASSERT(generation.Length() > index && generation[index] == aObj,
                "Object is lying about its index");
     // Move the last object to fill the hole created by removing aObj
-    uint32_t last = generation.Length() - 1;
-    T* lastObj = generation[last];
-    generation[index] = lastObj;
+    T* lastObj = generation.PopLastElement();
+    // XXX It looks weird that index might point to the element that was just
+    // removed. Is that really correct?
+    if (index < generation.Length()) {
+      generation[index] = lastObj;
+    }
     lastObj->GetExpirationState()->mIndexInGeneration = index;
-    generation.RemoveElementAt(last);
-    MOZ_ASSERT(generation.Length() == last);
     state->mGeneration = nsExpirationState::NOT_TRACKED;
     // We do not check whether we need to stop the timer here. The timer
     // will check that itself next time it fires. Checking here would not
@@ -304,7 +308,7 @@ class ExpirationTrackerImpl {
 
   friend class Iterator;
 
-  bool IsEmptyLocked(const AutoLock& aAutoLock) {
+  bool IsEmptyLocked(const AutoLock& aAutoLock) const {
     for (uint32_t i = 0; i < K; ++i) {
       if (!mGenerations[i].IsEmpty()) {
         return false;
@@ -313,6 +317,26 @@ class ExpirationTrackerImpl {
     return true;
   }
 
+  size_t Length(const AutoLock& aAutoLock) const {
+    size_t len = 0;
+    for (uint32_t i = 0; i < K; ++i) {
+      len += mGenerations[i].Length();
+    }
+    return len;
+  }
+
+  // @return The amount of memory used by this ExpirationTrackerImpl, excluding
+  // sizeof(*this). If you want to measure anything hanging off the mGenerations
+  // array, you must iterate over the elements and measure them individually;
+  // hence the "Shallow" prefix.
+  size_t ShallowSizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const {
+    size_t bytes = 0;
+    for (uint32_t i = 0; i < K; ++i) {
+      bytes += mGenerations[i].ShallowSizeOfExcludingThis(aMallocSizeOf);
+    }
+    return bytes;
+  }
+
  protected:
   /**
    * This must be overridden to catch notifications. It is called whenever
@@ -348,14 +372,14 @@ class ExpirationTrackerImpl {
    * done while still holding the lock. It will be called once after each timer
    * event, and each low memory event has been handled.
    */
-  virtual void NotifyHandlerEndLocked(const AutoLock&){};
+  virtual void NotifyHandlerEndLocked(const AutoLock&) {};
 
   /**
    * This may be overridden to perform any post-aging work that needs to be
    * done outside the lock. It will be called once after each
    * NotifyEndTransactionLocked call.
    */
-  virtual void NotifyHandlerEnd(){};
+  virtual void NotifyHandlerEnd() {};
 
   virtual Mutex& GetMutex() = 0;
 
@@ -430,17 +454,10 @@ class ExpirationTrackerImpl {
     if (mTimer || !mTimerPeriod) {
       return NS_OK;
     }
-    nsCOMPtr<nsIEventTarget> target = mEventTarget;
-    if (!target && !NS_IsMainThread()) {
-      // TimerCallback should always be run on the main thread to prevent races
-      // to the destruction of the tracker.
-      target = do_GetMainThread();
-      NS_ENSURE_STATE(target);
-    }
 
     return NS_NewTimerWithFuncCallback(
         getter_AddRefs(mTimer), TimerCallback, this, mTimerPeriod,
-        nsITimer::TYPE_REPEATING_SLACK_LOW_PRIORITY, mName, target);
+        nsITimer::TYPE_REPEATING_SLACK_LOW_PRIORITY, mName, mEventTarget);
   }
 };
 
@@ -473,12 +490,12 @@ class nsExpirationTracker
   Lock mLock;
 
   AutoLock FakeLock() {
-    MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+    NS_ASSERT_OWNINGTHREAD(nsExpirationTracker);
     return AutoLock(mLock);
   }
 
   Lock& GetMutex() override {
-    MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+    NS_ASSERT_OWNINGTHREAD(nsExpirationTracker);
     return mLock;
   }
 
@@ -495,6 +512,8 @@ class nsExpirationTracker
   void NotifyHandlerEnd() final {}
 
  protected:
+  NS_DECL_OWNINGTHREAD
+
   virtual void NotifyExpired(T* aObj) = 0;
 
  public:
@@ -503,7 +522,9 @@ class nsExpirationTracker
       : ::detail::SingleThreadedExpirationTracker<T, K>(aTimerPeriod, aName,
                                                         aEventTarget) {}
 
-  virtual ~nsExpirationTracker() {}
+  virtual ~nsExpirationTracker() {
+    NS_ASSERT_OWNINGTHREAD(nsExpirationTracker);
+  }
 
   nsresult AddObject(T* aObj) {
     return this->AddObjectLocked(aObj, FakeLock());
