@@ -5,12 +5,17 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "double-conversion/double-conversion.h"
+#include "mozilla/Assertions.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Printf.h"
 #include "mozilla/ResultExtensions.h"
 
+#include <iterator>
+#include "fmt/format.h"
+#include "fmt/xchar.h"
+
 #include "nsASCIIMask.h"
 #include "nsCharTraits.h"
 #include "nsISupports.h"
@@ -39,15 +44,6 @@ using double_conversion::DoubleToStringConverter;
 
 // ---------------------------------------------------------------------------
 
-static const char16_t gNullChar = 0;
-
-char* const nsCharTraits<char>::sEmptyBuffer =
-    (char*)const_cast<char16_t*>(&gNullChar);
-char16_t* const nsCharTraits<char16_t>::sEmptyBuffer =
-    const_cast<char16_t*>(&gNullChar);
-
-// ---------------------------------------------------------------------------
-
 static void ReleaseData(void* aData, nsAString::DataFlags aFlags) {
   if (aFlags & nsAString::DataFlags::REFCOUNTED) {
     mozilla::StringBuffer::FromData(aData)->Release();
@@ -442,6 +438,11 @@ void nsTSubstring<T>::AssignASCII(const char* aData, size_type aLength) {
   }
 }
 
+template <typename T>
+void nsTSubstring<T>::AssignASCII(const nsLiteralCString& aData) {
+  AssignASCII(aData.get(), aData.Length());
+}
+
 template <typename T>
 bool nsTSubstring<T>::AssignASCII(const char* aData, size_type aLength,
                                   const fallible_t& aFallible) {
@@ -700,23 +701,36 @@ bool nsTSubstring<T>::Replace(index_type aCutStart, size_type aCutLength,
 template <typename T>
 void nsTSubstring<T>::Replace(index_type aCutStart, size_type aCutLength,
                               const substring_tuple_type& aTuple) {
+  if (!Replace(aCutStart, aCutLength, aTuple, mozilla::fallible)) {
+    AllocFailed(this->Length() - aCutLength + aTuple.Length());
+  }
+}
+
+template <typename T>
+bool nsTSubstring<T>::Replace(index_type aCutStart, size_type aCutLength,
+                              const substring_tuple_type& aTuple,
+                              const fallible_t& aFallible) {
   const auto [isDependentOnThis, tupleLength] =
       aTuple.IsDependentOnWithLength(this->mData, this->mData + this->mLength);
 
   if (isDependentOnThis) {
     nsTAutoString<T> temp;
     if (!temp.AssignNonDependent(aTuple, tupleLength, mozilla::fallible)) {
-      AllocFailed(tupleLength);
+      return false;
     }
-    Replace(aCutStart, aCutLength, temp);
-    return;
+    return Replace(aCutStart, aCutLength, temp, aFallible);
   }
 
   aCutStart = XPCOM_MIN(aCutStart, this->Length());
 
-  if (ReplacePrep(aCutStart, aCutLength, tupleLength) && tupleLength > 0) {
+  if (!ReplacePrep(aCutStart, aCutLength, tupleLength)) {
+    return false;
+  }
+
+  if (tupleLength > 0) {
     aTuple.WriteTo(this->mData + aCutStart, tupleLength);
   }
+  return true;
 }
 
 template <typename T>
@@ -803,6 +817,11 @@ void nsTSubstring<T>::AppendASCII(const char* aData, size_type aLength) {
   }
 }
 
+template <typename T>
+void nsTSubstring<T>::AppendASCII(const nsLiteralCString& aData) {
+  AppendASCII(aData.get(), aData.Length());
+}
+
 template <typename T>
 bool nsTSubstring<T>::AppendASCII(const char* aData,
                                   const fallible_t& aFallible) {
@@ -1024,8 +1043,7 @@ void nsTSubstring<T>::StripChars(const char_type* aChars) {
     char_type theChar = *from++;
     const char_type* test = aChars;
 
-    for (; *test && *test != theChar; ++test)
-      ;
+    for (; *test && *test != theChar; ++test);
 
     if (!*test) {
       // Not stripped, copy this char.
@@ -1083,6 +1101,73 @@ void nsTSubstring<T>::StripCRLF() {
   StripTaggedASCII(mozilla::ASCIIMask::MaskCRLF());
 }
 
+// An adapter type for a `nsTSubstring<T>` to provide a std::string-like
+// interface for `{fmt}`.
+template <typename T>
+class nsTSubstringStdCollectionAdapter {
+ public:
+  using value_type = T;
+
+  explicit nsTSubstringStdCollectionAdapter(nsTSubstring<T>& aString)
+      : mSize(aString.Length()), mHandle(InfallibleBulkWrite(aString)) {}
+
+  ~nsTSubstringStdCollectionAdapter() { mHandle.Finish(mSize, false); }
+
+  size_t size() const { return mSize; }
+  void resize(size_t aNewSize) {
+    EnsureCapacity(aNewSize);
+    mSize = aNewSize;
+    // XXX: technically resize should zero-initialize any new bytes to match
+    // std::string.
+  }
+
+  T& operator[](size_t i) {
+    MOZ_RELEASE_ASSERT(i < mSize);
+    return mHandle.Elements()[i];
+  }
+  const T& operator[](size_t i) const {
+    MOZ_RELEASE_ASSERT(i < mSize);
+    return mHandle.Elements()[i];
+  }
+
+ private:
+  void EnsureCapacity(size_t aNewCapacity) {
+    if (aNewCapacity > mHandle.Length()) {
+      auto result = mHandle.RestartBulkWrite(aNewCapacity, mSize, false);
+      if (result.isErr()) {
+        ::NS_ABORT_OOM(aNewCapacity * sizeof(value_type));
+      }
+    }
+  }
+  static mozilla::BulkWriteHandle<T> InfallibleBulkWrite(
+      nsTSubstring<T>& aString) {
+    size_t length = aString.Length();
+    auto res = aString.BulkWrite(length, length, false);
+    if (res.isErr()) {
+      ::NS_ABORT_OOM(length * sizeof(value_type));
+    }
+    return res.unwrap();
+  }
+
+  size_t mSize;
+  mozilla::BulkWriteHandle<T> mHandle;
+};
+
+// Mark the collection as contiguous for {fmt} so that the buffer will be used
+// directly.
+namespace fmt {
+template <typename T>
+struct is_contiguous<nsTSubstringStdCollectionAdapter<T>> : std::true_type {};
+}  // namespace fmt
+
+template <typename T>
+void nsTSubstring<T>::AppendVfmt(
+    fmt::basic_string_view<char_type> aFormatStr,
+    fmt::basic_format_args<fmt::buffered_context<char_type>> aArgs) {
+  nsTSubstringStdCollectionAdapter<char_type> adapter{*this};
+  fmt::vformat_to(std::back_inserter(adapter), aFormatStr, aArgs);
+}
+
 template <typename T>
 struct MOZ_STACK_CLASS PrintfAppend : public mozilla::PrintfTarget {
   explicit PrintfAppend(nsTSubstring<T>* aString) : mString(aString) {}
