@@ -7,11 +7,13 @@
 #include <functional>
 #include <queue>
 #include <string>
+#include <utility>
 
 #include "MainThreadUtils.h"
+#include "gtest/gtest.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/CondVar.h"
-#include "mozilla/Move.h"
+#include "mozilla/gtest/MozAssertions.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/ThrottledEventQueue.h"
@@ -19,15 +21,16 @@
 #include "nsError.h"
 #include "nsIRunnable.h"
 #include "nsISerialEventTarget.h"
+#include "nsIThread.h"
+#include "nsThreadUtils.h"
 #include "prinrval.h"
 
-#include "gtest/gtest.h"
-
 using mozilla::CondVar;
 using mozilla::MakeRefPtr;
 using mozilla::Mutex;
+using mozilla::MutexAutoLock;
+using mozilla::ThrottledEventQueue;
 using std::function;
-using std::move;
 using std::string;
 
 namespace TestThrottledEventQueue {
@@ -44,9 +47,9 @@ struct RunnableQueue : nsISerialEventTarget {
   bool IsEmpty() { return runnables.empty(); }
   size_t Length() { return runnables.size(); }
 
-  MOZ_MUST_USE nsresult Run() {
+  [[nodiscard]] nsresult Run() {
     while (!runnables.empty()) {
-      auto runnable = move(runnables.front());
+      auto runnable = std::move(runnables.front());
       runnables.pop();
       nsresult rv = runnable->Run();
       if (NS_FAILED(rv)) return rv;
@@ -57,15 +60,15 @@ struct RunnableQueue : nsISerialEventTarget {
 
   // nsIEventTarget methods
 
-  MOZ_MUST_USE NS_IMETHODIMP Dispatch(already_AddRefed<nsIRunnable> aRunnable,
-                                      uint32_t aFlags) override {
+  [[nodiscard]] NS_IMETHODIMP Dispatch(already_AddRefed<nsIRunnable> aRunnable,
+                                       uint32_t aFlags) override {
     MOZ_ALWAYS_TRUE(aFlags == nsIEventTarget::DISPATCH_NORMAL);
     runnables.push(aRunnable);
     return NS_OK;
   }
 
-  MOZ_MUST_USE NS_IMETHODIMP DispatchFromScript(nsIRunnable* aRunnable,
-                                                uint32_t aFlags) override {
+  [[nodiscard]] NS_IMETHODIMP DispatchFromScript(nsIRunnable* aRunnable,
+                                                 uint32_t aFlags) override {
     RefPtr<nsIRunnable> r = aRunnable;
     return Dispatch(r.forget(), aFlags);
   }
@@ -73,29 +76,37 @@ struct RunnableQueue : nsISerialEventTarget {
   NS_IMETHOD_(bool)
   IsOnCurrentThreadInfallible(void) override { return NS_IsMainThread(); }
 
-  MOZ_MUST_USE NS_IMETHOD IsOnCurrentThread(bool* retval) override {
+  [[nodiscard]] NS_IMETHOD IsOnCurrentThread(bool* retval) override {
     *retval = IsOnCurrentThreadInfallible();
     return NS_OK;
   }
 
-  MOZ_MUST_USE NS_IMETHODIMP DelayedDispatch(
+  [[nodiscard]] NS_IMETHODIMP DelayedDispatch(
       already_AddRefed<nsIRunnable> aEvent, uint32_t aDelay) override {
     return NS_ERROR_NOT_IMPLEMENTED;
   }
 
+  NS_IMETHOD RegisterShutdownTask(nsITargetShutdownTask*) override {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+
+  NS_IMETHOD UnregisterShutdownTask(nsITargetShutdownTask*) override {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+
   // nsISupports methods
 
   NS_DECL_THREADSAFE_ISUPPORTS
 
  private:
-  virtual ~RunnableQueue() {}
+  virtual ~RunnableQueue() = default;
 };
 
 NS_IMPL_ISUPPORTS(RunnableQueue, nsIEventTarget, nsISerialEventTarget)
 
 static void Enqueue(nsIEventTarget* target, function<void()>&& aCallable) {
-  nsresult rv =
-      target->Dispatch(NS_NewRunnableFunction("TEQ GTest", move(aCallable)));
+  nsresult rv = target->Dispatch(
+      NS_NewRunnableFunction("TEQ GTest", std::move(aCallable)));
   MOZ_ALWAYS_TRUE(NS_SUCCEEDED(rv));
 }
 
@@ -113,7 +124,7 @@ TEST(ThrottledEventQueue, RunnableQueue)
   Enqueue(queue, [&]() { log += 'c'; });
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(queue->Run()));
+  ASSERT_NS_SUCCEEDED(queue->Run());
   ASSERT_EQ(log, "abc");
 }
 
@@ -126,7 +137,7 @@ TEST(ThrottledEventQueue, SimpleDispatch)
       ThrottledEventQueue::Create(base, "test queue 1");
 
   Enqueue(throttled, [&]() { log += 'a'; });
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "a");
 
   ASSERT_TRUE(base->IsEmpty());
@@ -172,7 +183,7 @@ TEST(ThrottledEventQueue, MixedDispatch)
   // - the executor again, which runs the second event from the
   // ThrottledEventQueue.
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "acb");
 
   ASSERT_TRUE(base->IsEmpty());
@@ -198,7 +209,7 @@ TEST(ThrottledEventQueue, EnqueueFromRun)
   Enqueue(throttled, [&]() { log += 'd'; });
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "abdc");
 
   ASSERT_TRUE(base->IsEmpty());
@@ -218,14 +229,14 @@ TEST(ThrottledEventQueue, RunFromRun)
   Enqueue(throttled, [&]() {
     log += '(';
     // This should run subsequent events from throttled.
-    ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+    ASSERT_NS_SUCCEEDED(base->Run());
     log += ')';
   });
 
   Enqueue(throttled, [&]() { log += 'a'; });
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "(a)");
 
   ASSERT_TRUE(base->IsEmpty());
@@ -246,13 +257,13 @@ TEST(ThrottledEventQueue, DropWhileRunning)
   }
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "a");
 }
 
 TEST(ThrottledEventQueue, AwaitIdle)
 {
-  Mutex mutex("TEQ AwaitIdle");
+  Mutex mutex MOZ_UNANNOTATED("TEQ AwaitIdle");
   CondVar cond(mutex, "TEQ AwaitIdle");
 
   string dequeue_await;           // mutex
@@ -279,7 +290,7 @@ TEST(ThrottledEventQueue, AwaitIdle)
   nsCOMPtr<nsIThread> thread;
   nsresult rv =
       NS_NewNamedThread("TEQ AwaitIdle", getter_AddRefs(thread), await);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   // We can't guarantee that the thread has reached the AwaitIdle call, but we
   // can get pretty close. Either way, it shouldn't affect the behavior of the
@@ -294,7 +305,7 @@ TEST(ThrottledEventQueue, AwaitIdle)
     ASSERT_FALSE(threadFinished);
   }
   ASSERT_FALSE(runnableFinished);
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_TRUE(runnableFinished);
 
   // Wait for the thread to finish.
@@ -304,7 +315,7 @@ TEST(ThrottledEventQueue, AwaitIdle)
     ASSERT_EQ(dequeue_await, "dequeue await");
   }
 
-  ASSERT_TRUE(NS_SUCCEEDED(thread->Shutdown()));
+  ASSERT_NS_SUCCEEDED(thread->Shutdown());
 }
 
 TEST(ThrottledEventQueue, AwaitIdleMixed)
@@ -315,7 +326,7 @@ TEST(ThrottledEventQueue, AwaitIdleMixed)
   ASSERT_TRUE(NS_SUCCEEDED(
       NS_NewNamedThread("AwaitIdleMixed", getter_AddRefs(thread))));
 
-  Mutex mutex("AwaitIdleMixed");
+  Mutex mutex MOZ_UNANNOTATED("AwaitIdleMixed");
   CondVar cond(mutex, "AwaitIdleMixed");
 
   // The following are protected by mutex and cond, above.
@@ -366,7 +377,7 @@ TEST(ThrottledEventQueue, AwaitIdleMixed)
     ASSERT_EQ(log, "");
   }
 
-  ASSERT_TRUE(NS_SUCCEEDED(thread->Dispatch(await.forget())));
+  ASSERT_NS_SUCCEEDED(thread->Dispatch(await.forget()));
 
   // Wait for the thread to be ready to await. We can't be sure it will actually
   // be blocking before we get around to draining the event queue, but that's
@@ -379,7 +390,7 @@ TEST(ThrottledEventQueue, AwaitIdleMixed)
   }
 
   // Let the queue drain.
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
 
   {
     MutexAutoLock lock(mutex);
@@ -396,7 +407,7 @@ TEST(ThrottledEventQueue, AwaitIdleMixed)
     ASSERT_TRUE(log == "(a)b" || log == "(ab)");
   }
 
-  ASSERT_TRUE(NS_SUCCEEDED(thread->Shutdown()));
+  ASSERT_NS_SUCCEEDED(thread->Shutdown());
 }
 
 TEST(ThrottledEventQueue, SimplePauseResume)
@@ -412,23 +423,23 @@ TEST(ThrottledEventQueue, SimplePauseResume)
   Enqueue(throttled, [&]() { log += 'a'; });
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "a");
 
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(true)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(true));
   ASSERT_TRUE(throttled->IsPaused());
 
   Enqueue(throttled, [&]() { log += 'b'; });
 
   ASSERT_EQ(log, "a");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "a");
 
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
   ASSERT_FALSE(throttled->IsPaused());
 
   ASSERT_EQ(log, "a");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "ab");
 
   ASSERT_TRUE(base->IsEmpty());
@@ -454,7 +465,7 @@ TEST(ThrottledEventQueue, MixedPauseResume)
   Enqueue(base, [&]() { log += 'D'; });
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   // Since the 'b' event paused the throttled queue, 'c' should not have run.
   // but 'D' was enqueued directly on the base, and should have run.
   ASSERT_EQ(log, "AbD");
@@ -463,11 +474,11 @@ TEST(ThrottledEventQueue, MixedPauseResume)
   ASSERT_TRUE(throttled->IsPaused());
 
   Enqueue(base, [&]() { log += 'E'; });
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
   Enqueue(base, [&]() { log += 'F'; });
   ASSERT_FALSE(throttled->IsPaused());
 
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   // Since we've unpaused, 'c' should be able to run now. The executor should
   // have been enqueued between 'E' and 'F'.
   ASSERT_EQ(log, "AbDEcF");
@@ -478,7 +489,7 @@ TEST(ThrottledEventQueue, MixedPauseResume)
 
 TEST(ThrottledEventQueue, AwaitIdlePaused)
 {
-  Mutex mutex("AwaitIdlePaused");
+  Mutex mutex MOZ_UNANNOTATED("AwaitIdlePaused");
   CondVar cond(mutex, "AwaitIdlePaused");
 
   string dequeue_await;           // mutex
@@ -489,7 +500,7 @@ TEST(ThrottledEventQueue, AwaitIdlePaused)
   RefPtr<ThrottledEventQueue> throttled =
       ThrottledEventQueue::Create(base, "test queue 10");
 
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(true)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(true));
 
   // Put an event in the queue so the AwaitIdle might block. Since throttled is
   // paused, this should not enqueue an executor in the base target.
@@ -510,7 +521,7 @@ TEST(ThrottledEventQueue, AwaitIdlePaused)
   nsCOMPtr<nsIThread> thread;
   nsresult rv =
       NS_NewNamedThread("AwaitIdlePaused", getter_AddRefs(thread), await);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   // We can't guarantee that the thread has reached the AwaitIdle call, but we
   // can get pretty close. Either way, it shouldn't affect the behavior of the
@@ -528,14 +539,14 @@ TEST(ThrottledEventQueue, AwaitIdlePaused)
 
   // A paused TEQ contributes no events to its base target. (This is covered by
   // other tests...)
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_TRUE(base->IsEmpty());
   ASSERT_FALSE(throttled->IsEmpty());
 
   // Resume and drain the queue.
   ASSERT_FALSE(runnableFinished);
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_TRUE(base->IsEmpty());
   ASSERT_TRUE(throttled->IsEmpty());
   ASSERT_TRUE(runnableFinished);
@@ -547,7 +558,7 @@ TEST(ThrottledEventQueue, AwaitIdlePaused)
     ASSERT_EQ(dequeue_await, "dequeue await");
   }
 
-  ASSERT_TRUE(NS_SUCCEEDED(thread->Shutdown()));
+  ASSERT_NS_SUCCEEDED(thread->Shutdown());
 }
 
 TEST(ThrottledEventQueue, ExecutorTransitions)
@@ -558,7 +569,7 @@ TEST(ThrottledEventQueue, ExecutorTransitions)
   RefPtr<ThrottledEventQueue> throttled =
       ThrottledEventQueue::Create(base, "test queue 11");
 
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(true)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(true));
 
   // Since we're paused, queueing an event on throttled shouldn't queue the
   // executor on the base target.
@@ -568,35 +579,35 @@ TEST(ThrottledEventQueue, ExecutorTransitions)
 
   // Resuming throttled should create the executor, since throttled is not
   // empty.
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
   ASSERT_EQ(throttled->Length(), 1U);
   ASSERT_EQ(base->Length(), 1U);
 
   // Pausing can't remove the executor from the base target since we've already
   // queued it there, but it can ensure that it doesn't do anything.
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(true)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(true));
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "");
   ASSERT_EQ(throttled->Length(), 1U);
   ASSERT_EQ(base->Length(), 0U);
 
   // As before, resuming must create the executor, since throttled is not empty.
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
   ASSERT_EQ(throttled->Length(), 1U);
   ASSERT_EQ(base->Length(), 1U);
 
   ASSERT_EQ(log, "");
-  ASSERT_TRUE(NS_SUCCEEDED(base->Run()));
+  ASSERT_NS_SUCCEEDED(base->Run());
   ASSERT_EQ(log, "a");
   ASSERT_EQ(throttled->Length(), 0U);
   ASSERT_EQ(base->Length(), 0U);
 
   // Since throttled is empty, pausing and resuming now should not enqueue an
   // executor.
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(true)));
-  ASSERT_TRUE(NS_SUCCEEDED(throttled->SetIsPaused(false)));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(true));
+  ASSERT_NS_SUCCEEDED(throttled->SetIsPaused(false));
   ASSERT_EQ(throttled->Length(), 0U);
   ASSERT_EQ(base->Length(), 0U);
 }
