@@ -8,17 +8,15 @@
 #include "mozilla/Monitor.h"
 #include "mozilla/ReentrantMonitor.h"
 #include "mozilla/Services.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/StaticPtr.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
 #include "nsXPCOMCIDInternal.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIThreadManager.h"
 #include "nsThreadPool.h"
-#ifdef XP_WIN
-#  include "ThreadPoolCOMListener.h"
-#endif
 
 namespace mozilla {
 
@@ -27,8 +25,7 @@ static StaticAutoPtr<ReentrantMonitor> sMonitor;
 
 // Hashtable, maps thread pool name to SharedThreadPool instance.
 // Modified only on the main thread.
-static StaticAutoPtr<nsDataHashtable<nsCStringHashKey, SharedThreadPool*>>
-    sPools;
+static StaticAutoPtr<nsTHashMap<nsCStringHashKey, SharedThreadPool*>> sPools;
 
 static already_AddRefed<nsIThreadPool> CreateThreadPool(const nsCString& aName);
 
@@ -37,7 +34,7 @@ class SharedThreadPoolShutdownObserver : public nsIObserver {
   NS_DECL_ISUPPORTS
   NS_DECL_NSIOBSERVER
  protected:
-  virtual ~SharedThreadPoolShutdownObserver() {}
+  virtual ~SharedThreadPoolShutdownObserver() = default;
 };
 
 NS_IMPL_ISUPPORTS(SharedThreadPoolShutdownObserver, nsIObserver, nsISupports)
@@ -50,10 +47,10 @@ SharedThreadPoolShutdownObserver::Observe(nsISupports* aSubject,
 #ifdef EARLY_BETA_OR_EARLIER
   {
     ReentrantMonitorAutoEnter mon(*sMonitor);
-    if (!sPools->Iter().Done()) {
+    if (!sPools->IsEmpty()) {
       nsAutoCString str;
-      for (auto i = sPools->Iter(); !i.Done(); i.Next()) {
-        str.AppendPrintf("\"%s\" ", nsAutoCString(i.Key()).get());
+      for (const auto& key : sPools->Keys()) {
+        str.AppendPrintf("\"%s\" ", nsAutoCString(key).get());
       }
       printf_stderr(
           "SharedThreadPool in xpcom-shutdown-threads. Waiting for "
@@ -72,7 +69,7 @@ void SharedThreadPool::InitStatics() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(!sMonitor && !sPools);
   sMonitor = new ReentrantMonitor("SharedThreadPool");
-  sPools = new nsDataHashtable<nsCStringHashKey, SharedThreadPool*>();
+  sPools = new nsTHashMap<nsCStringHashKey, SharedThreadPool*>();
   nsCOMPtr<nsIObserverService> obsService =
       mozilla::services::GetObserverService();
   nsCOMPtr<nsIObserver> obs = new SharedThreadPoolShutdownObserver();
@@ -88,7 +85,7 @@ bool SharedThreadPool::IsEmpty() {
 /* static */
 void SharedThreadPool::SpinUntilEmpty() {
   MOZ_ASSERT(NS_IsMainThread());
-  SpinEventLoopUntil([]() -> bool {
+  SpinEventLoopUntil("SharedThreadPool::SpinUntilEmpty"_ns, []() -> bool {
     sMonitor->AssertNotCurrentThreadIn();
     return IsEmpty();
   });
@@ -99,42 +96,44 @@ already_AddRefed<SharedThreadPool> SharedThreadPool::Get(
   MOZ_ASSERT(sMonitor && sPools);
   ReentrantMonitorAutoEnter mon(*sMonitor);
   RefPtr<SharedThreadPool> pool;
-  nsresult rv;
 
-  if (auto entry = sPools->LookupForAdd(aName)) {
-    pool = entry.Data();
-    if (NS_FAILED(pool->EnsureThreadLimitIsAtLeast(aThreadLimit))) {
-      NS_WARNING("Failed to set limits on thread pool");
-    }
-  } else {
-    nsCOMPtr<nsIThreadPool> threadPool(CreateThreadPool(aName));
-    if (NS_WARN_IF(!threadPool)) {
-      sPools->Remove(aName);  // XXX entry.Remove()
-      return nullptr;
-    }
-    pool = new SharedThreadPool(aName, threadPool);
+  return sPools->WithEntryHandle(
+      aName, [&](auto&& entry) -> already_AddRefed<SharedThreadPool> {
+        if (entry) {
+          pool = entry.Data();
+          if (NS_FAILED(pool->EnsureThreadLimitIsAtLeast(aThreadLimit))) {
+            NS_WARNING("Failed to set limits on thread pool");
+          }
+        } else {
+          nsCOMPtr<nsIThreadPool> threadPool(CreateThreadPool(aName));
+          if (NS_WARN_IF(!threadPool)) {
+            sPools->Remove(aName);  // XXX entry.Remove()
+            return nullptr;
+          }
+          pool = new SharedThreadPool(aName, threadPool);
 
-    // Set the thread and idle limits. Note that we don't rely on the
-    // EnsureThreadLimitIsAtLeast() call below, as the default thread limit
-    // is 4, and if aThreadLimit is less than 4 we'll end up with a pool
-    // with 4 threads rather than what we expected; so we'll have unexpected
-    // behaviour.
-    rv = pool->SetThreadLimit(aThreadLimit);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      sPools->Remove(aName);  // XXX entry.Remove()
-      return nullptr;
-    }
+          // Set the thread and idle limits. Note that we don't rely on the
+          // EnsureThreadLimitIsAtLeast() call below, as the default thread
+          // limit is 4, and if aThreadLimit is less than 4 we'll end up with a
+          // pool with 4 threads rather than what we expected; so we'll have
+          // unexpected behaviour.
+          nsresult rv = pool->SetThreadLimit(aThreadLimit);
+          if (NS_WARN_IF(NS_FAILED(rv))) {
+            sPools->Remove(aName);  // XXX entry.Remove()
+            return nullptr;
+          }
 
-    rv = pool->SetIdleThreadLimit(aThreadLimit);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      sPools->Remove(aName);  // XXX entry.Remove()
-      return nullptr;
-    }
+          rv = pool->SetIdleThreadLimit(aThreadLimit);
+          if (NS_WARN_IF(NS_FAILED(rv))) {
+            sPools->Remove(aName);  // XXX entry.Remove()
+            return nullptr;
+          }
 
-    entry.OrInsert([pool]() { return pool.get(); });
-  }
+          entry.Insert(pool.get());
+        }
 
-  return pool.forget();
+        return pool.forget();
+      });
 }
 
 NS_IMETHODIMP_(MozExternalRefCountType) SharedThreadPool::AddRef(void) {
@@ -174,11 +173,9 @@ NS_IMETHODIMP_(MozExternalRefCountType) SharedThreadPool::Release(void) {
 NS_IMPL_QUERY_INTERFACE(SharedThreadPool, nsIThreadPool, nsIEventTarget)
 
 SharedThreadPool::SharedThreadPool(const nsCString& aName, nsIThreadPool* aPool)
-    : mName(aName), mPool(aPool), mRefCnt(0) {
-  mEventTarget = aPool;
-}
+    : mName(aName), mPool(aPool), mRefCnt(0) {}
 
-SharedThreadPool::~SharedThreadPool() {}
+SharedThreadPool::~SharedThreadPool() = default;
 
 nsresult SharedThreadPool::EnsureThreadLimitIsAtLeast(uint32_t aLimit) {
   // We limit the number of threads that we use. Note that we
@@ -218,13 +215,6 @@ static already_AddRefed<nsIThreadPool> CreateThreadPool(
   rv = pool->SetThreadStackSize(nsIThreadManager::kThreadPoolStackSize);
   NS_ENSURE_SUCCESS(rv, nullptr);
 
-#ifdef XP_WIN
-  // Ensure MSCOM is initialized on the thread pools threads.
-  nsCOMPtr<nsIThreadPoolListener> listener = new MSCOMInitThreadPoolListener();
-  rv = pool->SetListener(listener);
-  NS_ENSURE_SUCCESS(rv, nullptr);
-#endif
-
   return pool.forget();
 }
 
